package parMSpindle;
/*
 * RatesControl... vary biochemical reaction rates, concentrations, etc
*/

/*  <ParMSpindle - a simulation of ParM based plasmid segregation>
    Copyright (C) <2008>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;

import java.io.*;
import java.text.*;
import java.util.*;
import javax.media.j3d.*;

public class FilHistoControl extends JFrame implements ChangeListener, ActionListener, WindowListener, ComponentListener, KeyListener, MouseListener, MouseMotionListener, AdjustmentListener {
	static ParMSpindle_Graphics parentSim;
	static MenuBar parentMenu;
	static JPanel titlePanel,paramPanel,applyPanel;
	static BareButton cancelB,applyB;
	static ParamGui deltaTGui;
	static ParamGui [] params = new ParamGui[30];
	static int paramCt = 0;
	static int guiLineCt = 0;
	
	static final HistogramPlus filLengthHisto = new HistogramPlus(Env.binCt,Env.rStart,Env.rStop,Env.keepHeads,Env.keepTails,"","Filament Lengths (m)",null);
	static final HistoPlotPanel hPanel = new HistoPlotPanel(Env.histWidth,Env.histHeight);
	
	public FilHistoControl (ParMSpindle_Graphics parentSim) {
		this.parentSim = parentSim;
		
		makeRatesPanel();
		makeApplyPanel();
		
		if (!Env.remote) {
			addKeyListener(this);
			addComponentListener(this);
			addMouseListener(this);
			addMouseMotionListener(this);
			addWindowListener(this);
			
			this.setVisible(true);
			initialize();
			this.setTitle("Filament Histogram Control");
			Dimension prefDim = getPreferredSize();
			prefDim.height += (paramCt+1)*10;
			setSize(prefDim);
			this.setVisible(false);
			
		}
		
	}
	

	
	public void initialize () {
		getContentPane().removeAll();
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add (hPanel, BorderLayout.NORTH);
		getContentPane().add (paramPanel, BorderLayout.CENTER);
		getContentPane().add (applyPanel, BorderLayout.SOUTH);
		hPanel.initializeFromParams();
	
	}
	
	public void setInfoFields () {
		for (int i=0;i<paramCt;i++) {
			params[i].syncToParameter();
		}
	}
	
	public void updateInfoFields () {
		for (int i=0;i<paramCt;i++) {
			if (!params[i].isInEdit()) {
				params[i].syncToParameter();
			} else {
				editsOn();
			}
		}
	}
	
	public void refreshAll () {
		for (int i=0;i<paramCt;i++) { params[i].syncToParameter(); }
	}
	
	public void revertAll () {
		for (int i=0;i<paramCt;i++) { params[i].setToDefaultValue(); }
	}

	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
		
		if (act.equals("Apply")) {
			for (int i=0;i<paramCt;i++) { params[i].setValueFromGui(); }
			refreshAll();
			updateHistogramSettings();
			editsOff();
		}
		
		if (act.equals("Revert")) {
			revertAll();
			updateHistogramSettings();
			editsOff();
		}
		
		if (act.equals("Cancel")) {
			refreshAll();
			editsOff();
		}
		

	}
	
	public static void addValToFilLengthHisto (int monCt) {
		double filLength = (monCt+1)*Env.parMHalfMonoSize;
		synchronized (filLengthHisto.histLock) {
			if (Env.mapMonomerCtToHisto.isActive()) {
				filLengthHisto.addValue(filLength,monCt);
			} else {
				filLengthHisto.addValue(filLength);
			}
		}
	}
	
	public static void setFilLengthHisto () {
		synchronized (filLengthHisto.histLock) {
			filLengthHisto.clearBins();
			for (int i=0;i<FilSegment.filSegmentCt;i++) {
				if (FilSegment.theFilSegments[i].length > Env.minLengthToCount.getValue()) {
					if (Env.mapMonomerCtToHisto.isActive()) {
						filLengthHisto.addValue(FilSegment.theFilSegments[i].length,FilSegment.theFilSegments[i].monomerCt);
					} else {
						filLengthHisto.addValue(FilSegment.theFilSegments[i].length);
					}
				}
			}
			if (!Env.remote) {
				plotHist(); 
			}
		}
	}
	
	public static void plotHist () {
		hPanel.updateHist(filLengthHisto);
	}
	
	public void updateHistogramSettings () {
		if (!filLengthHisto.isCurrentWithParams()) {
			filLengthHisto.makeCurrentWithParams();
		}
		
		if (!hPanel.isCurrentWithParams()) {
			this.setVisible(true);
			hPanel.makeCurrentWithParams();
			this.setVisible(false);
		}
	}
	
	
	public void stateChanged(ChangeEvent e) {
	    JSlider source = (JSlider)e.getSource();
	}
	
	public void adjustmentValueChanged (AdjustmentEvent e) {
	}
	
	
	public void windowActivated (WindowEvent e) {}
	public void windowClosing (WindowEvent e) {}
	public void windowDeactivated (WindowEvent e) {}
	public void windowDeiconified (WindowEvent e) {}
	public void windowIconified (WindowEvent e) {}
	public void windowOpened (WindowEvent e) {}
	public void windowClosed (WindowEvent e) {}
	
	public void keyReleased (KeyEvent e) {}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
	}
	
	public void mouseClicked (MouseEvent e) {}
	public void mouseEntered (MouseEvent e) {}
	public void mouseExited (MouseEvent e) {}
	public void mousePressed (MouseEvent e) {
		int xPos = e.getX();
		int yPos = e.getY();
	}
	
	public void mouseReleased (MouseEvent e) {}
	public void mouseDragged (MouseEvent e) {}
	public void mouseMoved (MouseEvent e) {}
	
	public void componentHidden (ComponentEvent e) {}
	public void componentShown (ComponentEvent e) {}
	public void componentMoved (ComponentEvent e) {}
	public void componentResized (ComponentEvent e) {}
	
	public void makeTitlePanel () {
		titlePanel = new JPanel();
		titlePanel.setBackground(Env.controlBackColor);
		JLabel titleLabel = new JLabel("Rates");
		titleLabel.setForeground(Env.controlForeColor);
		titlePanel.add(titleLabel);
	}
	
	private void addParam (ParamGui newRate) {
		params[paramCt] = newRate;
		params[paramCt].addToPanel(paramPanel);
		paramCt++;
		guiLineCt++;
	}
	
	public void makeRatesPanel() {
		paramPanel = new JPanel();
		paramPanel.setBackground(Env.controlBackColor);
		paramPanel.setLayout(new GridLayout(Env.maxLayout,3));
		
		addParam(new ParamGui(Env.makeFilLengthHisto,ParamGui.CHECKBOX_ON,"Check to make filament length histogram"));
		addParam(new ParamGui(Env.mapMonomerCtToHisto,ParamGui.CHECKBOX_ON,"Map monomer count at each length, instead of filament count"));
		addParam(new ParamGui(Env.minLengthToCount,ParamGui.CHECKBOX_ON,"Filter out filaments below this length"));
		
		addBlankHead();
		
		addParam(new ParamGui(Env.binCt));
		addParam(new ParamGui(Env.rStart));
		addParam(new ParamGui(Env.rStop));
		
		addBlankHead();
		addInertGuiHead("   Scaling, too high or low values...","","");
		addParam(new ParamGui(Env.fixedHistScale,ParamGui.CHECKBOX_ON,"Manually set a fixed vertical scale, otherwise each frame is autoscaled to current max bin count"));
		addParam(new ParamGui(Env.keepHeads,ParamGui.CHECKBOX_ON,"Collect too low values into first bin"));
		addParam(new ParamGui(Env.keepTails,ParamGui.CHECKBOX_ON,"Collect too high values into last bin"));
		
		addBlankHead();
		addInertGuiHead("   Size and location...","","");
		
		addParam(new ParamGui(Env.histWidth));
		addParam(new ParamGui(Env.histHeight));
		
		addParam(new ParamGui(Env.histX));
		addParam(new ParamGui(Env.histY));
		
		addBlankHead();
		addInertGuiHead("   Other visuals","","");
		addParam(new ParamGui(Env.flatTops,ParamGui.CHECKBOX_ON,"Draw flat tops on each histogram bin"));
		addParam(new ParamGui(Env.showYHistTics,ParamGui.CHECKBOX_ON,"Show vertical scale"));
		addParam(new ParamGui(Env.showHistBox,ParamGui.CHECKBOX_ON,"Box around the histogram"));
	
		while (guiLineCt < Env.maxLayout) { addBlankHead(); }
		
	}
	
	public void addBlankHead() {
		InertGuiHead.addToPanel(paramPanel, new InertGuiHead("","",""));
		guiLineCt++;
	}
	
	public void addInertGuiHead(String s1, String s2, String s3) {
		InertGuiHead newHead = new InertGuiHead(s1,s2,s3);
		newHead.addToPanel(paramPanel);
		guiLineCt++;
	}
	
	public void addInertGuiHead(InertGuiHead newHead) {
		newHead.addToPanel(paramPanel);
		guiLineCt++;
	}
		
	
	public void makeApplyPanel () {
		applyPanel = new JPanel();
		applyPanel.setBackground(Env.controlBackColor);
	
		cancelB = new BareButton ("Cancel");
		cancelB.setActionCommand("Cancel");
		cancelB.setToolTipText("Cancel these changes");
		cancelB.addActionListener(this);
		cancelB.setEnabled(false);
		
		BareButton revertB = new BareButton ("Revert");
		revertB.setActionCommand("Revert");
		revertB.setToolTipText("Revert to default values");
		revertB.addActionListener(this);
		revertB.setEnabled(true);
		
		applyB = new BareButton("Apply");
		applyB.setActionCommand("Apply");
		applyB.setToolTipText("Apply Changes");
		applyB.addActionListener(this);
		applyB.setEnabled(false);
		
		applyPanel.setLayout(new FlowLayout(FlowLayout.CENTER,10,10));
		applyPanel.add(cancelB);
		applyPanel.add(revertB);
		applyPanel.add(applyB);
		
	}
	
	public void editsOn () {
		cancelB.setEnabled(true);
		applyB.setEnabled(true);
	}
	
	public void editsOff () {
		cancelB.setEnabled(false);
		applyB.setEnabled(false);
	}
		
}
