package parMSpindle;
/*
	FilLink .... the links between filaments
*/

/*  <ParMSpindle - a simulation of ParM based plasmid segregation>
    Copyright (C) <2008>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import javax.swing.*;
import java.lang.Math.*;



import com.sun.j3d.utils.geometry.Cylinder;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class FilLink {
	static FilLink [] filLinks = new FilLink[10000];
	static int filLinkCt = 0;
	static double restLength = 0;//Env.parMMonoSize;
	FilSegment fil1,fil2;
	double loc1,loc2;
	Pt3D pt1 = new Pt3D();
	Pt3D pt2 = new Pt3D();
	double linkLength;
	ValueTracker forceMag = new ValueTracker(Env.filLinkForcesToAve);
	ValueTracker torqueMag = new ValueTracker(Env.filLinkForcesToAve);
	ValueTracker strainTrack = new ValueTracker(Env.filLinkStrainToAve);
	double simTimeFormed;
	Pt3D linkVec = new Pt3D();
	Pt3D torsionVec = new Pt3D();
	Pt3D forceVec = new Pt3D();
	Pt3D R = new Pt3D();
	Pt3D RCrossF = new Pt3D();
	int filLinkNum;
	boolean removeMe = false;
	boolean orientSame = true;
	boolean fil1Relax = true;	// flag for which fil link moves on... set at init
	RetObj relaxO = new RetObj();	// used for relaxing link (point and line collision check)
	
	// for Java3D
	BranchGroup G = new BranchGroup();
	LineArray linkLine;
	Shape3D linkShape;
	boolean graphicsMade = false;
	
	public FilLink (Pt3D pt1, Pt3D pt2) {  // instantiate from QK file
		this.pt1.copy(pt1);
		this.pt2.copy(pt2);
		addFilLink(this);
	}
	
	public FilLink (FilSegment fil1, double loc1, FilSegment fil2, double loc2) {
		this.fil1 = fil1;
		this.fil2 = fil2;
		this.loc1 = loc1;
		this.loc2 = loc2;
		updatePts();
		simTimeFormed = Env.simulationTime;
		addFilLink(this);
		
		// store relaxation fil info
		if (Env.mtRNG.raw() < 0.5 ) { fil1Relax = false; }
		
		// store orientation.. fixed for life of FilLink
		double vecDot = Pt3D.Dot(fil1.uVec,fil2.uVec);
		double angTween = Math.acos(vecDot);
		//System.out.println ("initial ang tween is " + angTween);
		if (angTween > Math.PI/2) { orientSame = false; }
		
	}
	
	public FilLink (FilSegment fil1, double loc1, FilSegment fil2, double loc2,double angTween) {
		this.fil1 = fil1;
		this.fil2 = fil2;
		this.loc1 = loc1;
		this.loc2 = loc2;
		updatePts();
		simTimeFormed = Env.simulationTime;
		addFilLink(this);
		
		// store relaxation fil info
		if (Env.mtRNG.raw() < 0.5 ) { fil1Relax = false; }
		
		// store orientation.. fixed for life of FilLink
		if (angTween > Math.PI/2) { orientSame = false; }
		
	}
	
	public class RetObj {
		// this is the object passed by from line-line and line-point intersect tests
		Pt3D conPt1, conPt2, ray1, ray2, ray3, ray4;
		double conDist = 0;
		double alpha, beta;	 // the coefficients of ray1 and ray2, respectively, to define contact pts from end1s
		boolean collision = false;
		
		public RetObj () {
			conPt1 = new Pt3D();
			conPt2 = new Pt3D();
			ray1 = new Pt3D();
			ray2 = new Pt3D();
			ray3 = new Pt3D();
			ray4 = new Pt3D();
		}
		
		public void reset () {
			collision = false;
		}
	}
	
	public void sepaku () {
		fil1 = null;
		fil2 = null;
		pt1 = null;
		pt2 = null;
		forceMag = null;
		torqueMag = null;
		strainTrack = null;
		linkVec = null;
		torsionVec = null;
		G = null;
		linkLine = null;
		linkShape = null;
	}
	
	public void updatePts () {
		pt1.add(fil1.end1,loc1,fil1.uVec);
		pt2.add(fil2.end1,loc2,fil2.uVec);
		linkLength = Pt3D.ptDist(pt1, pt2);
		linkVec.sub(pt2,pt1);
	}
	
	public void updateFilLink () {
		// conditions for link dissolution
		if ((fil1 == null) | (fil1.removeMe)) { removeMe = true; return; }
		if ((fil2 == null) | (fil2.removeMe)) { removeMe = true; return; }
		
		// spontaneous dissolution (allow relaxation of stress and binding of other segments)
		if (Env.mtRNG.raw() < Env.linkDissolveRate.getValue()*Env.deltaT.getValue()) { removeMe = true; return; }
		
		// angle has grown too large
		if (angleTooLarge()) { removeMe = true; return; }
		
		// update loc1 and loc2 if minus end of filsegments has de/polymerized
		loc1 += fil1.minusEndDelta*Env.parMHalfMonoSize;
		loc2 += fil2.minusEndDelta*Env.parMHalfMonoSize;
		// more conditions for link dissolution
		if ((loc1 < 0) | (loc1 > fil1.length)) { removeMe = true; return; }
		if ((loc2 < 0) | (loc2 > fil2.length)) { removeMe = true; return; }
		
		updatePts();
	}
	
	public void applyForces() {
		applyTransForce();
		applyTorsionForce();
	}
	
	public void applyTransForce () {
		// strains
		double curStrainDist = linkLength-restLength;
		if (curStrainDist < 0) { curStrainDist = 0; }
		strainTrack.registerValue(curStrainDist);
		double aveStrainDist = strainTrack.averageVal();
		if (aveStrainDist > Env.maxFilLinkDist.getValue()) { 
			//System.out.println ("max strain exceeded");
			removeMe = true; return; 
		}
		
		//forces and accompanying torques
		double curForceMag = (linkLength-restLength)*Env.filLinkSpring.getValue();
		forceMag.registerValue(curForceMag);
		forceVec.scale(forceMag.averageVal(),linkVec);
		fil1.forceSum.inc(forceVec);
		R.sub(pt1,fil1.coord);
		R.scale(1e-6);
		RCrossF.cross(R,forceVec);
		fil1.torqueSum.inc(RCrossF);
		
		forceVec.reverse();
		fil2.forceSum.inc(forceVec);
		R.sub(pt2,fil2.coord);
		R.scale(1e-6);
		RCrossF.cross(R, forceVec);
		fil2.torqueSum.inc(RCrossF);
	}
	
	public void applyTorsionForce () {
		// rotational spring which works to align attached filament segments
		// need to know whether trying to align in same, or opposite, orientations
	
		double dotVecs;
		double angTween;
		if (orientSame) {
			torsionVec.cross(fil1.uVec,fil2.uVec);
			torsionVec.unitVec();
			dotVecs = Pt3D.Dot(fil1.uVec,fil2.uVec);
			angTween = Math.acos(dotVecs);
		} else {
			torsionVec.cross(fil1.uVec,fil2.uVecR);
			torsionVec.unitVec();
			dotVecs = Pt3D.Dot(fil1.uVec,fil2.uVec);
			angTween = Math.abs(Math.acos(dotVecs)-Math.PI);
		}
		
		if (dotVecs > 1.0) { dotVecs = 1.0; }
		
		//System.out.println ("OrientSame is " + orientSame + "  DotVecs is " + dotVecs + " and angTween is " + angTween);
		double curTorqueMag = Env.filLinkTorqSpring.getValue()*angTween;
		
		if (torsionVec.checkPt3D()) {
			torqueMag.registerValue(curTorqueMag);
			torsionVec.scale(torqueMag.averageVal());
			fil1.incTorqueSum(torsionVec);
			
			torsionVec.scale(-1);
			fil2.incTorqueSum(torsionVec);
		} else {
			System.out.println ("Crazy torque in FilLink.applyTorsionForce()");
		}

	}
	
	public void registerWithFilSegments() {
		fil1.registerFilLink(loc1,fil2.filID);
		fil2.registerFilLink(loc2,fil1.filID);
	}
	
	public boolean angleTooLarge () {
		double curAng;
		if (orientSame) { 
			curAng = Pt3D.Dot(fil1.uVec, fil2.uVec);
		} else {
			curAng = Pt3D.Dot(fil1.uVec, fil2.uVecR);
		}
		curAng = Math.acos(curAng);
		if (curAng > Math.PI/2) { return true; } 
		return false;
	}
		
	
	public boolean annealed() {
		if (fil1.filID == fil2.filID) { return true; } else { return false; }
	}
	
	/*public void checkForAnnealing () {
		if (Env.simulationTime - simTimeFormed > 1) { 
			FilSegment.annealSegments(fil1, fil1.end2, fil2, fil2.end1);
		}
	}*/

	
	public void checkForAnnealing () {
		double ptD, cosAngTween;
		if (fil1.filAtEnd1 & fil1.filAtEnd2) { return; } // fil1 is interior, can't anneal
		if (fil2.filAtEnd1 & fil2.filAtEnd2) { return; } // fil2 is interior, can't anneal
		
		// check a mess (4) of different possibilities
		if (!fil1.filAtEnd2 & !fil1.plasmidAtEnd2) {
			if (!fil2.filAtEnd1 & !fil2.plasmidAtEnd1) {
				ptD = Pt3D.ptDist(fil1.end2, fil2.end1);
				if (ptD < Env.annealDist.getValue()) { 
					//System.out.println("1st Ck passed: ptD = " + ptD);
					cosAngTween = Pt3D.Dot(fil1.uVec, fil2.uVec);
					if (cosAngTween > Env.annealAngleCosine.getValue()) {
						//System.out.println ("case1");
						FilSegment.annealSegments(fil1, fil1.end2, fil2, fil2.end1);
						//System.out.println ("Annealed!");
						this.removeMe = true;
						return;
					}
				}
			}
			
			if (!fil2.filAtEnd2 & !fil2.plasmidAtEnd2) {
				ptD = Pt3D.ptDist(fil1.end2,fil2.end2);
				if (ptD < Env.annealDist.getValue()) { 
					cosAngTween = Pt3D.Dot(fil1.uVec, fil2.uVecR);
					if (cosAngTween > Env.annealAngleCosine.getValue()) {
						//System.out.println ("case2");
						FilSegment.annealSegments(fil1, fil1.end2, fil2, fil2.end2);
						//System.out.println ("Annealed!");
						this.removeMe = true;
						return;
					}
				}
			}
		}
		
		if (!fil1.filAtEnd1 & !fil1.plasmidAtEnd1) {
			if (!fil2.filAtEnd1 & !fil2.plasmidAtEnd1) {
				ptD = Pt3D.ptDist(fil1.end1, fil2.end1);
				if (ptD < Env.annealDist.getValue()) { 
					cosAngTween = Pt3D.Dot(fil1.uVecR, fil2.uVec);
					if (cosAngTween > Env.annealAngleCosine.getValue()) {
						//System.out.println ("case3");
						FilSegment.annealSegments(fil1, fil1.end1, fil2, fil2.end1);
						//System.out.println ("Annealed!");
						this.removeMe = true;
						return;
					}
				}
			}
			
			if (!fil2.filAtEnd2 & !fil2.plasmidAtEnd2) {
				ptD = Pt3D.ptDist(fil1.end1, fil2.end2);
				if (ptD < Env.annealDist.getValue()) { 
					cosAngTween = Pt3D.Dot(fil1.uVecR, fil2.uVecR);
					if (cosAngTween > Env.annealAngleCosine.getValue()) {
						//System.out.println ("case4");
						FilSegment.annealSegments(fil1, fil1.end1, fil2, fil2.end2);
						//System.out.println ("Annealed!");
						this.removeMe = true;
						return;
					}
				}
			}
		}
	}
	
	public void relaxLink () {
		//if (Env.mtRNG.raw() < 0.5 ) {	// fixed point is on fil1, relax binding point on fil2
		if (!fil1Relax) {	// fixed point is on fil1, relax binding point on fil2
			pointAndLineIntersectTest(pt1,fil2.end1,fil2.end2,relaxO);
			if (relaxO.collision) { 	// the relaxation point is on the line segment
				pt2.copy(relaxO.conPt1);
				loc2 = Pt3D.ptDist(fil2.end1, pt2);
			} else {					// otherwise use extreme points
				if (relaxO.alpha <= 0) {
					pt2.copy(fil2.end1);
					loc2 = 0;
				}
				if (relaxO.alpha >= 1) {
					pt2.copy(fil2.end2);
					loc2 = fil2.length;
				}
			}
		} else { 	// fixed point is on fil2, relax binding point on fil1
			pointAndLineIntersectTest(pt2,fil1.end1,fil1.end2,relaxO);
			if (relaxO.collision) { 	// the relaxation point is on the line segment
				pt1.copy(relaxO.conPt1);
				loc1 = Pt3D.ptDist(fil1.end1, pt1);
			} else {					// otherwise use extreme points
				if (relaxO.alpha <= 0) {
					pt1.copy(fil1.end1);
					loc1 = 0;
				}
				if (relaxO.alpha >= 1) {
					pt1.copy(fil1.end2);
					loc1 = fil1.length;
				}
			}
		}
	}
			
	
	public static void enforceAllFilLinks () {
		// first set the linkCt to zero for each FilSegment
		FilSegment.zeroAllLinkCts();
		FilLink curFL;
		for (int i=0;i<filLinkCt;i++) {
			curFL = filLinks[i];
			if (!curFL.annealed()) {
				curFL.updateFilLink();
				if (!curFL.removeMe) { 
					//if (Env.filamentsAnneal.isActive()) { curFL.checkForAnnealing(); }
					if (Env.sideBondsRelax.isActive()) { curFL.relaxLink(); }
					curFL.applyForces();
					curFL.registerWithFilSegments();
				}
			} else {
				curFL.removeMe = true;
			}
		}
	}
	
	static void addFilLink (FilLink addMe) {
		filLinks[filLinkCt] = addMe;
		addMe.filLinkNum = filLinkCt;
		filLinkCt++;
	}
	
	public static void removeFilLink (FilLink rmMe) {
		if (rmMe.graphicsMade) { rmMe.G.detach(); }
		int swapId = rmMe.filLinkNum;
		filLinks[swapId] = filLinks[filLinkCt-1];
		filLinks[swapId].filLinkNum = swapId;
		rmMe.sepaku();
		filLinkCt--;
	}
	
	public static void removeDeadFilLinks () {
		for (int i=0;i<filLinkCt;i++) {
			if (filLinks[i] == null) { break; }		// this means we've gotten to the end of our shortening list of things
			if (filLinks[i].removeMe) {
				removeFilLink(filLinks[i]);
			}
		}
	}		
	
	public static void removeAll () {
		for (int i=0;i<filLinkCt;i++) {
			if (filLinks[i] != null) { 
				if (filLinks[i].graphicsMade) { filLinks[i].G.detach(); }
				filLinks[i].sepaku();
			}
		}
		filLinkCt = 0;
	}
		
	public void setPtsFromQKFile (Pt3D newPt1, Pt3D newPt2) {
		if (newPt1 != null & newPt2 != null) {
			pt1.copy(newPt1);
			pt2.copy(newPt2);
		} else {
			pt1.copy(Env.farfarAway);
			pt2.copy(Env.farfarAway);
		}
	}
	
	public static void pointAndLineIntersectTest (Pt3D point, Pt3D ptA, Pt3D ptB, RetObj retO) {
		// Point and Line Segment Intersection test... 
		// see derivation of the following formulae in work book... uses dot product as zero to enforce
		// the perpendicularity and parameterization of line segment to check if the perpendicular drop
		// from sphere to line is on the line segment.
		// A line segment is {x1,y1,z1,x2,y2,z2}
		// A point is {x,y,z}
		retO.reset();
		
		retO.ray1.sub(ptB,ptA);
		retO.ray2.sub(point,ptA);
		double numer = Pt3D.Dot(retO.ray2,retO.ray1);
		double denom = Pt3D.vecMagSqrd(retO.ray1);
		double alpha = numer/denom;
		if ((alpha <= 1) & (alpha >= 0)) {	// the perpendicular projection is on the line segment...  then check distance
			retO.collision = true;
			retO.conPt1.add(ptA, alpha,retO.ray1);		// define perpendicular point
			retO.conDist = Pt3D.ptDist (retO.conPt1,point);
		} else {
			retO.alpha = alpha;
		}	
	}
	
	public void makeGraphics () {
		// capabilities for graphics objects
		G.setCapability(BranchGroup.ALLOW_DETACH);
		
		Color3f linkColor = new Color3f(1.0f,1.0f,1.0f);
		ColoringAttributes cA = new ColoringAttributes(linkColor, ColoringAttributes.FASTEST);
		Appearance linkApp = new Appearance();
		linkApp.setColoringAttributes(cA);
		
		// line array
		linkLine = new LineArray(2,LineArray.COORDINATES);
		linkLine.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
		linkLine.setCoordinate(0,new Point3d(pt1.x,pt1.y,pt1.z));
		linkLine.setCoordinate(1,new Point3d(pt2.x,pt2.y,pt2.z));
		linkShape = new Shape3D();
		linkShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		linkShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		linkShape.setGeometry(linkLine);
		linkShape.setAppearance(linkApp);
		
		G.addChild(linkShape);
		graphicsMade = true;
	}
	
	public void updateGraphics () {
		//linkLine.setCoordinate(0,new Point3d(pt1.x,pt1.y,pt1.z));
		//linkLine.setCoordinate(1,new Point3d(pt2.x,pt2.y,pt2.z));
		linkLine.setCoordinate(0,pt1);
		linkLine.setCoordinate(1,pt2);
		linkShape.setGeometry(linkLine);
	}
	
	public Node getGraphicsNode () {
		if (!graphicsMade) { makeGraphics();}
		updateGraphics();
		return G;
	}
	
	
}


