package parMSpindle;
/*
	FilSegment .... the sections that form a filament
*/

/*  <ParMSpindle - a simulation of ParM based plasmid segregation>
    Copyright (C) <2008>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import javax.swing.*;
import java.lang.Math.*;
import com.sun.j3d.utils.geometry.Cylinder;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class FilSegment extends Thing {
	static FilSegment [] theFilSegments = new FilSegment [10000];	// array holding all filament segments
	static int filSegmentCt = 0;
	static int filamentRenderCt = 0;	// for rendering only
	static int filSegRenderCt = 0;		// for rendering only
	static double monosize=Env.parMMonoSize;
	static double halfmono=Env.parMHalfMonoSize;
	static double radius = Env.parMRadius;		// (nm) radius of parM filament
	static final Object filSync = new Object();	// for synchronizing filament creation
	
	// filSegments grouped by the filament they belong to... assign each a unique filament number
	static int filCt = 0; 		// tracks number of filaments
	static int filIDCt = 0;	// ever increasing id numbers for filaments
	int filID;			// all segments in the same filament share the same id number
	int filArrayPos;		// position in the static array of filsegments
	
	// statistics
	static HistogramPlus lifeTimes = new HistogramPlus(Env.lifeBins,Env.minLife,Env.maxLife,Env.keepHeads,Env.keepTails,"","Filament Lifetimes (s)",null);
	static HistogramPlus capStates = new HistogramPlus(Env.capStateBins,Env.minCapState,Env.maxCapState,Env.keepHeads,Env.keepTails,"","Filament Cap States (atp mons)",null);
	static HistogramPlus capStateLo = new HistogramPlus(Env.capStateBins,Env.minCapState,Env.maxCapState,Env.keepHeads,Env.keepTails,"","Filament Cap States (atp mons)",null);
	static HistogramPlus capStateHi = new HistogramPlus(Env.capStateBins,Env.minCapState,Env.maxCapState,Env.keepHeads,Env.keepTails,"","Filament Cap States (atp mons)",null);
	double bornTime;	// set when a filament is created... for tracking lifetimes
	
	// empirical fit for viscous drags
	static final double aParallel = -0.20;  // approx to constant in damping for parallel motion
	static final double aOrthog = 0.84;		// ...for orthogonal motion
	static final double aTurning = -0.662; 	// ...for rotational motion
	// if drags are for filaments near plane (Howard Chapt 6)
	static double hFromPlane = 0.5;	// in microns
	static double cParallel = 2*Math.PI*Env.aeta.getValue()/(Math.log(2*hFromPlane/radius)); // per unit length
	static double cPerp = 2*cParallel;
	static double cRot = cPerp/3;
	
	// y-positions for ordered filaments
	static final double incY = 1.5*Env.parMMonoSize;
	static double currentPosY = 0.5*incY;
	static double currentNegY = -0.5*incY;
	static boolean positiveY = true;
	static final double [] openY = new double [1000];
	static int openYCt = 0;
	
	Pt3D end1 = new Pt3D();
	Pt3D end2 = new Pt3D();
	// Pt3Ds for plasmid force calculations
	Pt3D F = new Pt3D();
	Pt3D Fopp = new Pt3D();
	Pt3D R = new Pt3D();
	Pt3D RcrossF = new Pt3D();
	Pt3D toPlasmidUVec = new Pt3D();
	Pt3D torsionVec = new Pt3D();
	
	// for collision detection
	double xRange,yRange,zRange;
	
	// cross-link locations and max
	static double minFilLinkSep = 2*Env.parMMonoSize;
	double [] linkLocs = new double [10*Env.maxLinksOnSeg.getIntValue()];
	int [] linkIds = new int [10*Env.maxLinksOnSeg.getIntValue()]; // store ids of other filsegment
	int linkCt = 0;
	
	// info about end states and segments
	boolean cap1Stable = true;
	boolean cap2Stable = true;
	boolean filAtEnd1 = false;
	boolean filAtEnd2 = false;
	FilSegment end1Fil = null;
	FilSegment end2Fil = null;
	Pt3D ptAtEnd1;
	Pt3D ptAtEnd2;
	boolean end1LinkCkd = false;
	boolean end2LinkCkd = false;
	boolean end1TorqCkd = false;
	boolean end2TorqCkd = false;
	
	int end2LinkThingNumber;		// used in reading QK state... myThingNumber of end2 filsegment if any
	int end1PlasmidThingNumber;		// ditto
	int end2PlasmidThingNumber;		// ditto
	
	Pt3D linkUVec = new Pt3D();		// a recycled Pt3D for segment link force calculations
	Pt3D linkUVecR = new Pt3D();		// reverse direction of above
	Pt3D linkPt = new Pt3D();			// link point location... recycled
	ValueTracker end1ToPlasStrain = new ValueTracker(Env.plasTetherStrainToAverage);
	ValueTracker end2ToPlasStrain = new ValueTracker(Env.plasTetherStrainToAverage);
	ValueTracker compressionTrack = new ValueTracker(Env.compressionStepsToTrack);
	ValueTracker end1SegAng = new ValueTracker(Env.segDistToTrack);
	ValueTracker end2SegAng = new ValueTracker(Env.segDistToTrack);
	ValueTracker end1SegDist = new ValueTracker(Env.segDistToTrack);
	ValueTracker end2SegDist = new ValueTracker(Env.segDistToTrack);
	
	int end1DetachCounter = 0;
	int end2DetachCounter = 0;
	int minusEndDelta = 0;		// keeps track of changes to minus end position
	
	Pt3D monInc=new Pt3D(),monOffset=new Pt3D(),evenStart=new Pt3D(),evenStop=new Pt3D(),oddStart=new Pt3D(),oddStop=new Pt3D();
	double end1AxialF = 0;
	double end2AxialF = 0;
	double end1TipC = 1e6; // large number for initial tip clearance of end1
	double end2TipC = 1e6; // large number for initial tip clearance of end2
	double fturn;
	double ftrans;
	double fnorm;
	Pt3D Fcoll = new Pt3D();	// recycled pt for force calculations
	Pt3D Tcoll = new Pt3D();	// recycled pt for torque calculations
	boolean lengthChanged = false;
	boolean plasmidAtEnd1 = false;
	boolean plasmidAtEnd2 = false;
	boolean globalPlasAtEnd1 = false;
	boolean globalPlasAtEnd2 = false;
	boolean end1Catastrophy = false;
	boolean end2Catastrophy = false;
	Plasmid end1Plasmid,end2Plasmid;
	Pt3D end1PAttachPt = new Pt3D(); Pt3D end2PAttachPt = new Pt3D();
	Pt3D end1PAttachPtInX = new Pt3D(); Pt3D end2PAttachPtInX = new Pt3D();
	Monomer minusMon, plusMon;
	Pt3D end1MonCenter = new Pt3D();
	Pt3D end2MonCenter = new Pt3D();
	Pt3D coordMonCenter = new Pt3D();
	Vector3d end1Vec3d = new Vector3d();
	Vector3d end2Vec3d = new Vector3d();
	Vector3d coordVec3d = new Vector3d();
	Pt3D curMonStart = new Pt3D();
	Pt3D curMonStop = new Pt3D();
	
	double length;
	double l;
	double helixAng = 2*Math.PI*Env.mtRNG.raw();	// keeps track of the helix angle of minusMon
	int monomerCt = 0;

	// for graphics
	TransformGroup cylTG,end1TG,end2TG,coordTG;
	Transform3D cylT3D,end1T3D,end2T3D,coordT3D;
	BranchGroup end1BG,end2BG,coordBG,coordSysG;
	Cylinder myCyl;
	Sphere end1S,end2S,coordS;
	static boolean graphicsInitialized = false;
	static float shiny = 128.0f;
	static Color3f ambientC = new Color3f(0.0f,1.0f,0.0f);
	static Color3f diffuseC = new Color3f(1.0f,0.0f,0.0f);
	static Color3f specularC = new Color3f(1.0f,1.0f,1.0f);
	static Color3f emissiveC = new Color3f(0.0f,0.0f,1.0f);
	static Color3f catasphColor = new Color3f(1.0f,0.0f,0.0f);
	static Color3f noPlasmidColor = new Color3f(0.0f,1.0f,1.0f);
	static Color3f gotPlasmidColor = new Color3f(0.0f,1.0f,1.0f);
	static Appearance catasphApp = new Appearance();
	static Appearance noPlasmidApp = new Appearance();
	static Appearance gotPlasmidApp = new Appearance();
	static ColoringAttributes catasphCA = new ColoringAttributes(catasphColor, ColoringAttributes.FASTEST);
	static ColoringAttributes noPlasmidCA = new ColoringAttributes(noPlasmidColor, ColoringAttributes.FASTEST);
	static ColoringAttributes gotPlasmidCA = new ColoringAttributes(gotPlasmidColor, ColoringAttributes.FASTEST);
	static Material catasphMat = new Material(catasphColor,catasphColor,catasphColor,catasphColor,shiny);
	static Material noPlasmidMat = new Material(noPlasmidColor,noPlasmidColor,noPlasmidColor,noPlasmidColor,shiny);
	static Material gotPlasmidMat = new Material(gotPlasmidColor,gotPlasmidColor,gotPlasmidColor,gotPlasmidColor,shiny);
	static TransparencyAttributes tA = new TransparencyAttributes (TransparencyAttributes.FASTEST,0.6f);
	// for coord sys drawing
	static Color3f xLineColor = new Color3f(1.0f,0.0f,0.0f);
	static Color3f yLineColor = new Color3f(0.0f,1.0f,0.0f);
	static Color3f zLineColor = new Color3f(0.0f,0.0f,1.0f);
	static Appearance xLineApp = new Appearance();
	static Appearance yLineApp = new Appearance();
	static Appearance zLineApp = new Appearance();
	static ColoringAttributes xLineCA = new ColoringAttributes(xLineColor, ColoringAttributes.FASTEST);
	static ColoringAttributes yLineCA = new ColoringAttributes(yLineColor, ColoringAttributes.FASTEST);
	static ColoringAttributes zLineCA = new ColoringAttributes(zLineColor, ColoringAttributes.FASTEST);
	double coordLineLength = 5*Env.parMMonoSize;
	LineArray xLine, yLine, zLine;
	Shape3D xLineShape, yLineShape, zLineShape;
	Pt3D xLineEndPt,yLineEndPt,zLineEndPt;
	boolean coordSysOn = false;
	boolean end1BGOn = false;
	boolean end2BGOn = false;
	
	public FilSegment (Pt3D initCoord, Pt3D initUVec, int filID) {
		super(initCoord);
		synchronized (filSync) {
			uVec.copy(initUVec);
			yVec = Pt3D.RandomUnitVec();
			monomerCt = Env.parMSeed.getIntValue();
			length = (monomerCt+1)*Env.parMHalfMonoSize;
			addFilSegment(this);
			setFilamentID(this, filID);
			calculateProperties();
			initialize();
			int nonHydros = makeInitialMonomers();
			theBug.takeParM(monomerCt-nonHydros);
			theBug.takeNonHydroParM(nonHydros);
		}
	}

	public FilSegment (Pt3D initCoord, Pt3D initUVec, int filID, int monomerCt, boolean fromFile) {
		super(initCoord);
		synchronized (filSync) {
			uVec.copy(initUVec);
			yVec = Pt3D.RandomUnitVec();
			this.monomerCt = monomerCt;
			length = (monomerCt+1)*Env.parMHalfMonoSize;
			addFilSegment(this);
			setFilamentID(this, filID);
			calculateProperties();
			initialize();
			int nonHydros = makeInitialMonomers();
			if (!fromFile) {
				theBug.takeParM(monomerCt-nonHydros);
				theBug.takeNonHydroParM(nonHydros);
			}
		}
	}
	
	public FilSegment (Pt3D initCoord, Pt3D initUVec, int monomerCt, FilSegment splitFromFil) {
		super(initCoord);
		synchronized (filSync) {
			uVec.copy(initUVec);
			yVec = Pt3D.RandomUnitVec();
			this.monomerCt = monomerCt;
			length = (monomerCt+1)*Env.parMHalfMonoSize;
			addFilSegment(this);
			setFilamentID(this, splitFromFil.filID);
			globalPlasAtEnd1 = splitFromFil.globalPlasAtEnd1;
			globalPlasAtEnd2 = splitFromFil.globalPlasAtEnd2;
			calculateProperties();
			initialize();
			setEnd1Links(splitFromFil, true);
			if (splitFromFil.filAtEnd2) {
				if (splitFromFil.ptAtEnd2 == splitFromFil.end2Fil.end1) { // same orientation
					setEnd2Links(splitFromFil.end2Fil, true);
					splitFromFil.end2Fil.setEnd1Links(this, true);
				} else {
					setEnd2Links(splitFromFil.end2Fil, false);
					splitFromFil.end2Fil.setEnd2Links(this, false);
				}
			}
			
			if (splitFromFil.plasmidAtEnd2) { 				// transfer plasmid link
				transferEnd2Plasmid(splitFromFil,this);
			}
		}
	}
	
	public void sepaku () {
		super.sepaku();
		end1 = null;
		end2 = null;
		F = null;
		Fopp = null;
		R = null;
		RcrossF = null;
		toPlasmidUVec = null;
		torsionVec = null;
		
		linkLocs = null;
		
		end1Fil = null;
		end2Fil = null;
		ptAtEnd1 = null;
		ptAtEnd2 = null;
		
		linkUVec = null;
		linkUVecR = null;
		linkPt = null;
		end1ToPlasStrain = null;
		end2ToPlasStrain = null;
		//filTorque1Track = null;
		//filTorque2Track = null;
		//filLink1Track = null;
		//filLink2Track = null;
		
		monInc=null;
		monOffset=null;
		evenStart=null;
		evenStop=null;
		oddStart=null;
		oddStop=null;
		
		Fcoll = null;
		Tcoll = null;
		
		end1Plasmid = null;
		end2Plasmid = null;
		end1PAttachPt = null; 
		end2PAttachPt = null;
		end1PAttachPtInX = null; 
		end2PAttachPtInX = null;
		minusMon = null;
		plusMon = null;
		end1MonCenter = null;
		end2MonCenter = null;
		coordMonCenter = null;
		end1Vec3d = null;
		end2Vec3d = null;
		coordVec3d = null;
		curMonStart = null;
		curMonStop = null;
		
		cylTG=null; end1TG=null; end2TG=null; coordTG=null;
		cylT3D=null; end1T3D=null; end2T3D=null; coordT3D=null;
		end1BG=null; end2BG=null; coordBG=null; coordSysG=null;
		myCyl=null;
		end1S=null; end2S=null; coordS=null;
	
		xLine=null; yLine=null; zLine=null;
		xLineShape=null; yLineShape=null; zLineShape=null;
		xLineEndPt = null;
		yLineEndPt = null;
		zLineEndPt = null;
	}
	
	/*public void calculateProperties () {
		// define the constants for motion of this rod in viscous medium
		// Remember that the dimensions we've been using are in micrometers so....
		// **WARNING** below a certain number of monomers, depending on values like aParallel, etc
		// the rod approximation will give NaN... hence the IF statement below
		int numberToSlow = 30;
		if (length >= numberToSlow*halfmono) {
			double lengthM = length*1.0e-6;  	// length in meters
			double radiusM = radius*1.0e-6;
			double denomLogTerm = Math.log(length/(2*radius));	//dimensionless
			bTransGam.x = (2*Math.PI*Env.aeta.getValue()*lengthM)/(denomLogTerm + aParallel);
			bTransGam.y = (4*Math.PI*Env.aeta.getValue()*lengthM)/(denomLogTerm + aOrthog);
			bTransGam.z = bTransGam.y;
			bRotGam.x = 4*Math.PI*Env.aeta.getValue()*radiusM*radiusM*lengthM;	// drag for turning about x
			bRotGam.y = (Math.PI*Env.aeta.getValue()*Math.pow(lengthM,3))/(3*(denomLogTerm + aTurning));
			bRotGam.z = bRotGam.y;
		} else {
			// use values for numberToSlow monomer filament
			double lengthM = numberToSlow*halfmono*1.0e-6/2;  	// length in meters
			double radiusM = radius*1.0e-6;
			double denomLogTerm = Math.log(numberToSlow*halfmono/(4*radius));	//dimensionless
			bTransGam.x = (2*Math.PI*Env.aeta.getValue()*lengthM)/(denomLogTerm + aParallel);
			bTransGam.y = (4*Math.PI*Env.aeta.getValue()*lengthM)/(denomLogTerm + aOrthog);
			bTransGam.z = bTransGam.y;
			bRotGam.x = 4*Math.PI*Env.aeta.getValue()*radiusM*radiusM*lengthM;	// drag for turning about x
			bRotGam.y = (Math.PI*Env.aeta.getValue()*Math.pow(lengthM,3))/(3*(denomLogTerm + aTurning));
			bRotGam.z = bRotGam.y;
		}
		
		bTransDiff.div(Env.Boltz*Env.tempK, bTransGam);	// Einstein's relation D=kT/gamma
		bRotDiff.div(Env.Boltz*Env.tempK, bRotGam);
		
		/*if (!bTransGam.checkPt3D()) { talkln ("bTransGam is crazy for FilSegment"); }
		if (!bRotGam.checkPt3D()) { talkln ("bRotGam is crazy for FilSegment"); }
		if (!bTransDiff.checkPt3D()) { talkln ("bTransDiff is crazy for FilSegment"); }
		if (!bRotDiff.checkPt3D()) { talkln ("bRotDiff is crazy for FilSegment"); }
		*/
	//}
	
	public void calculateProperties () {
		// define the constants for motion of this rod in viscous medium
		// Remember that the dimensions we've been using are in micrometers so....
		// **WARNING** below a certain number of monomers, depending on values like aParallel, etc
		// the rod approximation will give NaN... hence the IF statement below
		int minMonomerCt = 30;
		double minLength;
		if (filAtEnd1 | filAtEnd2) { 
			minLength = Env.stdSegLength.getIntValue()*halfmono; 
		} else {
			minLength = minMonomerCt*halfmono;
		}
		double asIfLength = length;	// default, use actual length
		if (asIfLength < minLength) { asIfLength = minLength; }
		
		double asIfLengthM = 1.0e-6*asIfLength; // in meters
		double radiusM = radius*1.0e-6;
		/*
		double denomLogTerm = Math.log(asIfLengthM/(2*radiusM));	//dimensionless
		bTransGam.x = (2*Math.PI*Env.aeta.getValue()*asIfLengthM)/(denomLogTerm + aParallel);
		bTransGam.y = (4*Math.PI*Env.aeta.getValue()*asIfLengthM)/(denomLogTerm + aOrthog);
		bTransGam.z = bTransGam.y;
		bRotGam.x = 4*Math.PI*Env.aeta.getValue()*radiusM*radiusM*asIfLengthM;	// drag for turning about x
		bRotGam.y = (Math.PI*Env.aeta.getValue()*Math.pow(asIfLengthM,3))/(3*(denomLogTerm + aTurning));
		bRotGam.z = bRotGam.y;
		*/
		
		bTransGam.x = cParallel*asIfLengthM;
		bTransGam.y = cPerp*asIfLengthM;
		bTransGam.z = bTransGam.y;
		bRotGam.x = 4*Math.PI*Env.aeta.getValue()*radiusM*radiusM*asIfLengthM;	// drag for turning about x
		bRotGam.y = cRot*Math.pow(asIfLengthM, 3)/4;
		bRotGam.z = bRotGam.y;
		
		bTransDiff.div(Env.Boltz*Env.tempK, bTransGam);	// Einstein's relation D=kT/gamma
		bRotDiff.div(Env.Boltz*Env.tempK, bRotGam);
		
		/*if (!bTransGam.checkPt3D()) { talkln ("bTransGam is crazy for FilSegment"); }
		if (!bRotGam.checkPt3D()) { talkln ("bRotGam is crazy for FilSegment"); }
		if (!bTransDiff.checkPt3D()) { talkln ("bTransDiff is crazy for FilSegment"); }
		if (!bRotDiff.checkPt3D()) { talkln ("bRotDiff is crazy for FilSegment"); }
		*/
	}
	
	public void initialize () {
		// this method assumes the unit x and y vectors have been set (though maybe not orthogonal), or are unchanged
		// determine z-unit vectors, then reset y-unit vector to ensure orthogonality with uVec
		zVec.cross(uVec, yVec);
		yVec.cross(zVec, uVec);
		// find the transformation matrices at this time step
		transMat ();
		// define opposite to uVec direction, used frequently
		uVecR.scale(-1,uVec);
		// length may have changed due to poly/depoly/split
		length=(monomerCt+1)*Env.parMHalfMonoSize;
		// reset coord value to keep centered
		if (Env.orderedCentered) { coord.x = 0; } 
		// re-define the end points of the rod to make sure they meet length criteria
		end1.add(coord, -length/2, uVec);
		end2.add(coord, length/2, uVec);
		
		// for collision detection
		xRange = Math.abs(coord.x-end2.x);
		yRange = Math.abs(coord.y-end2.y);
		zRange = Math.abs(coord.z-end2.z);
	}
	
	public int makeInitialMonomers() {
		double nonHydroProb = Env.parMConcNonHydro.getValue()/(Env.parMConc.getValue()+Env.parMConcNonHydro.getValue());
		int nonHydros = 0;
		boolean isNonHydro = Env.mtRNG.raw() < nonHydroProb;
		if (isNonHydro) { nonHydros ++; }
		Monomer.polymerize(null,this,Monomer.MINUSSEED, !isNonHydro);
		Monomer curMon = minusMon;
		for (int i=1;i<monomerCt;i++) {
			isNonHydro = Env.mtRNG.raw() < nonHydroProb;
			if (isNonHydro) { nonHydros ++; }
			Monomer.polymerize(curMon,this,Monomer.PLUSEND, !isNonHydro);
			curMon = curMon.frontMon;
		}
		//talkln("nonHydros = " + nonHydros + "/" + monomerCt);
		return nonHydros;
	}
	
	public void translate (double dist, Pt3D vec) {
		coord.inc(dist,vec);
		end1.inc(dist,vec);
		end2.inc(dist,vec);
	}
	
	public void translateCoord (double dist, Pt3D vec) {
		coord.inc(dist,vec);
	}
	
	public void step () {
		// increment counters that control how often different bits are run
		collCheckCt++;
		biochemCheckCt++;
		
		// resolve mechanical interaction and simulate biochemistry over the next time-step
		if (!Env.biochemOnly.isActive()) {
			//if (end1DetachCounter > 0) { end1DetachCounter --; }
			//if (end2DetachCounter > 0) { end2DetachCounter --; }
			
			if (collCheckCt >= collisionCheckInt | Env.simulationTime == 0) {
				end1TipC = end2TipC = 1e6;		// reset these to big numbers since we are about to check them again
				
				if (!Env.bugOff) { checkBugCollision(); }		// these should add forces and torques to forceSum and torqueSum
				
				if (!filAtEnd1 | !filAtEnd2) { checkPlasmidBinding(); }		// any segment with a free end might collide with a plasmid.... if so then bind
				
				//if (!plasmidAtEnd1 & !plasmidAtEnd2) { checkPlasmidCollision();	}	// every non-plasmid bound segment can collide and be repelled by plasmids
				if (!Env.cantileverElasticityTst) { checkPlasmidCollision(); }
				collCheckCt = 0;
				
			}
			
			addLinkForces();				// if linked to other segments
			
			addTorsionSpringForces();		// bending rigidity proxy
			
			addPlasmidForces();				// calculate elastic forces to keep filament ends and bound plasmids together
		
			setCompression();				// register compressive force in filament, if any
			
			moveThing();					// apply all collision and tethering forces to update this Things' coordinates
		}
		
		if (biochemCheckCt >= biochemCheckInt) {
			hydrolizeInFilaments();		// monomer-by-monomer hydrolysis and dissociate
			
			//define cap-states once here, since potentially used in several places
			cap1Stable = capConditionOKEnd1();
			cap2Stable = capConditionOKEnd2();
		
			if (!filAtEnd1) { end1BiochemSim (); }			// catastrophy, polymerization, and depolymerization simulations for end1
			if (!filAtEnd2) { end2BiochemSim (); }			// and for end2
		
			biochemCheckCt = 0;
		} 
		
		initialize();				// calculate transformation matrices, etc given the new coordinates
			
		if (!Env.biochemOnly.isActive()) {
			if (lengthChanged) {
				calculateProperties(); 	// calculate new drag coefficients, etc if length has changed
			}
			
			// relatively rare splits and joins... 
			if (monomerCt >= 2*Env.stdSegLength.getIntValue()) { 
				splitSegment();
				calculateProperties();	// again if split
				initialize();
			}	
			
			if (monomerCt <= Env.stdSegLength.getIntValue()/2) {
				joinSegments();
				calculateProperties();	// again if joined
				initialize();
			}
		}
		
		if (Env.fragRate.isActive()) { fragmentSegment(); }
		
		resetCounters();				// reset counters, zero force sums, etc in preparation for the next time step
	}
	
	public void moveThing () {
		// Given the forces/torques at this time point... move with explicit Euler approximation to ODE solution
			                   
		if (Env.cantileverElasticityTst & !filAtEnd2 & Env.simulationTime > 10*Env.deltaT.getValue()) { addCantileverTestForce(this); }	// comment out perhaps when running sims
		if (Env.simpleSupportElasticityTst) {
			if (Env.secondMode) {
				if (filArrayPos == Env.forceSeg1 & Env.simulationTime > 10*Env.deltaT.getValue()) { addSimpleSupportTestForce2ndMode(this); }
				if (filArrayPos == Env.forceSeg2 & Env.simulationTime > 10*Env.deltaT.getValue()) { addSimpleSupportTestForce2ndMode(this); }
			} else {
				if (filArrayPos == Env.forceSeg1 & Env.simulationTime > 10*Env.deltaT.getValue()) { addSimpleSupportTestForce1stMode(this); }
			}
		}
		
		// first check that forceSum and torqueSum aren't wacky... exit method if they are
		if (!forceSum.checkPt3D()) { 
			talkln ("Crazy forceSum in FilSegment"); 
			forceSum.zero();
			Env.brownianFilMotionOff = false;
		}
		if (!torqueSum.checkPt3D()) { 
			talkln ("Crazy torqueSum in FilSegment"); 
			torqueSum.zero();
			Env.brownianFilMotionOff = false;
		}
		
		// Work in coordinates aligned with the rod... transform forces and torques into body-fixed axis....
		bForceSum.XTox(this, forceSum);
		bTorqueSum.XTox(this, torqueSum);

		// add these instantaneous body-fixed forces to the tracked list, then reset them as average value
		bForceTrack.registerValue(bForceSum);
		bTorqueTrack.registerValue(bTorqueSum);
		bForceSum.copy(bForceTrack.averagePtVal());
		bTorqueSum.copy(bTorqueTrack.averagePtVal());
		
		// add brownian force and torque... these are zero except at every chosen time-step
		if (!Env.brownianFilMotionOff) {
			// trans
			double transScale = Env.BTransCoeff.getValue();
			if (!filAtEnd1 & !filAtEnd2) { transScale = 1.0; }	// for unattached segment use rigid rod value
			if (linkCt > 0) { transScale = transScale/(1 + Env.xLinkTransAttn.getValue()*linkCt); }
			if (plasmidAtEnd1 | plasmidAtEnd2) { transScale = Env.plasmidBoundTransScale; }
			bForceSum.inc(transScale,randForces);
			
			// rot
			double rotScale = Env.BRotCoeff.getValue();
			if (!filAtEnd1 & !filAtEnd2) { rotScale = 1.0; }	// for unattached segment use rigid rod value
			if (linkCt > 0) { rotScale = rotScale/(1+ Env.xLinkRotAttn.getValue()*linkCt); }
			if ((!filAtEnd1 & !plasmidAtEnd1) | (!filAtEnd2 & !plasmidAtEnd2)) {			// only apply brownian torques to end filaments.. best matches expected angular correlations
				bTorqueSum.inc(rotScale,randTorques);
			}
		}
		// now that the forces and torques are in the body fixed frame, we apply the eoms....
		bVeloc.div(1.0e6, bForceSum, bTransGam);		// in micron/sec
		bAngVeloc.div(bTorqueSum, bRotGam);			// in radians/sec
		
		// ** before progressing .... check that bVeloc and bAngVeloc are not NaN... exit if wacky
		if (!bVeloc.checkPt3D()) { talkln ("** problem with bVeloc for filament " + String.valueOf(this)); return; }
		if (!bAngVeloc.checkPt3D()) { talkln ("** problem with bAngVeloc for filament " + String.valueOf(this)); return; }
		
		// New Positions
		// the body-fixed angular velocities can just be transformed into fixed-frame velocities, and the coord updated
		veloc.xToX(this, bVeloc);
		coord.inc(Env.deltaT.getValue(),veloc);  // just position = velocity*time
		
		//deltaBAng.inc(Env.deltaT.getValue(),bAngVeloc);
		// to apply the body-fixed angular velocities, approximate new unit vector from arc of rotations.. good for small rotations
		// for uVec
		double uVecTransInZ = -bAngVeloc.y * Env.deltaT.getValue();	// arclength out at 1 micron
		double uVecTransInY = bAngVeloc.z * Env.deltaT.getValue();
		uVec.setVals(1,uVecTransInY,uVecTransInZ);	// in body-fixed, not a unit vector yet
		uVec.xToX(this);	// make in fixed-frame, not a unit vector yet
		uVec.unitVec();		// make a unit vector
		
		// for yVec 
		double yVecTransInX = - uVecTransInY;
		double yVecTransInZ = bAngVeloc.x * Env.deltaT.getValue();	// arclength at 1 micron
		yVec.setVals(yVecTransInX, 1, yVecTransInZ);
		yVec.xToX(this);
		yVec.unitVec();
		
		initialize();
		
	}
	
	public void joinSegments () {
		try {
			// determine which case and call appropriate joining method
			if (!filAtEnd1 & !filAtEnd2) { return; }  	// no one to join to
			
			FilSegment joinTo = null;
			boolean chooseEnd1Fil = filAtEnd1;
			if (filAtEnd1 & filAtEnd2) { 	// choose end2Fil if shorter...
				if (end2Fil.monomerCt < end1Fil.monomerCt) { chooseEnd1Fil = false; }
			}
			if (chooseEnd1Fil) {
				joinTo = end1Fil;
				if (ptAtEnd1 == end1Fil.end2) { 	// normal alignment
					joinSegs21(end1Fil,this);
					
				} else {
					joinSegs11(end1Fil,this);
				}
			} else {	// must be filAtEnd2
				joinTo = end2Fil;
				if (ptAtEnd2 == end2Fil.end1) { 	// normal alignment
					joinSegs12(end2Fil,this);
				} else {
					joinSegs22(end2Fil,this);
				}
			}
			if (joinTo != null) {
				joinTo.initialize();
				joinTo.calculateProperties();
			}
		} catch (NullPointerException npe) { System.out.println ("caught null pointer in joinSegments"); }
	}
	
	public static void joinSegs11 (FilSegment stayFil, FilSegment byeFil) {
		// case11:  stayFil.end1 linked to byeFil end1
		//talkln ("case11: joining filseg with " + byeFil.monomerCt + " to filseg with " + stayFil.monomerCt);
		// will take what's left of byeFil and add it to stayFil
		// transfer monomers
		byeFil.switchMonomerLinkDirection();	// affects minusMon and plusMon pointers
		stayFil.minusMon.backMon = byeFil.plusMon;	
		byeFil.plusMon.frontMon = stayFil.minusMon;
		stayFil.minusMon = byeFil.minusMon;	
		stayFil.minusMon.backMon = Monomer.minusGhost;
		// increment monomerCt of stayFil, and shift CM
		stayFil.monomerCt += byeFil.monomerCt;
		stayFil.coord.inc(byeFil.monomerCt*halfmono/2,stayFil.uVecR);
		// reassign graphics, if rendering
		Monomer curMon = byeFil.plusMon;
		if (!Env.remote) {
			while (curMon != Monomer.minusGhost) {
				curMon.reassignGraphics(byeFil, stayFil);
				curMon = curMon.backMon;
			}
		}
		// get rid of byeFil, reassign links if any
		byeFil.monomerCt = 0;
		byeFil.minusMon = Monomer.minusGhost; // remove pointers to real monomers from byeFil
		byeFil.plusMon = Monomer.plusGhost;
		cleanup(byeFil, true);
	}
	
	public static void joinSegs12 (FilSegment stayFil, FilSegment byeFil) {
		// case12:  usual orientation in reverse, stayFil.end2 linked to byeFil end1
		//talkln ("case12: joining filseg with " + byeFil.monomerCt + " to filseg with " + stayFil.monomerCt);
		// will take what's left of byeFil and add it to stayFil
		// transfer monomers
		stayFil.minusMon.backMon = byeFil.plusMon;	// link plusmon to byeFil.minusmon
		byeFil.plusMon.frontMon = stayFil.minusMon;
		stayFil.minusMon = byeFil.minusMon;	// declare that byeFils minusMon is new minusMon for stayfil
		stayFil.minusMon.backMon = Monomer.minusGhost;
		// increment monomerCt of stayFil, and shift CM
		stayFil.monomerCt += byeFil.monomerCt;
		stayFil.coord.inc(byeFil.monomerCt*halfmono/2,stayFil.uVecR);
		// reassign graphics, if rendering
		Monomer curMon = byeFil.plusMon;
		if (!Env.remote) {
			while (curMon != Monomer.minusGhost) {
				curMon.reassignGraphics(byeFil, stayFil);
				curMon = curMon.backMon;
			}
		}
		// get rid of byeFil, reassign links if any
		byeFil.monomerCt = 0;
		byeFil.minusMon = Monomer.minusGhost; // remove pointers to real monomers from byeFil
		byeFil.plusMon = Monomer.plusGhost;
		cleanup(byeFil, true);
	}
	
	public static void joinSegs21 (FilSegment stayFil, FilSegment byeFil) {
		// case21:  usual orientation, stayFil.end2 linked to byeFil end1
		//talkln ("case21: joining filseg with " + byeFil.monomerCt + " to filseg with " + stayFil.monomerCt);
		// will take what's left of byeFil and add it to stayFil
		// transfer monomers
		stayFil.plusMon.frontMon = byeFil.minusMon;	// link plusmon to byeFil.minusmon
		byeFil.minusMon.backMon = stayFil.plusMon;
		stayFil.plusMon = byeFil.plusMon;	// declare that byeFils plus mon is new plusmon for stayfil
		stayFil.plusMon.frontMon = Monomer.plusGhost;
		// increment monomerCt of stayFil, and shift CM
		stayFil.monomerCt += byeFil.monomerCt;
		stayFil.coord.inc(byeFil.monomerCt*halfmono/2,stayFil.uVec);
		// reassign graphics, if rendering
		Monomer curMon = byeFil.minusMon;
		if (!Env.remote) {
			while (curMon != Monomer.plusGhost) {
				curMon.reassignGraphics(byeFil, stayFil);
				curMon = curMon.frontMon;
			}
		}
		// get rid of byeFil, reassign links if any
		byeFil.monomerCt = 0;
		byeFil.minusMon = Monomer.minusGhost; // remove pointers to real monomers from byeFil
		byeFil.plusMon = Monomer.plusGhost;
		cleanup(byeFil, true);
	}
	
	public static void joinSegs22 (FilSegment stayFil, FilSegment byeFil) {
		// case22:  stayFil.end2 linked to byeFil end2
		//talkln ("case22: joining filseg with " + byeFil.monomerCt + " to filseg with " + stayFil.monomerCt);
		// will take what's left of byeFil and add it to stayFil
		// transfer monomers
		byeFil.switchMonomerLinkDirection();	// plusMon and minusMon pointers swapped as well
		stayFil.plusMon.frontMon = byeFil.minusMon;	
		byeFil.minusMon.backMon = stayFil.plusMon;
		stayFil.plusMon = byeFil.plusMon;	// declare that byeFils plus mon is new plusmon for stayfil
		stayFil.plusMon.frontMon = Monomer.plusGhost;
		// increment monomerCt of stayFil, and shift CM
		stayFil.monomerCt += byeFil.monomerCt;
		stayFil.coord.inc(byeFil.monomerCt*halfmono/2,stayFil.uVec);
		// reassign graphics, if rendering
		Monomer curMon = byeFil.minusMon;
		if (!Env.remote) {
			while (curMon != Monomer.plusGhost) {
				curMon.reassignGraphics(byeFil, stayFil);
				curMon = curMon.frontMon;
			}
		}
		// get rid of byeFil, reassign links if any
		byeFil.monomerCt = 0;
		byeFil.minusMon = Monomer.minusGhost; // remove pointers to real monomers from byeFil
		byeFil.plusMon = Monomer.plusGhost;
		cleanup(byeFil, true);
	}
	
	public void switchMonomerLinkDirection () { 
		Monomer [] mons = new Monomer [monomerCt];
		int monCt = 0;
		Monomer curMon = minusMon;
		while (curMon != Monomer.plusGhost) {
			mons[monCt] = curMon;
			monCt++;
			curMon = curMon.frontMon;
		}
		for (int i=0;i<monCt-1;i++) {
			mons[i].backMon = mons[i+1];
			mons[i+1].frontMon = mons[i];
		}
		//reverse plusMon and minusMon pointers
		plusMon = mons[0];
		minusMon = mons[monCt-1];
	}
	
	public void splitSegment () {
		int halfSegCt = Env.stdSegLength.getIntValue();
		monomerCt = monomerCt - halfSegCt;
		length = (monomerCt+1)*Env.parMHalfMonoSize;
		coord.add(end1,0.5*length,uVec);
		end2.add(end1, length, uVec);
		double nextFilLength = (halfSegCt+1)*Env.parMHalfMonoSize;
		Pt3D nextFilCoord = Pt3D.Add(end2,0.5*nextFilLength-Env.parMHalfMonoSize,uVec);
		FilSegment nextFil = new FilSegment (nextFilCoord,this.uVec,halfSegCt,this);
		setEnd2Links(nextFil, true);
		transferMons (monomerCt,nextFil);
	}
	
	public void fragmentSegment () {
		if (monomerCt < 2*Env.parMSeed.getIntValue()) { return; }
		double fragProb = Env.fragRate.getValue()*length*Env.deltaT.getValue();
		if (Env.rdm()<fragProb) {
			int fragMonCt = (int)((monomerCt-2*Env.parMSeed.getValue())*Env.rdm()+Env.parMSeed.getValue()); // ensure that split leaves at least seed # of mons 
			monomerCt = monomerCt - fragMonCt;
			length = (monomerCt+1)*Env.parMHalfMonoSize;
			coord.add(end1,0.5*length,uVec);
			end2.add(end1, length, uVec);
			Pt3D orderedLoc = new Pt3D(0,getOrderedY(),0);
			FilSegment nextFil = new FilSegment (orderedLoc,this.uVec,fragMonCt,this);
			setEnd2Links(nextFil, true);
			transferMons (monomerCt,nextFil);
			//talkln ("frag'd seg into " + monomerCt + " and " + fragMonCt + " monomers");
		}
	}
	
	public void transferMons (int startMonLoc, FilSegment targetFil) {
		// get handle to starting mon
		Monomer startMon = minusMon;
		
		for (int i=0;i<startMonLoc;i++) {
			startMon = startMon.frontMon;
		}
		targetFil.plusMon = plusMon;						// take over this filaments current plusMon
		targetFil.plusMon.frontMon = Monomer.plusGhost;
		plusMon = startMon.backMon;
		plusMon.frontMon = Monomer.plusGhost;
		targetFil.minusMon = startMon;
		targetFil.minusMon.backMon = Monomer.minusGhost;
		
		Monomer curMon = startMon;
		if (!Env.remote) {
			while (curMon != Monomer.plusGhost) {
				curMon.reassignGraphics(this, targetFil);
				curMon = curMon.frontMon;
			}
		}
	}
	
	public void transferAllMons (FilSegment targetFil) {
		//talkln ("transfering all mons:  " + this.monomerCt + " to fil with " + targetFil.monomerCt);
		int monsToTransfer = monomerCt;
		for (int i=0;i<monsToTransfer;i++) {
			Monomer.polymerize(targetFil.plusMon,targetFil,Monomer.PLUSEND, true);
			targetFil.plusMon.setState(minusMon.getState());
			targetFil.monomerCt++;
			targetFil.length+=halfmono;
			
			minusMon.depolymerize(this,Monomer.MINUSEND);
			monomerCt--;
			length+=-halfmono;
		}
		//talkln ("cleaning up filament after mon transfer.. monomerCt = " + monomerCt);
		cleanup(this,true);
	}
	
	/*public void transferAllMons (FilSegment targetFil) {
		if (minusMon != null) {
		try {
		targetFil.plusMon.frontMon = minusMon;
		minusMon.backMon = targetFil.plusMon;
		targetFil.plusMon = plusMon;
		} catch (NullPointerException npe) { 
			System.out.println (minusMon + ";" + targetFil.plusMon); 
			System.out.println ("numMons =  " + monomerCt);
		}
	
		Monomer curMon = minusMon;
		if (!Env.remote) {
			while (curMon != Monomer.plusGhost) {
				curMon.reassignGraphics(this, targetFil);
				curMon = curMon.frontMon;
			}
		}
		targetFil.monomerCt += monomerCt;
		removeMe = true;
		monomerCt = 0;
		}
	}*/
	
	public static void annealBiochemOnlySegments (FilSegment seg1, FilSegment seg2) {
		// randomize annealing of ends
		if (Env.mtRNG.raw() < 0.5) {
			seg1.transferAllMons(seg2);
		} else {
			seg2.transferAllMons(seg1);
		}
			
	}
	
	public static void annealSegments (FilSegment seg1, Pt3D pt1, FilSegment seg2, Pt3D pt2) {
		//talkln ("In annealSegments");
		// four cases
		if (pt1.equals(seg1.end2) & pt2.equals(seg2.end1)) {
			seg1.setEnd2Links(seg2,true);
			seg2.setEnd1Links(seg1,true);
			if (seg1.filID < seg2.filID) { seg2.filID = seg1.filID; } else { seg1.filID = seg2.filID; }
			return;
		}
		
		if (pt1.equals(seg1.end1) & pt2.equals(seg2.end2)) {
			seg1.setEnd1Links(seg2,true);
			seg2.setEnd2Links(seg1,true);
			if (seg1.filID < seg2.filID) { seg2.filID = seg1.filID; } else { seg1.filID = seg2.filID; }
			return;
		}
		
		if (pt1.equals(seg1.end2) & pt2.equals(seg2.end2)) {
			seg1.setEnd2Links(seg2,false);
			seg2.setEnd2Links(seg1,false);
			if (seg1.filID < seg2.filID) { seg2.filID = seg1.filID; } else { seg1.filID = seg2.filID; }
			return;
		}
		
		if (pt1.equals(seg1.end1) & pt2.equals(seg2.end1)) {
			seg1.setEnd1Links(seg2,false);
			seg2.setEnd1Links(seg1,false);
			if (seg1.filID < seg2.filID) { seg2.filID = seg1.filID; } else { seg1.filID = seg2.filID; }
			return;
		}
			
	}
	
	public double getPolyRateEnd1 () {
		double rate;
		if (plasmidAtEnd1) { 
			rate = Env.kATPOn1WithPlas.getValue(); 
		} else { rate = Env.kATPOn1.getValue(); }
		return rate;
	}
	
	public double getNonHydroPolyRateEnd1 () {
		double rate;
		if (plasmidAtEnd1) { 
			rate = Env.kATPOn1NonHydroWithPlas.getValue(); 
		} else { rate = Env.kATPOn1NonHydro.getValue(); }
		return rate;
	}
	
	public double getPolyRateEnd2 () {
		double rate;
		if (plasmidAtEnd2) { 
			rate = Env.kATPOn2WithPlas.getValue(); 
		} else { rate = Env.kATPOn2.getValue(); }
		return rate;
	}
	
	public double getNonHydroPolyRateEnd2 () {
		double rate;
		if (plasmidAtEnd2) { 
			rate = Env.kATPOn2NonHydroWithPlas.getValue(); 
		} else { rate = Env.kATPOn2NonHydro.getValue(); }
		return rate;
	}
	
	public double getDepolyRateEnd1 () {
		if (minusMon.hydrolyzable) { 
			if (plasmidAtEnd1) { 
				if (minusMon.isADP()) { return Env.kADPOff1WithPlas.getValue(); } else { return Env.kATPOff1WithPlas.getValue(); }
			} else if ((linkCt > 0) & (Env.sideBondsStabilize.isActive())) {
				if (minusMon.isADP()) { return Env.sideBondsStabilize.getValue(); } else { return Env.kATPOff1WithPlas.getValue(); }
			} else {
				if (Env.noCapThenADPdepoly.isActive() & !cap1Stable) { return Env.kADPOff1.getValue(); }  // use ADP off-rate no matter what if no stabilizing cap
				if (minusMon.isADP()) { return Env.kADPOff1.getValue(); } else { return Env.kATPOff1.getValue(); }
			}
		} else {
			if (plasmidAtEnd1) { return Env.kATPOff1NonHydroWithPlas.getValue(); } 
			if (Env.noCapThenADPdepoly.isActive() & !cap1Stable) { return Env.kADPOff1.getValue(); }  // use ADP off-rate no matter what if no stabilizing cap
			return Env.kATPOff1NonHydro.getValue(); 
		}
	}
	
	public double getDepolyRateEnd2 () {
		if (plusMon.hydrolyzable) { 
			if (plasmidAtEnd2) { 
				if (plusMon.isADP()) { return Env.kADPOff2WithPlas.getValue(); } else { return Env.kATPOff2WithPlas.getValue(); }
			} else if ((linkCt > 0) & (Env.sideBondsStabilize.isActive())) {
				if (minusMon.isADP()) { return Env.sideBondsStabilize.getValue(); } else { return Env.kATPOff2WithPlas.getValue(); }
			} else {
				if (Env.noCapThenADPdepoly.isActive() & !cap2Stable) { return Env.kADPOff2.getValue(); }  // use ADP off-rate no matter what if no stabilizing cap
				if (plusMon.isADP()) { return Env.kADPOff2.getValue(); } else { return Env.kATPOff2.getValue(); }
			}
		} else {
			if (plasmidAtEnd2) { return Env.kATPOff2NonHydroWithPlas.getValue(); }
			if (Env.noCapThenADPdepoly.isActive() & !cap2Stable) { return Env.kADPOff2.getValue(); }  // use ADP off-rate no matter what if no stabilizing cap
			return Env.kATPOff2NonHydro.getValue(); 
		}
		
	}
	
	public void end1BiochemSim () {
		if (!removeMe) { 
			minusEndDelta = 0;
			
			//if (!stericHindranceEnd1()) { talkln ("no stericHindrance end1"); }
			if ((!inCompression("at end1")) & (!stericHindranceEnd1()) & (cap1Stable)) {
				// normal ParM pool polymerization sim
				double rate = getPolyRateEnd1();
				boolean monomerAdded = addMonomerSim(rate);
				if (monomerAdded) { 
					//talkln ("end1 norm poly");
					coord.inc(-halfmono/2,uVec); 
					helixAng += (Math.PI - Env.helixAngInc);
					Monomer.polymerize(minusMon,this,Monomer.MINUSEND, true);
					minusEndDelta++;
				}
				// non-hydrolyzable ParM pool polymerization sim
				rate = getNonHydroPolyRateEnd1();
				monomerAdded = addNonHydroMonomerSim(rate);
				if (monomerAdded) { 
					//talkln ("end1 non-hydro poly");
					coord.inc(-halfmono/2,uVec); 
					helixAng += (Math.PI - Env.helixAngInc);
					Monomer.polymerize(minusMon,this,Monomer.MINUSEND, false);
					minusEndDelta++;
				}
			}
			
			// depolymerization...
			if (monomerCt >= Env.parMSeed.getIntValue()) {
				boolean monomerRemoved = removeMonomerSim(getDepolyRateEnd1(),minusMon);
				if (monomerRemoved) { 
					coord.inc(halfmono/2,uVec); 
					helixAng += (Math.PI + Env.helixAngInc);
					minusMon.depolymerize(this,Monomer.MINUSEND);
					minusEndDelta--;
				}
			} else {
				cleanup(this,true);
			}
		}
	}
	
	public void end2BiochemSim () {
		if (!removeMe) {
			
			//if (!stericHindranceEnd2()) { talkln ("no stericHindrance end2"); }
			if ((!inCompression("at end2")) & (!stericHindranceEnd2()) & (cap2Stable)) {
				// normal ParM polymerization
				double rate = getPolyRateEnd2();
				boolean monomerAdded = addMonomerSim(rate);
				if (monomerAdded) { 
					//talkln ("end2 norm poly");
					coord.inc(halfmono/2,uVec); 
					Monomer.polymerize(plusMon,this,Monomer.PLUSEND, true);
			    }
				// non-hydrolyzable ParM polymerization
				rate = getNonHydroPolyRateEnd2();
				monomerAdded = addNonHydroMonomerSim(rate);
				if (monomerAdded) { 
					//talkln ("end2 non-hydro poly");
					coord.inc(halfmono/2,uVec); 
					Monomer.polymerize(plusMon,this,Monomer.PLUSEND, false);
			    }
			}
			
			// depolymerize...
			if (monomerCt >= Env.parMSeed.getIntValue()) {
				boolean monomerRemoved = removeMonomerSim(getDepolyRateEnd2(),plusMon);
				if (monomerRemoved) { 
					coord.inc(-halfmono/2,uVec); 
					plusMon.depolymerize(this,Monomer.PLUSEND);
				}
			} else {
				cleanup(this,true);
			}
		}
	}
			
	
	public void resetCounters() {
		super.resetCounters();	// call the generic Thing method
		end1AxialF = 0;			// reset axial force at end1 to zero
		end2AxialF = 0; 			// reset axial force at end2 to zero
		lengthChanged = false;	// 
		end1LinkCkd = false;
		end2LinkCkd = false;
		end1TorqCkd = false;
		end2TorqCkd = false;
	}
	
	public static void zeroAllLinkCts () {
		for (int i=0;i<filSegmentCt;i++) {
			theFilSegments[i].linkCt = 0;
		}
	}

	public void checkBugCollision() {
		CollisionEvent CE1 = theBug.amIcolliding(end1,radius);
			if(CE1!=null){
				addBugForces(CE1,end1);
			}
		
		CollisionEvent CE2 = theBug.amIcolliding(end2,radius);
			if(CE2!=null){
				addBugForces(CE2,end2);
			}
	
	}
	
	/*public void addLinkForces () {
		// link forces only translational here, save rotation for a torsional spring
		// this filament takes care of link, if any, at end2 only
		
		if (filAtEnd2) {
			double strainDist = Pt3D.ptDist(end2,end2Fil.end1);
			double forceMag = Env.fracMove*1.0e-6*strainDist/((1/bTransGam.x + 1/end2Fil.bTransGam.x)*Env.deltaT.getValue());
			linkUVec.unitVec(strainDist,end2Fil.end1,end2);
			F.scale(forceMag,linkUVec);
			incForceSum(F);
		
			Fopp.scale(-1,F);
			end2Fil.incForceSum(Fopp);
		}
	}*/
	
	/*public void addLinkForces () {
		// version1.2 of soft lagrange multipliers; constraining forces translational and rotational.
		// this filament takes care of link, if any, at end2 only
		if (filAtEnd2) {
			linkPt.add(end2,Env.parMHalfMonoSize,uVecR);		// link point is half a monomer back from end2 tip
			double strainDist = Pt3D.ptDist(linkPt,end2Fil.end1);
			linkUVec.unitVec(strainDist,end2Fil.end1,linkPt);
			linkUVecR.scale(-1,linkUVec);
			// define cosines of angles between filament uVecs and line between endpoints
			double cosAngTween1 = Pt3D.CrossMag(uVec,linkUVec);  // use magnitude of cross product (which is Sin(theta)) 'cause we want Cos(90-theta)=Sin(theta)
			double cosAngTween2 = Pt3D.CrossMag(linkUVecR,end2Fil.uVec);
			double moveCoeff1 = 1/bTransGam.x + Math.pow(1e-6*length*cosAngTween1/2,2)/bRotGam.y;
			double moveCoeff2 = 1/end2Fil.bTransGam.x + Math.pow(1e-6*end2Fil.length*cosAngTween2/2,2)/end2Fil.bRotGam.y;
			double forceMag = (Env.fracMove*1.0e-6*strainDist)/(Env.deltaT.getValue()*(moveCoeff1 + moveCoeff2));
			
			F.scale(forceMag,linkUVec);
			incForceSum(F);
			R.scale(0.5e-6*length,uVec);
			RcrossF.cross(R,F);
			incTorqueSum(RcrossF);
			
			Fopp.scale(-1,F);
			end2Fil.incForceSum(Fopp);
			R.scale(0.5e-6*end2Fil.length,end2Fil.uVecR);
			RcrossF.cross(R,Fopp);
			end2Fil.incTorqueSum(RcrossF);
		}
	}*/
	
	public void validateEnd2Link() {
		// check state of link between end2  and end2Fil.. dissolve if problem found
		if (end2Fil == null | ptAtEnd2 == null) { 
			// no handle to the linked segment, or don't know which end linked on other seg... can only nullify this segments objects
			ptAtEnd2 = null;
			filAtEnd2 = false;
			return;
		}
		// if we've gotten here then check pointers of other segment
		if (ptAtEnd2 == end2Fil.end1) {
			boolean breakLink = false;
			if (!breakLink & !end2Fil.filAtEnd1) { breakLink = true; }
			if (!breakLink & end2Fil.ptAtEnd1 == null) { breakLink = true; }
			if (!breakLink & end2Fil.end1Fil == null) { breakLink = true; }
			if (!breakLink & end2Fil.end1Fil != this) { breakLink = true; }
			if (breakLink) { 
				end2Fil.removeEnd1Links();
				removeEnd2Links();
			}
		} else {
			boolean breakLink = false;
			if (!breakLink & !end2Fil.filAtEnd2) { breakLink = true; }
			if (!breakLink & end2Fil.ptAtEnd2 == null) { breakLink = true; }
			if (!breakLink & end2Fil.end2Fil == null) { breakLink = true; }
			if (!breakLink & end2Fil.end2Fil != this) { breakLink = true; }
			if (breakLink) { 
				end2Fil.removeEnd2Links();
				removeEnd2Links();
			}
		}
	}
	
	public void validateEnd1Link() {
		// check state of link between end1  and end1Fil.. dissolve if problem found
		if (end1Fil == null | ptAtEnd1 == null) { 
			// no handle to the linked segment, or don't know which end linked on other seg... can only nullify this segments objects
			ptAtEnd1 = null;
			filAtEnd1 = false;
			return;
		}
		// if we've gotten here then check pointers of other segment
		if (ptAtEnd1 == end1Fil.end1) {
			boolean breakLink = false;
			if (!breakLink & !end1Fil.filAtEnd1) { breakLink = true; }
			if (!breakLink & end1Fil.ptAtEnd1 == null) { breakLink = true; }
			if (!breakLink & end1Fil.end1Fil == null) { breakLink = true; }
			if (!breakLink & end1Fil.end1Fil != this) { breakLink = true; }
			if (breakLink) { 
				end1Fil.removeEnd1Links();
				removeEnd1Links();
			}
		} else {
			boolean breakLink = false;
			if (!breakLink & !end1Fil.filAtEnd2) { breakLink = true; }
			if (!breakLink & end1Fil.ptAtEnd2 == null) { breakLink = true; }
			if (!breakLink & end1Fil.end2Fil == null) { breakLink = true; }
			if (!breakLink & end1Fil.end2Fil != this) { breakLink = true; }
			if (breakLink) { 
				end1Fil.removeEnd2Links();
				removeEnd1Links();
			}
		}
	}
	
	public void breakAtEnd2() {
		if (filAtEnd2) {
			if (ptAtEnd2 == end2Fil.end1) {
				end2Fil.removeEnd1Links();
			} else {
				end2Fil.removeEnd2Links();
			}
		}
		removeEnd2Links();
	}
	
	public void breakAtEnd1() {
		if (filAtEnd1) {
			if (ptAtEnd1 == end1Fil.end2) {
				end1Fil.removeEnd2Links();
			} else {
				end1Fil.removeEnd1Links();
			}
		}
		removeEnd1Links();
	}
	
	public double moveCoeff (int end, Pt3D linkUVec) {
		double cosBeta;
		if (end == 2) {
			cosBeta = Pt3D.Dot(uVec, linkUVec);
		} else {
			cosBeta = Pt3D.Dot(uVecR, linkUVec);
		}
		double cosAlpha = Pt3D.OppTrig(cosBeta);
		double lSqrd = 1e-12*length*length;
		double Cx = cosBeta*cosBeta/bTransGam.x;
		double Cperp = cosAlpha*cosAlpha/bTransGam.y;
		double Ctheta = lSqrd*cosAlpha*cosAlpha/(4*bRotGam.y);
		double moveC = Cx + Cperp + Ctheta;
		return moveC;
	}
	
	public void registerFilLink (double loc, int id) {
		// FilLinks register every time-step with FilSegments.. linkCt set to zero in resetCounters()
		if (linkCt > linkLocs.length-1) { 
			linkLocs = new double[2*Env.maxLinksOnSeg.getIntValue()]; 
			linkIds = new int[2*Env.maxLinksOnSeg.getIntValue()];
		}
		linkLocs[linkCt] = loc;
		linkIds[linkCt] = id;
		linkCt++;
	}
		
	public void addLinkForces () {
		// version1.2 of soft lagrange multipliers; constraining forces translational and rotational.
		// this filament takes care of both end links, if not already visited, then marks as visited on this and linked fils
		// this method doesn't care if ptAtEnd2 is end2Fil.end1 or end2Fil.end2
		
		// first double-check validity of links
		// non-existence of attached filament
		validateEnd2Link();
		validateEnd1Link();
		
		if (filAtEnd2 & !end2LinkCkd) {
			linkPt.add(end2,Env.parMHalfMonoSize,uVecR);		// link point is half a monomer back from end2 tip
			double strainDist = Pt3D.ptDist(linkPt,ptAtEnd2);
			
			// check distance if fils can break apart
			end2SegDist.registerValue(strainDist);
			if (Env.maxSegDist.isActive() & end2SegDist.averageVal() > Env.maxSegDist.getValue()) {
				breakAtEnd2();
				talkln ("broke because dist too large between linked segments");
				return;
			}
			
			linkUVec.unitVec(strainDist,ptAtEnd2,linkPt);
			linkUVecR.scale(-1,linkUVec);
			
			double moveCoeff1 = moveCoeff(2,linkUVec);
			double moveCoeff2;
			if (ptAtEnd2 == end2Fil.end1) {
				moveCoeff2 = end2Fil.moveCoeff(1, linkUVecR);
			} else {
				moveCoeff2 = end2Fil.moveCoeff(2, linkUVecR);
			}
			double forceMag = (Env.fracMove.getValue()*1.0e-6*strainDist)/(Env.deltaT.getValue()*(moveCoeff1 + moveCoeff2));
			
			// filter instantaneous F through averaging
			F.scale(forceMag,linkUVec);
			//filLink2Track.registerValue(F);
			//F.copy(filLink2Track.averagePtVal());
			
			incForceSum(F);
			R.scale(0.5e-6*length*Env.fracR.getValue(),uVec);
			RcrossF.cross(R,F);
			incTorqueSum(RcrossF);
			end2LinkCkd = true;
			
			Fopp.scale(-1,F);
			end2Fil.incForceSum(Fopp);
			if (ptAtEnd2 == end2Fil.end1) {
				R.scale(0.5e-6*end2Fil.length*Env.fracR.getValue(),end2Fil.uVecR);
				end2Fil.end1LinkCkd = true;
			} else {
				R.scale(0.5e-6*end2Fil.length*Env.fracR.getValue(),end2Fil.uVec);
				end2Fil.end2LinkCkd = true;
			}
			RcrossF.cross(R,Fopp);
			end2Fil.incTorqueSum(RcrossF);
			
			// add these link forces to the axial loads on each segment
			incEnd2AxialForce(Pt3D.Dot(uVec,F)); // axial force contribution
			if (ptAtEnd2 == end2Fil.end1) {
				end2Fil.incEnd1AxialForce(Pt3D.Dot(end2Fil.uVecR,Fopp)); // axial force contribution
			} else {
				end2Fil.incEnd2AxialForce(Pt3D.Dot(end2Fil.uVec,Fopp)); // axial force contribution
			}
			
			// propagate a change in filID.... lower filID always used
			if (filID != end2Fil.filID) {
				if (filID < end2Fil.filID) { end2Fil.filID = filID; } else { filID = end2Fil.filID; }
			}
		}
		
		// take care of link at end1
		if (filAtEnd1 & !end1LinkCkd) {
			linkPt.add(end1,Env.parMHalfMonoSize,uVec);		// link point is half a monomer back from end2 tip
			double strainDist = Pt3D.ptDist(linkPt,ptAtEnd1);
			
			// check distance if fils can break apart
			end1SegDist.registerValue(strainDist);
			if (Env.maxSegDist.isActive() & end1SegDist.averageVal() > Env.maxSegDist.getValue()) {
				breakAtEnd1();
				talkln ("broke because dist too large between linked segments");
				return;
			}
			
			linkUVec.unitVec(strainDist,ptAtEnd1,linkPt);
			linkUVecR.scale(-1,linkUVec);
			
			double moveCoeff1 = moveCoeff(1,linkUVec);
			double moveCoeff2;
			if (ptAtEnd1 == end1Fil.end1) {
				moveCoeff2 = end1Fil.moveCoeff(1, linkUVecR);
			} else {
				moveCoeff2 = end1Fil.moveCoeff(2, linkUVecR);
			}
			double forceMag = (Env.fracMove.getValue()*1.0e-6*strainDist)/(Env.deltaT.getValue()*(moveCoeff1 + moveCoeff2));
			
			//filter instantaneous F through averaging
			F.scale(forceMag,linkUVec);
			//filLink1Track.registerValue(F);
			//F.copy(filLink1Track.averagePtVal());
			
			incForceSum(F);
			R.scale(0.5e-6*length*Env.fracR.getValue(),uVecR);
			RcrossF.cross(R,F);
			incTorqueSum(RcrossF);
			end1LinkCkd = true;
			
			Fopp.scale(-1,F);
			end1Fil.incForceSum(Fopp);
			if (ptAtEnd1 == end1Fil.end1) {
				R.scale(0.5e-6*end1Fil.length*Env.fracR.getValue(),end1Fil.uVecR);
				end1Fil.end1LinkCkd = true;
			} else {
				R.scale(0.5e-6*end1Fil.length*Env.fracR.getValue(),end1Fil.uVec);
				end1Fil.end2LinkCkd = true;
			}
			//R.zero();	// remove if you want torque from links
			RcrossF.cross(R,Fopp);
			end1Fil.incTorqueSum(RcrossF);
			
			// add these link forces to the axial loads on each segment
			incEnd1AxialForce(Pt3D.Dot(uVecR,F)); // axial force contribution
			if (ptAtEnd1 == end1Fil.end1) {
				end1Fil.incEnd1AxialForce(Pt3D.Dot(end1Fil.uVecR,Fopp)); // axial force contribution
			} else {
				end1Fil.incEnd2AxialForce(Pt3D.Dot(end1Fil.uVec,Fopp)); // axial force contribution
			}
			
			// propagate a change in filID.... lower filID always used
			if (filID != end1Fil.filID) {
				if (filID < end1Fil.filID) { end1Fil.filID = filID; } else { filID = end1Fil.filID; }
			}
		}
	}
	
	public void addLinkForcesOld () {
		// version1.2 of soft lagrange multipliers; constraining forces translational and rotational.
		// this filament takes care of both end links, if not already visited, then marks as visited on this and linked fils
		// this method doesn't care if ptAtEnd2 is end2Fil.end1 or end2Fil.end2
		
		// first double-check validity of links
		// non-existence of attached filament
		validateEnd2Link();
		validateEnd1Link();
		
		if (filAtEnd2 & !end2LinkCkd) {
			linkPt.add(end2,Env.parMHalfMonoSize,uVecR);		// link point is half a monomer back from end2 tip
			double strainDist = Pt3D.ptDist(linkPt,ptAtEnd2);
			
			// check distance if fils can break apart
			end2SegDist.registerValue(strainDist);
			if (Env.maxSegDist.isActive() & end2SegDist.averageVal() > Env.maxSegDist.getValue()) {
				breakAtEnd2();
				talkln ("broke because dist too large between linked segments");
				return;
			}
				
			linkUVec.unitVec(strainDist,ptAtEnd2,linkPt);
			linkUVecR.scale(-1,linkUVec);
			// define cosines of angles between filament uVecs and line between endpoints
			double cosAngTween1 = Pt3D.CrossMag(uVec,linkUVec);  // use magnitude of cross product (which is Sin(theta)) 'cause we want Cos(90-theta)=Sin(theta)
			double cosAngTween2;
			if (ptAtEnd2 == end2Fil.end1) {
				cosAngTween2 = Pt3D.CrossMag(linkUVecR,end2Fil.uVec);
			} else {
				cosAngTween2 = Pt3D.CrossMag(linkUVecR,end2Fil.uVecR);
			}
			double moveCoeff1 = 1/bTransGam.x + Math.pow(1e-6*length*cosAngTween1/2,2)/bRotGam.y;
			double moveCoeff2 = 1/end2Fil.bTransGam.x + Math.pow(1e-6*end2Fil.length*cosAngTween2/2,2)/end2Fil.bRotGam.y;
			double forceMag = (Env.fracMove.getValue()*1.0e-6*strainDist)/(Env.deltaT.getValue()*(moveCoeff1 + moveCoeff2));
		
			// filter instantaneous F through averaging
			F.scale(forceMag,linkUVec);
			//filLink2Track.registerValue(F);
			//F.copy(filLink2Track.averagePtVal());
			
			incForceSum(F);
			R.scale(0.5e-6*length,uVec);
			RcrossF.cross(R,F);
			incTorqueSum(RcrossF);
			end2LinkCkd = true;
			
			Fopp.scale(-1,F);
			end2Fil.incForceSum(Fopp);
			if (ptAtEnd2 == end2Fil.end1) {
				R.scale(0.5e-6*end2Fil.length,end2Fil.uVecR);
				end2Fil.end1LinkCkd = true;
			} else {
				R.scale(0.5e-6*end2Fil.length,end2Fil.uVec);
				end2Fil.end2LinkCkd = true;
			}
			RcrossF.cross(R,Fopp);
			end2Fil.incTorqueSum(RcrossF);
			
			// add these link forces to the axial loads on each segment
			incEnd2AxialForce(Pt3D.Dot(uVec,F)); // axial force contribution
			if (ptAtEnd2 == end2Fil.end1) {
				end2Fil.incEnd1AxialForce(Pt3D.Dot(end2Fil.uVecR,Fopp)); // axial force contribution
			} else {
				end2Fil.incEnd2AxialForce(Pt3D.Dot(end2Fil.uVec,Fopp)); // axial force contribution
			}
			
			// propagate a change in filID.... lower filID always used
			if (filID != end2Fil.filID) {
				if (filID < end2Fil.filID) { end2Fil.filID = filID; } else { filID = end2Fil.filID; }
			}
		}
		
		// take care of link at end1
		if (filAtEnd1 & !end1LinkCkd) {
			linkPt.add(end1,Env.parMHalfMonoSize,uVec);		// link point is half a monomer back from end2 tip
			double strainDist = Pt3D.ptDist(linkPt,ptAtEnd1);
			
			// check distance if fils can break apart
			end1SegDist.registerValue(strainDist);
			if (Env.maxSegDist.isActive() & end1SegDist.averageVal() > Env.maxSegDist.getValue()) {
				breakAtEnd1();
				talkln ("broke because dist too large between linked segments");
				return;
			}
			
			linkUVec.unitVec(strainDist,ptAtEnd1,linkPt);
			linkUVecR.scale(-1,linkUVec);
			// define cosines of angles between filament uVecs and line between endpoints
			double cosAngTween1 = Pt3D.CrossMag(uVecR,linkUVec);  // use magnitude of cross product (which is Sin(theta)) 'cause we want Cos(90-theta)=Sin(theta)
			double cosAngTween2;
			if (ptAtEnd1 == end1Fil.end1) {
				cosAngTween2 = Pt3D.CrossMag(linkUVecR,end1Fil.uVec);
			} else {
				cosAngTween2 = Pt3D.CrossMag(linkUVecR,end1Fil.uVecR);
			}
			double moveCoeff1 = 1/bTransGam.x + Math.pow(1e-6*length*cosAngTween1/2,2)/bRotGam.y;
			double moveCoeff2 = 1/end1Fil.bTransGam.x + Math.pow(1e-6*end1Fil.length*cosAngTween2/2,2)/end1Fil.bRotGam.y;
			double forceMag = (Env.fracMove.getValue()*1.0e-6*strainDist)/(Env.deltaT.getValue()*(moveCoeff1 + moveCoeff2));
			
			//filter instantaneous F through averaging
			F.scale(forceMag,linkUVec);
			//filLink1Track.registerValue(F);
			//F.copy(filLink1Track.averagePtVal());
			
			incForceSum(F);
			R.scale(0.5e-6*length,uVecR);
			//R.zero();	// remove if you want torque from links
			RcrossF.cross(R,F);
			incTorqueSum(RcrossF);
			end1LinkCkd = true;
			
			Fopp.scale(-1,F);
			end1Fil.incForceSum(Fopp);
			if (ptAtEnd1 == end1Fil.end1) {
				R.scale(0.5e-6*end1Fil.length,end1Fil.uVecR);
				end1Fil.end1LinkCkd = true;
			} else {
				R.scale(0.5e-6*end1Fil.length,end1Fil.uVec);
				end1Fil.end2LinkCkd = true;
			}
			//R.zero();	// remove if you want torque from links
			RcrossF.cross(R,Fopp);
			end1Fil.incTorqueSum(RcrossF);
			
			// add these link forces to the axial loads on each segment
			incEnd1AxialForce(Pt3D.Dot(uVecR,F)); // axial force contribution
			if (ptAtEnd1 == end1Fil.end1) {
				end1Fil.incEnd1AxialForce(Pt3D.Dot(end1Fil.uVecR,Fopp)); // axial force contribution
			} else {
				end1Fil.incEnd2AxialForce(Pt3D.Dot(end1Fil.uVec,Fopp)); // axial force contribution
			}
			
			// propagate a change in filID.... lower filID always used
			if (filID != end1Fil.filID) {
				if (filID < end1Fil.filID) { end1Fil.filID = filID; } else { filID = end1Fil.filID; }
			}
		}
	}
	

	
	public void addTorsionSpringForces () {
		// rotational spring which works to straighten out the filament
		// this filament takes care of spring at both ends, if not visited, then marks as visited for involved fils
		// this method doesn't care if ptAtEnd2 is end2Fil.end1 or end2Fil.end2
		
		if (filAtEnd2 & !end2TorqCkd) {
			end2TorqCkd = true;
			double dotVecs;
			if (ptAtEnd2 == end2Fil.end1) {
				end2Fil.end1TorqCkd = true;
				torsionVec.cross(uVec,end2Fil.uVec);
				torsionVec.unitVec();
				dotVecs = Pt3D.Dot(uVec,end2Fil.uVec);
			} else {
				end2Fil.end2TorqCkd = true;
				torsionVec.cross(uVec,end2Fil.uVecR);
				torsionVec.unitVec();
				dotVecs = Pt3D.Dot(uVec,end2Fil.uVecR);
			}
			
			if (dotVecs > 1.0) { dotVecs = 1.0; }
			double angTween = Math.acos(dotVecs)*180/Math.PI;
			
			// check if angle too large
			end2SegAng.registerValue(angTween);
			/*if (Env.maxSegAngle.isActive() & end2SegAng.averageVal() > Env.maxSegAngle.getValue()/4) { 
				talkln ("Something happening!");
				Env.paintOn = true;
			}*/
			if (Env.maxSegAngle.isActive() & end2SegAng.averageVal() > Env.maxSegAngle.getValue()) { 
				angTween = 0;
				talkln ("broke because angle too large between linked segments");
				filAtEnd2 = false; ptAtEnd2 = null; 
			}
			
			//talkln ("DotVecs is " + dotVecs + " and angTween is " + angTween);
			double torsionMag;
			if (Env.filTorqSpring.isActive()) {
				torsionMag = Env.fracMoveTorq.getValue()*Env.filTorqSpring.getValue()*angTween;
			} else { 
				//torsionMag = Env.fracMoveTorq.getValue()*(Math.PI/180)*end2SegAng.averageVal()/((1/bRotGam.y + 1/end2Fil.bRotGam.y)*Env.deltaT.getValue());
				torsionMag = Env.fracMoveTorq.getValue()*(Math.PI/180)*angTween/((1/bRotGam.y + 1/end2Fil.bRotGam.y)*Env.deltaT.getValue());
			}
		
			if (torsionVec.checkPt3D()) {
				torsionVec.scale(torsionMag);
				//filTorque2Track.registerValue(torsionVec);
				//torsionVec.copy(filTorque2Track.averagePtVal());
				incTorqueSum(torsionVec);
			
				torsionVec.scale(-1);
				end2Fil.incTorqueSum(torsionVec);
			} else {
				talkln ("Crazy torque result in FilSegment.addTorsionSpringForce() part 2");
			}
		}
		
		
		if (filAtEnd1 & !end1TorqCkd) {
			end1TorqCkd = true;
			double dotVecs;
			if (ptAtEnd1 == end1Fil.end1) {
				end1Fil.end1TorqCkd = true;
				torsionVec.cross(uVecR,end1Fil.uVec);
				torsionVec.unitVec();
				dotVecs = Pt3D.Dot(uVecR,end1Fil.uVec);
			} else {
				end1Fil.end2TorqCkd = true;
				torsionVec.cross(uVecR,end1Fil.uVecR);
				torsionVec.unitVec();
				dotVecs = Pt3D.Dot(uVecR,end1Fil.uVecR);
			}
			
			if (dotVecs > 1.0) { dotVecs = 1.0; }
			double angTween = Math.acos(dotVecs)*180/Math.PI;
			
			// check if angle too large
			end1SegAng.registerValue(angTween);
			/*if (Env.maxSegAngle.isActive() & end1SegAng.averageVal() > Env.maxSegAngle.getValue()/4) { 
				talkln ("Something happening!");
				Env.paintOn = true;
			}*/
			if (Env.maxSegAngle.isActive() & end1SegAng.averageVal() > Env.maxSegAngle.getValue()) { 
				angTween = 0;
				talkln ("broke because angle too large between linked segments");
				filAtEnd1 = false; ptAtEnd1 = null; 
			}
			
			//talkln ("DotVecs is " + dotVecs + " and angTween is " + angTween);
			double torsionMag;
			if (Env.filTorqSpring.isActive()) {
				torsionMag = Env.fracMoveTorq.getValue()*Env.filTorqSpring.getValue()*angTween;
			} else { 
				//torsionMag = Env.fracMoveTorq.getValue()*(Math.PI/180)*end1SegAng.averageVal()/((1/bRotGam.y + 1/end1Fil.bRotGam.y)*Env.deltaT.getValue());
				torsionMag = Env.fracMoveTorq.getValue()*(Math.PI/180)*angTween/((1/bRotGam.y + 1/end1Fil.bRotGam.y)*Env.deltaT.getValue());
			}
			if (torsionVec.checkPt3D()) {
				torsionVec.scale(torsionMag);
				//filTorque1Track.registerValue(torsionVec);
				//torsionVec.copy(filTorque1Track.averagePtVal());
				incTorqueSum(torsionVec);
				
				torsionVec.scale(-1);
				end1Fil.incTorqueSum(torsionVec);
			} else {
				talkln ("Crazy torque result in FilSegment.addTorsionSpringForce() part 1");
			}
		}
	}
	
	public boolean bindNoMore (int bindId) {
		int otherBound = 0;
		if (Env.maxSegsToBind.getIntValue() == 0) { return true; }
		if (linkCt < Env.maxSegsToBind.getIntValue()) { return false; } 
		for (int i=0;i<linkCt;i++) {
			if (linkIds[i] == bindId) {
				return false;
			} else {
				otherBound++;
				if (otherBound >= Env.maxSegsToBind.getIntValue()) { return true; }
			}
		}
		return false;
	}
	
	public boolean tooCloseFilLinkLoc (double locToCheck) {
		if (linkCt >= Env.maxLinksOnSeg.getValue()) return true;		// fails if max links achieved
		double curD;
		for (int i=0;i<linkCt;i++) {
			curD = Math.abs(locToCheck - linkLocs[i]);
			if (curD < minFilLinkSep) { return true; }	// fails if one of the existing links is too close to this new location
		}
		return false;
	}
	
	public static void checkFilSegCollisions () {
		FilSegment iSeg,jSeg;
		for (int i=0;i<filSegmentCt;i++) {
			iSeg = theFilSegments[i];
			if (iSeg.linkCt < Env.maxLinksOnSeg.getValue()) {
				for (int j=i+1;j<filSegmentCt;j++) {
					jSeg = theFilSegments[j];
					if (jSeg.linkCt < Env.maxLinksOnSeg.getValue()) {
						if (roughCollisionCheck(iSeg,jSeg)) {
							boolean annealed = false;
							if (Env.filamentsAnneal.isActive()) { annealed = checkForAnnealing(iSeg,jSeg); }
							if (!annealed & Env.sideBonds.isActive()) { checkToLink(iSeg,jSeg); }
							//if (Env.sideBonds.isActive()) { checkToLink(iSeg,jSeg); }
						}
					}
				}
			}
		}
	}
	
	
	public static boolean roughCollisionCheck (FilSegment fil1, FilSegment fil2) {
		if (fil1.filID == fil2.filID) { return false; }
		if (Math.abs(fil1.coord.x - fil2.coord.x) > fil1.xRange+fil2.xRange) { return false; }		// quick checks
		if (Math.abs(fil1.coord.y - fil2.coord.y) > fil1.yRange+fil2.yRange) { return false; }
		if (Math.abs(fil1.coord.z - fil2.coord.z) > fil1.zRange+fil2.zRange) { return false; }
			
		return true;
	}
	
	public static void checkToLink (FilSegment fil1, FilSegment fil2) {
		RetObj retO = fil1.retObj;
		lineSegmentIntersectTest(fil1.end1,fil1.end2,fil2.end1,fil2.end2,retO);
		if (retO.collision) {
			double loc1 = Pt3D.ptDist(fil1.end1,retO.conPt1);// + (2*Env.mtRNG.raw()-1)*(fil1.minFilLinkSep);
			double loc2 = Pt3D.ptDist(fil2.end1,retO.conPt2);// + (2*Env.mtRNG.raw()-1)*(fil2.minFilLinkSep);
			if (loc1 > fil1.length) { loc1 = fil1.length; }
			if (loc1 < 0) { loc1 = 0; }
			if (loc2 > fil2.length) { loc2 = fil2.length; }
			if (loc2 < 0) { loc2 = 0; }
			
			if (fil1.tooCloseFilLinkLoc(loc1)) { return; }
			if (fil2.tooCloseFilLinkLoc(loc2)) { return; }
			
			if (Env.maxSegsToBind.isActive()) {
				if (fil1.bindNoMore(fil2.filID)) { return; }
				if (fil2.bindNoMore(fil1.filID)) { return; }
			}
			
			// check angle of filaments
			boolean filAngleOK = false;
			double vecDot = Pt3D.Dot(fil1.uVec,fil2.uVec);
			double angTween = Math.acos(vecDot);
			if (Env.bindOppositePolarity.isActive()) {
				if (angTween < Math.PI & angTween > Math.PI-Math.PI*Env.maxToLinkAngle.getValue()/180) { filAngleOK = true; }
			}
			if (Env.bindSamePolarity.isActive()) {
				if (angTween > 0 & angTween < Math.PI*Env.maxToLinkAngle.getValue()/180) { filAngleOK = true; }
			}
			
			if (filAngleOK) { new FilLink (fil1, loc1, fil2, loc2,angTween); }
		}
	}
	
	public static FilSegment getRandomFilSeg () {
		int luckyOne = (int)(Math.floor(Env.mtRNG.raw()*filSegmentCt));
		return theFilSegments[luckyOne];
	}
	
	public static void oneRateAnnealing () {
		// roll the dice and if an annealing event occurs, pick 2 fils at random to anneal
		if (!Env.annealingConstant.isActive()) { return; }
		if (FilSegment.filSegmentCt < 2) { return; }
		if (Env.mtRNG.raw() < Env.oneRateAnnealing*Env.deltaT.getValue()) {
			FilSegment fil1 = getRandomFilSeg();
			while (fil1.monomerCt < 3) { fil1 = getRandomFilSeg(); }
			FilSegment fil2 = getRandomFilSeg();
			while ((fil1.equals(fil2)) | (fil2.monomerCt < 3)) { fil2 = getRandomFilSeg(); }
			annealBiochemOnlySegments(fil1,fil2);
		}
	}
	
	public static boolean checkForAnnealing (FilSegment fil1, FilSegment fil2) {
		double ptD, cosAngTween;
		if (fil1.filAtEnd1 & fil1.filAtEnd2) { return false; } // fil1 is interior, can't anneal
		if (fil2.filAtEnd1 & fil2.filAtEnd2) { return false; } // fil2 is interior, can't anneal
		if (Env.mtRNG.raw() > Env.annealProb) { return false; }
		
		// check a mess (4) of different possibilities
		if (!fil1.filAtEnd2 & !fil1.plasmidAtEnd2 & fil1.capConditionOKEnd2()) {
			if (!fil2.filAtEnd1 & !fil2.plasmidAtEnd1 & fil2.capConditionOKEnd1()) {
				ptD = Pt3D.ptDist(fil1.end2, fil2.end1);
				if (ptD < Env.annealDist.getValue()) { 
					//System.out.println("1st Ck passed: ptD = " + ptD);
					cosAngTween = Pt3D.Dot(fil1.uVec, fil2.uVec);
					if (cosAngTween > Env.annealAngleCosine.getValue()) {
						//System.out.println ("case1");
						FilSegment.annealSegments(fil1, fil1.end2, fil2, fil2.end1);
						//System.out.println ("Annealed!");
						return true;
					}
				}
			}
			
			if (!fil2.filAtEnd2 & !fil2.plasmidAtEnd2 & fil2.capConditionOKEnd2()) {
				ptD = Pt3D.ptDist(fil1.end2,fil2.end2);
				if (ptD < Env.annealDist.getValue()) { 
					cosAngTween = Pt3D.Dot(fil1.uVec, fil2.uVecR);
					if (cosAngTween > Env.annealAngleCosine.getValue()) {
						//System.out.println ("case2");
						FilSegment.annealSegments(fil1, fil1.end2, fil2, fil2.end2);
						//System.out.println ("Annealed!");
						return true;
					}
				}
			}
		}
		
		if (!fil1.filAtEnd1 & !fil1.plasmidAtEnd1 & fil1.capConditionOKEnd1()) {
			if (!fil2.filAtEnd1 & !fil2.plasmidAtEnd1 & fil2.capConditionOKEnd1()) {
				ptD = Pt3D.ptDist(fil1.end1, fil2.end1);
				if (ptD < Env.annealDist.getValue()) { 
					cosAngTween = Pt3D.Dot(fil1.uVecR, fil2.uVec);
					if (cosAngTween > Env.annealAngleCosine.getValue()) {
						//System.out.println ("case3");
						FilSegment.annealSegments(fil1, fil1.end1, fil2, fil2.end1);
						//System.out.println ("Annealed!");
						return true;
					}
				}
			}
			
			if (!fil2.filAtEnd2 & !fil2.plasmidAtEnd2 & fil2.capConditionOKEnd2()) {
				ptD = Pt3D.ptDist(fil1.end1, fil2.end2);
				if (ptD < Env.annealDist.getValue()) { 
					cosAngTween = Pt3D.Dot(fil1.uVecR, fil2.uVecR);
					if (cosAngTween > Env.annealAngleCosine.getValue()) {
						//System.out.println ("case4");
						FilSegment.annealSegments(fil1, fil1.end1, fil2, fil2.end2);
						//System.out.println ("Annealed!");
						return true;
					}
				}
			}
		}
		return false;
	}
	
	public boolean ptInPlasmidBoundingBox (Pt3D pt, Plasmid plas) {
		// quicker checks to see if this end could be colliding with this plasmid... bounding box
		double cushion = Env.parMMonoSize;
		double plasRad = plas.getRadius()+cushion;
		// x coord
		if (pt.x < plas.coord.x - plasRad) { return false; }
		if (pt.x > plas.coord.x + plasRad) { return false; }
		// y coord
		if (pt.y < plas.coord.y - plasRad) { return false; }
		if (pt.y > plas.coord.y + plasRad) { return false; }
		// z coord
		if (pt.z < plas.coord.z - plasRad) { return false; }
		if (pt.z > plas.coord.z + plasRad) { return false; }
		
		return true;
	}
	
	public void checkPlasmidCollision() {
		// if colliding with plasmid at end1 then push away
		for (int i=0;i<Plasmid.plasmidCt;i++){
			Plasmid curPlasmid = Plasmid.thePlasmids[i];
			if (ptInPlasmidBoundingBox(end1,curPlasmid)) {
				double distToEnd1 = Pt3D.ptDist(curPlasmid.coord,end1);
				double impDist = curPlasmid.getRadius() - distToEnd1;
				//if (impDist > -halfmono) { end1TipC = 0; } 	// steric hindrance to polymerization set
				if (impDist > 0) {
					linkUVec.unitVec(distToEnd1,end1,curPlasmid.coord);
					linkUVecR.scale(-1,linkUVec);
					// define cosines of angles between filament uVecs and line between endpoints
					double cosAngTween1 = Pt3D.CrossMag(uVecR,linkUVec);  // use magnitude of cross product (which is Sin(theta)) 'cause we want Cos(90-theta)=Sin(theta)
					double moveCoeff1 = 1/bTransGam.x + Math.pow(1e-6*length*cosAngTween1/2,2)/bRotGam.y;
					double moveCoeff2 = 1/curPlasmid.bTransGam.x;
					double forceMag = (Env.fracMove.getValue()*1.0e-6*impDist)/(Env.deltaT.getValue()*(moveCoeff1 + moveCoeff2));
					
					F.scale(forceMag,linkUVec);
					incForceSum(F);
					R.scale(1e-6*length/2,uVecR);
					RcrossF.cross(R,F);
					incTorqueSum(RcrossF);
					
					Fopp.scale(-1,F);
					curPlasmid.incForceSum(Fopp);
					// note: no torque on plasmid.... force through CM
				}
			}
		}

		// if colliding with plasmid at end2 then push away
		for (int i=0;i<Plasmid.plasmidCt;i++){
			Plasmid curPlasmid = Plasmid.thePlasmids[i];
			if (ptInPlasmidBoundingBox(end2,curPlasmid)) {
				double distToEnd2 = Pt3D.ptDist(curPlasmid.coord,end2);
				double impDist = curPlasmid.getRadius() - distToEnd2;
				//if (impDist > -halfmono) { end2TipC = 0; } 	// steric hindrance to polymerization set
				if (impDist > 0) {
					linkUVec.unitVec(distToEnd2,end2,curPlasmid.coord);
					linkUVecR.scale(-1,linkUVec);
					// define cosines of angles between filament uVecs and line between endpoints
					double cosAngTween1 = Pt3D.CrossMag(uVec,linkUVec);  // use magnitude of cross product (which is Sin(theta)) 'cause we want Cos(90-theta)=Sin(theta)
					double moveCoeff1 = 1/bTransGam.x + Math.pow(1e-6*length*cosAngTween1/2,2)/bRotGam.y;
					double moveCoeff2 = 1/curPlasmid.bTransGam.x;
					double forceMag = (Env.fracMove.getValue()*1.0e-6*impDist)/(Env.deltaT.getValue()*(moveCoeff1 + moveCoeff2));
					
					F.scale(forceMag,linkUVec);
					incForceSum(F);
					R.scale(1e-6*length/2,uVec);
					RcrossF.cross(R,F);
					incTorqueSum(RcrossF);
					
					Fopp.scale(-1,F);
					curPlasmid.incForceSum(Fopp);
					// note: no torque on plasmid.... force through CM
				}
			}
		}
	}
	
	public boolean end1CanBindThisPlasmid (Plasmid curPlas) {
		if (curPlas.canBindFilament()) { // | globalPlasAtEnd2) {
			if (ptInPlasmidBoundingBox(end1,curPlas)) { return true; }
		}
		return false;
	}
		
	public boolean end2CanBindThisPlasmid (Plasmid curPlas) {
		if (curPlas.canBindFilament()) { // | globalPlasAtEnd1) {
			if (ptInPlasmidBoundingBox(end2,curPlas)) { return true; }
		}
		return false;
	}
	
	public void checkPlasmidBinding() {
		if (Env.end1CanBindPlasmid.isActive()) { checkPlasmidBindingEnd1(); }
		if (Env.end2CanBindPlasmid.isActive()) { checkPlasmidBindingEnd2(); }
	}
	
	public void checkPlasmidBindingEnd1() {
		if ((!plasmidAtEnd1) & (!end1Catastrophy) & (!filAtEnd1) & (end1DetachCounter == 0)) {
			for (int i=0;i<Plasmid.plasmidCt;i++){
				Plasmid curPlasmid = Plasmid.thePlasmids[i];
				if (end1CanBindThisPlasmid(curPlasmid)) { 
					double distplasmid = Pt3D.ptDist(curPlasmid.coord,end1);
					if ((distplasmid<curPlasmid.getRadius()) & (curPlasmid != end2Plasmid)) {
						plasmidAtEnd1= true;
						end1Plasmid=curPlasmid;
						linkUVec.sub(curPlasmid.getRadius()/distplasmid,end1,curPlasmid.coord);	// vector to edge of plasmid in direction from coord to end1
						end1PAttachPt.XTox(curPlasmid,linkUVec);
						curPlasmid.filamentOn();
						//gotPlasmidAppearance();
						setGlobalEnd1Plasmid(true);
					}
				}
			}
		}

	}
	
	public void checkPlasmidBindingEnd2() {
		if ((!plasmidAtEnd2) & (!end2Catastrophy) & (!filAtEnd2) & (end2DetachCounter ==0)) {
			for (int i=0;i<Plasmid.plasmidCt;i++){
				Plasmid curPlasmid = Plasmid.thePlasmids[i];
				if (end2CanBindThisPlasmid(curPlasmid)) {
					double distplasmid = Pt3D.ptDist(curPlasmid.coord,end2);
					if ((distplasmid<curPlasmid.getRadius()) & (curPlasmid != end1Plasmid)) {
						plasmidAtEnd2= true;
						end2Plasmid=curPlasmid;
						linkUVec.sub(curPlasmid.getRadius()/distplasmid,end2,curPlasmid.coord);	// vector to edge of plasmid in direction from coord to end2
						end2PAttachPt.XTox(curPlasmid,linkUVec);
						curPlasmid.filamentOn();
						//gotPlasmidAppearance();
						setGlobalEnd2Plasmid(true);
					}
				}
			}
		}
	}
	
	public void addPlasmidForces() {
		if (plasmidAtEnd1) {
			end1Plasmid.registerFilament(this);
			if (Env.parRparCMoves.isActive()) {
				// reassign attachpt..
				end1PAttachPt.sub(end1,end1Plasmid.coord);
				end1PAttachPt.unitVec();
				end1PAttachPt.scale(end1Plasmid.getRadius());
				end1PAttachPt.XTox(end1Plasmid);
			}
			
			end1PAttachPtInX.xToXPlusxOrigin(end1Plasmid,end1PAttachPt);
			double strainDist = Pt3D.ptDist(end1PAttachPtInX,end1);
			double forceMag = Env.fracMove.getValue()*1.0e-6*strainDist/((1/bTransGam.x + 1/end1Plasmid.bTransGam.x)*Env.deltaT.getValue());
			//double forceMag=Env.kParM_PlasmidSpring*(Pt3D.ptDist(end1Plasmid.coord,end1));
			toPlasmidUVec.unitVec(end1PAttachPtInX,end1);
			F.scale(forceMag,toPlasmidUVec);
			incForceSum(F);
			R.scale(1e-6*(0.5*length),uVecR);
			double axialF = Pt3D.Dot(uVecR, F); // axial force contribution
			incEnd1AxialForce(axialF); 
			RcrossF.cross(R,F);
			incTorqueSum(RcrossF);
			
			Fopp.scale(-1,F);
			end1Plasmid.incForceSum(Fopp);
			R.sub(1e-6,end1PAttachPtInX,end1Plasmid.coord);
			RcrossF.cross(R,Fopp);
			end1Plasmid.incTorqueSum(RcrossF);
			
			/*// steric hindrance from plasmid
			double toEnd1Dist = Pt3D.ptDist(end1Plasmid.coord, end1);
			double tipC = toEnd1Dist - end1Plasmid.getRadius();
			//talkln ("end1TipC from plasmid is " + tipC);
			if (toEnd1Dist < (end1Plasmid.getRadius()-halfmono)) { end1TipC = 0; }
			*/
			
			// torsional spring between plasmid and segment
			toPlasmidUVec.unitVec(end1,end1Plasmid.coord);	// the unit vector orthogonal to the plasmid at the attachment point
			torsionVec.cross(uVec,toPlasmidUVec);
			torsionVec.unitVec();
			double angTween = Math.acos(Pt3D.Dot(uVec,toPlasmidUVec));
			double torsionMag;
			if (Env.plasmidTorqSpring.isActive()) {
				torsionMag = Env.plasmidTorqSpring.getValue()*angTween;
			} else {
				torsionMag = Env.fracMoveTorq.getValue()*angTween*bRotGam.y/Env.deltaT.getValue();
			}
			torsionVec.scale(torsionMag);
			incTorqueSum(torsionVec);	
			torsionVec.scale(-1);
			end1Plasmid.incTorqueSum(torsionVec);
			
			// register strainDist and check for filament detachment... several options
			end1ToPlasStrain.registerValue(strainDist);
			boolean removeTether = false;
			if ((Env.maxPlasTetherStrainDist.isActive()) & (end1ToPlasStrain.averageVal() > Env.maxPlasTetherStrainDist.getValue())) { 	// if strain greater than max allowable
				removeTether = true; 
			}
			if ((Env.plasTetherDetachRate.isActive()) & (Env.mtRNG.raw() < Env.plasTetherDetachRate.getValue()*Env.deltaT.getValue())) {
				removeTether = true;
			}
			double bCTave = end1Plasmid.bugCollidingTrack.averageVal();
			//if (bCTave > 0) { talkln ("bugCollidingTrack.averageVal() = " + bCTave); }
			if ((Env.plasCompressDetachFrac.isActive()) & (bCTave > Env.plasCompressDetachFrac.getValue())) {
				removeTether = true;
			}
			if (removeTether) {
				//talkln ("removing plasmid tether at end1");
				plasmidAtEnd1= false;
				end1Plasmid.filamentOff();
				end1Plasmid=null;
				end1PAttachPt.zero();
				end1DetachCounter = 10000;
				setGlobalEnd1Plasmid(false);
			}
		
		}
		if (plasmidAtEnd2) {
			end2Plasmid.registerFilament(this);
			if (Env.parRparCMoves.isActive()) {
				// reassign attachpt...
				end2PAttachPt.sub(end2,end2Plasmid.coord);
				end2PAttachPt.unitVec();
				end2PAttachPt.scale(end2Plasmid.getRadius());
				end2PAttachPt.XTox(end2Plasmid);
			}
			
			end2PAttachPtInX.xToXPlusxOrigin(end2Plasmid,end2PAttachPt);
			double strainDist = Pt3D.ptDist(end2PAttachPtInX,end2);
			double forceMag = Env.fracMove.getValue()*1.0e-6*strainDist/((1/bTransGam.x + 1/end2Plasmid.bTransGam.x)*Env.deltaT.getValue());
			//double forceMag=Env.kParM_PlasmidSpring*(Pt3D.ptDist(end2Plasmid.coord,end2));
			toPlasmidUVec.unitVec(end2PAttachPtInX,end2);
			F.scale(forceMag,toPlasmidUVec);
			incForceSum(F);
			R.scale(1e-6*(0.5*length),uVec);
			double axialF = Pt3D.Dot(uVec,F);  // axial force contribution
			incEnd2AxialForce(axialF);  
			RcrossF.cross(R,F);
			incTorqueSum(RcrossF);
			
			Fopp.scale(-1,F);
			end2Plasmid.incForceSum(Fopp);
			R.sub(1e-6,end2PAttachPtInX,end2Plasmid.coord);
			RcrossF.cross(R,Fopp);
			end2Plasmid.incTorqueSum(RcrossF);
			
			/*// steric hindrance from plasmid
			double toEnd2Dist = Pt3D.ptDist(end2Plasmid.coord, end2);
			if (toEnd2Dist < (end2Plasmid.getRadius()-halfmono)) { end2TipC = 0; }
			*/
			
			//	torsional spring between plasmid and segment
			toPlasmidUVec.unitVec(end2,end2Plasmid.coord);	// the unit vector orthogonal to the plasmid at the attachment point
			torsionVec.cross(uVecR,toPlasmidUVec);
			torsionVec.unitVec();
			double angTween = Math.acos(Pt3D.Dot(uVecR,toPlasmidUVec));
			double torsionMag;
			if (Env.plasmidTorqSpring.isActive()) {
				torsionMag = Env.plasmidTorqSpring.getValue()*angTween;
			} else {
				torsionMag = -Env.fracMoveTorq.getValue()*angTween*bRotGam.y/Env.deltaT.getValue();
			}
			torsionVec.scale(torsionMag);
			incTorqueSum(torsionVec);	
			torsionVec.scale(-1);
			end2Plasmid.incTorqueSum(torsionVec);
			
			// register strainDist and check for filament detachment
			end2ToPlasStrain.registerValue(strainDist);
			boolean removeTether = false;
			if ((Env.maxPlasTetherStrainDist.isActive()) & (end2ToPlasStrain.averageVal() > Env.maxPlasTetherStrainDist.getValue())) { 	// if strain greater than max allowable
				removeTether = true; 
			}
			if ((Env.plasTetherDetachRate.isActive()) & (Env.mtRNG.raw() < Env.plasTetherDetachRate.getValue()*Env.deltaT.getValue())) {
				removeTether = true;
			} 
			if ((Env.plasCompressDetachFrac.isActive()) & (end2Plasmid.bugCollidingTrack.averageVal() > Env.plasCompressDetachFrac.getValue())) {
				removeTether = true;
			}
			if (removeTether) {
				//talkln ("removing plasmid tether at end2");
				plasmidAtEnd2= false;
				end2Plasmid.filamentOff();
				end2Plasmid=null;
				end2PAttachPt.zero();
				end2DetachCounter = 10000;
				setGlobalEnd2Plasmid(false);
			}
			
		}
	}
	
	public void setGlobalEnd1Plasmid (boolean onState) {
		if (true) { return; }
		FilSegment curSeg = this;
		while (curSeg.filAtEnd2) {
			curSeg.globalPlasAtEnd1 = onState;
			curSeg = curSeg.end2Fil;
		}
		curSeg.globalPlasAtEnd1 = onState; // get last one
	}
	
	public void setGlobalEnd2Plasmid (boolean onState) {
		if (true) { return; }
		FilSegment curSeg = this;
		while (curSeg.filAtEnd1) {
			curSeg.globalPlasAtEnd2 = onState;
			curSeg = curSeg.end1Fil;
		}
		curSeg.globalPlasAtEnd2 = onState; // get end filSegment too
	}
	
	public int capLengthEnd1 () {
		int monsATP = 0;
		Monomer curMon = minusMon;
		while (curMon.isATP() && curMon != Monomer.plusGhost) {
			monsATP++;
			curMon = curMon.frontMon;
		}
		return monsATP;
	}
	
	public int capLengthEnd2 () {
		int monsATP = 0;
		Monomer curMon = plusMon;
		while (curMon.isATP() && curMon != Monomer.minusGhost) {
			monsATP++;
			curMon = curMon.backMon;
		}
		return monsATP;
	}
	
	public boolean capConditionOKEnd1 () {	
		if (removeMe) { return false; }
		int numMonsToCk = 0;
		if (plasmidAtEnd1) {
			if (!Env.capNumEnd1WithPlas.isActive()) { return true; } // plasmid bound with no cap condition
			numMonsToCk = Env.capNumEnd1WithPlas.getIntValue();
		} else {
			if (!Env.capNumEnd1.isActive()) { return true; } // free with no cap condition
			numMonsToCk = Env.capNumEnd1.getIntValue();
		}
		if (monomerCt < numMonsToCk) { numMonsToCk = monomerCt; }
		Monomer curMon = minusMon;
		for (int i=0;i<numMonsToCk;i++) {
			if (!curMon.isATP()) { return false; }
			curMon = curMon.frontMon;
		}
		return true;
	}
	
	public boolean capConditionOKEnd2 () {
		if (removeMe) { return false; }
		int numMonsToCk = 0;
		if (plasmidAtEnd2) {
			if (!Env.capNumEnd2WithPlas.isActive()) { return true; } // plasmid bound with no cap condition
			numMonsToCk = Env.capNumEnd2WithPlas.getIntValue();
		} else {
			if (!Env.capNumEnd2.isActive()) { return true; } // free with no cap condition
			numMonsToCk = Env.capNumEnd2.getIntValue();
		}
		if (monomerCt < numMonsToCk) { numMonsToCk = monomerCt; }
		Monomer curMon = plusMon;
		for (int i=0;i<numMonsToCk;i++) {
			if (!curMon.isATP()) { return false; }
			curMon = curMon.backMon;
		}
		return true;
	}
	
/*	public void criteriaEnd1Catastrophy () {
		double nucleotiderepartition = 0;
		Monomer curMon = minusMon;
		if (Env.criteriaLength1.getIntValue()<monomerCt){
			for (int i=0;i<Env.criteriaLength1.getIntValue();i++) {
				if (curMon.isADP()){
					nucleotiderepartition++;	
				}
				curMon=curMon.frontMon;	
			}
			if ((nucleotiderepartition/Env.criteriaLength1.getValue())>Env.criteriaRatio1.getValue()){ 
				end1Catastrophy = true;
				setEnd1CatastrophyAppearance();
			}
		} else {
			for (int i=0;i<monomerCt;i++) {
				if (curMon.isADP()){
					nucleotiderepartition++;	
				}
				curMon=curMon.frontMon;	
			}
			if ((nucleotiderepartition/monomerCt)>Env.criteriaRatio1.getValue()){ 
				end1Catastrophy = true;
				setEnd1CatastrophyAppearance();
			}
		}
	}
	
	public void criteriaEnd2Catastrophy() {
		double nucleotiderepartition = 0;
		Monomer curMon= plusMon;
		if (Env.criteriaLength2.getValue()<monomerCt){
			for (int i = 0; i<Env.criteriaLength2.getValue(); i++) {
				if (curMon.isADP()){
					nucleotiderepartition++;	
					}
				curMon=curMon.backMon;	
			}
			if ((nucleotiderepartition/Env.criteriaLength2.getValue())>Env.criteriaRatio2.getValue()){ 
				end2Catastrophy = true;
				setEnd2CatastrophyAppearance();
			}
		} else {
			for (int i = 0; i<monomerCt; i++) {
				if (curMon.isADP()){
					nucleotiderepartition++;	
					}
				curMon=curMon.backMon;	
			}
			if ((nucleotiderepartition/monomerCt)>Env.criteriaRatio2.getValue()){ 
				end2Catastrophy = true;
				setEnd2CatastrophyAppearance();
			}
		}
	}
	*/
	
	public void addBugForces (CollisionEvent X,Pt3D End){
		R.sub(End,coord); 		// define vector from center of filament out to the endpoint
		R.scale(1e-6);				// make units meters
		double RxFuVecSqrd = Pt3D.CrossMagSqrd(R,X.forceUVec);
		fturn = (1e-6*X.delta*bRotGam.y/(RxFuVecSqrd*Env.deltaT.getValue()));
		ftrans= (1e-6*X.delta*bTransGam.x/Env.deltaT.getValue());
		fnorm=Math.min(fturn,ftrans);
		Fcoll.scale(fnorm,X.forceUVec);
		Tcoll.cross(R,Fcoll);
		incForceSum(Fcoll);
		incTorqueSum(Tcoll);
		
		// axial force contribution
		if (End == end1) { 
			incEnd1AxialForce(Pt3D.Dot(uVecR,Fcoll)); 
			end1TipC = 0;
		}
		if (End == end2) { incEnd2AxialForce(Pt3D.Dot(uVec,Fcoll)); 
			end2TipC = 0;

		}
	}
	
	public boolean addMonomerSim (double onRate){
		if (Env.mtRNG.raw()< onRate*theBug.getParMConc()*Env.biochemDeltaT.getValue()){
			//talkln ("adding monomer, conc = " + theBug.getParMConc());
			monomerCt++;
			length+=halfmono;
			theBug.takeParM(1);
			lengthChanged = true;
			return true;
		}
		return false;
	}
	
	public boolean addNonHydroMonomerSim (double onRate){
		if (Env.mtRNG.raw()< onRate*theBug.getNonHydroParMConc()*Env.biochemDeltaT.getValue()){
			monomerCt++;
			length+=halfmono;
			theBug.takeNonHydroParM(1);
			lengthChanged = true;
			return true;
		}
		return false;
	}
	
	public boolean removeMonomerSim (double offRate, Monomer endMon) {
		if (Env.mtRNG.raw()< offRate*Env.biochemDeltaT.getValue()){
			monomerCt--;
			length+= -halfmono;
			if (endMon.hydrolyzable) {
				theBug.putParM(1);
			} else {
				theBug.putNonHydroParM(1);
			}
			lengthChanged = true;
			return true;
		}
		return false;
	}
	
	public static void linkEnd1Plasmid (FilSegment fil, Plasmid plas) {
		fil.plasmidAtEnd1 = true;
		fil.end1Plasmid = plas;
		plas.filamentOn();
	}
	
	public static void linkEnd2Plasmid (FilSegment fil, Plasmid plas) {
		fil.plasmidAtEnd2 = true;
		fil.end2Plasmid = plas;
		plas.filamentOn();
	}
	
	public static void link (FilSegment at1, FilSegment at2) {
		at1.setEnd2Links(at2,true);
		at2.setEnd1Links(at1, true);
	}
	
	public void setEnd1Links (FilSegment at1, boolean normOrientation) {
		filAtEnd1 = true;
		end1Fil = at1;
		if (normOrientation) { ptAtEnd1 = at1.end2; } else { ptAtEnd1 = at1.end1; }
	}
	
	public void setEnd2Links (FilSegment at2, boolean normOrientation) {
		filAtEnd2 = true;
		end2Fil = at2;
		if (normOrientation) { ptAtEnd2 = at2.end1; } else { ptAtEnd2 = at2.end2; }
	}
	
	public void removeEnd1Links() {
		filAtEnd1 = false;
		end1Fil = null;
		ptAtEnd1 = null;
	}
	
	public void removeEnd2Links() {
		filAtEnd2 = false;
		end2Fil = null;
		ptAtEnd2 = null;
	}
	
	public static void transferEnd1Plasmid (FilSegment givesP, FilSegment getsP) {
		getsP.plasmidAtEnd1 = true;
		getsP.end1Plasmid = givesP.end1Plasmid;
		getsP.end1PAttachPt.copy(givesP.end1PAttachPt);
		givesP.plasmidAtEnd1=false;
		givesP.end1Plasmid=null;
	}
	
	public static void transferEnd2Plasmid (FilSegment givesP, FilSegment getsP) {
		getsP.plasmidAtEnd2 = true;
		getsP.end2Plasmid = givesP.end2Plasmid;
		getsP.end2PAttachPt.copy(givesP.end2PAttachPt);
		givesP.plasmidAtEnd2=false;
		givesP.end2Plasmid=null;
	}
		
	public static void cleanup (FilSegment cleanF, boolean swapOut) {
			putMonsBack(cleanF);
			
			if (!cleanF.filAtEnd1 && !cleanF.filAtEnd2) { filCt--; }		// only decrement filament counter if unattached to other FilSegments
		
			transferPlasmidsAtCleanup(cleanF);
			
			joiningCleanup(cleanF);
			
			cleanF.breakAtEnd1();
			cleanF.breakAtEnd2();
			
			finalDepoly (cleanF);
			
			if (Env.biochemOnly.isActive()) { putOpenY(cleanF.coord.y); }
			
			if (swapOut) { removeFilSegment(cleanF); }
	}
	
	public static void putMonsBack (FilSegment cleanF) {
		Monomer curMon = cleanF.minusMon;
		while (curMon != null && curMon != Monomer.plusGhost) {
			if (curMon.hydrolyzable) { theBug.putParM(1); } else { theBug.putNonHydroParM(1); }
			curMon = curMon.frontMon;
		}
	}
	
	public static void transferPlasmidsAtCleanup (FilSegment cleanF) {
		if (cleanF.plasmidAtEnd1) {
			if (cleanF.filAtEnd2) {
				transferEnd1Plasmid(cleanF,cleanF.end2Fil);
			} else {
				cleanF.end1Plasmid.filamentOff(); 
			}
		}
		if (cleanF.plasmidAtEnd2) { 
			if (cleanF.filAtEnd1) {
				transferEnd2Plasmid(cleanF,cleanF.end1Fil);
			} else {
				cleanF.end2Plasmid.filamentOff(); 
			}
		}
	}
	
	public static void joiningCleanup(FilSegment cleanF) {
		// majorly messy code for a joining event... swapping links around for four possible arrangements
		if (cleanF.filAtEnd1 & cleanF.filAtEnd2) {		// joining event.. 
			talkln ("joining event.. setting links");
			if (cleanF.ptAtEnd1 == cleanF.end1Fil.end2) {
				cleanF.end1Fil.filAtEnd2 = true;
				cleanF.end1Fil.end2Fil = cleanF.end2Fil;
				cleanF.end1Fil.ptAtEnd2 = cleanF.ptAtEnd2;
			} else {
				cleanF.end1Fil.filAtEnd1 = true;
				cleanF.end1Fil.end1Fil = cleanF.end2Fil;
				cleanF.end1Fil.ptAtEnd1 = cleanF.ptAtEnd2;
			}
			
			if (cleanF.ptAtEnd2 == cleanF.end2Fil.end1) {
				cleanF.end2Fil.filAtEnd1 = true;
				cleanF.end2Fil.end1Fil = cleanF.end1Fil;
				cleanF.end2Fil.ptAtEnd1 = cleanF.ptAtEnd1;
			} else {
				cleanF.end2Fil.filAtEnd2 = true;
				cleanF.end2Fil.end2Fil = cleanF.end1Fil;
				cleanF.end2Fil.ptAtEnd2 = cleanF.ptAtEnd1;
			}
				
			// remove cleanF links
			cleanF.filAtEnd1 = false;
			cleanF.end1Fil = null;
			cleanF.ptAtEnd1 = null;
			cleanF.filAtEnd2 = false;
			cleanF.end2Fil = null;
			cleanF.ptAtEnd2 = null;
		}
	}
	
	public static void finalDepoly (FilSegment cleanF) {
		if (cleanF.monomerCt > 0) {
			Monomer curMon = cleanF.minusMon;
			Monomer tempMon;
			while (curMon != Monomer.plusGhost) {
				tempMon = curMon.frontMon;
				curMon.depolymerize(cleanF,Monomer.MINUSEND);
				curMon = tempMon;
			}
		}
	}
	
	public static void removeAll () {
		for (int i=0;i<filSegmentCt;i++) {
			cleanup(theFilSegments[i],false);
			theFilSegments[i].G.detach();
			theFilSegments[i].end1BG.detach();
			theFilSegments[i].end2BG.detach();
			theFilSegments[i].coordBG.detach();
			theFilSegments[i].coordSysG.detach();
		}
		for (int i=0;i<filSegmentCt;i++) {
			theFilSegments[i].removeMe = true;
			theFilSegments[i] = null;
		}
		filSegmentCt = 0;	// reset array counter
		filCt = 0;	// reset unique id counter
	}
	
	public boolean inCompression(String endString) {
		if (Env.biochemOnly.isActive()) { return false; }
		if (Env.compressionCritOff) { return false; }
		
		// to match the experimental observation of growth rates same with or without plasmid, ignore this poly. crit unless colliding with bug
		if (plasmidAtEnd1) { if (!end1Plasmid.collidedWithBugThisStep) { return false; } }
		if (plasmidAtEnd2) { if (!end2Plasmid.collidedWithBugThisStep) { return false; } }
		
		if (compressionTrack.runningAverageVal() < Env.polyCompressionCutoff.getValue()) { 
			//talkln ("Yes Compression" + endString);
			return true;
		}
		//talkln ("No Compression" + endString);
		return false;
	}
	
	public void setCompression() {
		if (end1AxialF < 0 & end2AxialF < 0) {
			if (end1AxialF > end2AxialF) { 
				compressionTrack.registerValue(end1AxialF); 
			} else {
				compressionTrack.registerValue(end2AxialF); 
			}
		} else {
			compressionTrack.registerValue(0);
		}
	}
	
	public boolean stericHindranceEnd1() {
		if (Env.biochemOnly.isActive()) { return false; }
		if (end1TipC < halfmono) return true;
		return false;
	}
	
	public boolean stericHindranceEnd2() {
		if (Env.biochemOnly.isActive()) { return false; }
		if (end2TipC < halfmono) return true;
		return false;
	}
	
	public void incEnd1AxialForce (double incF) {
		end1AxialF += incF;
	}
	
	public void incEnd2AxialForce (double incF) {
		end2AxialF += incF;
	}
	
	public static void makeInitialFilaments () {
		for (int i=0;i<Env.initialFilaments.getValue();i++) { 
			if (!Env.biochemOnly.isActive()) {
				makeRandomFilament(300); 
			} else {
				makeOrderedFilament();
			}
		}
	}
	
	public static void resetOrderedY () {
		openYCt = 0;
		positiveY = true;
		currentPosY = 0.5*incY;
		currentNegY = -0.5*incY;
	}
	
	public static double getOrderedY () {
		if (openYCt > 0) {
			return getOpenY();
		} else {
			double yVal = currentPosY;
			if (positiveY) {
				currentPosY += incY;
			} else {
				yVal = currentNegY;
				currentNegY += -incY;
			}
			positiveY = !positiveY;
			return yVal;
		}
	}
	
	public static double getOpenY () {
		openYCt--;
		return openY[openYCt];
	}
	
	public static void putOpenY (double yVal) {
		openY[openYCt] = yVal;
		openYCt++;
	}
	
	public static void makeOrderedFilament () {
		double yVal = getOrderedY();
		Pt3D orderedLoc = new Pt3D(0,yVal,0);
		Pt3D orderedAng = new Pt3D(1,0,0);
		new FilSegment (orderedLoc,orderedAng,-1,Env.parMSeed.getIntValue(),false);
	}
	
	public static void makeRandomFilament () {
		double nucLength = (Env.parMSeed.getIntValue()+1)*Env.parMHalfMonoSize;
		Pt3D randomLoc = theBug.rdmPtInsideBug(nucLength/2);
		Pt3D randomUVec = Pt3D.RandomUnitVec();
		new FilSegment (randomLoc,randomUVec,-1);
	}
	public static void makeRandomFilament (int monCt) {
		double filLength = (monCt+1)*Env.parMHalfMonoSize;
		Pt3D randomLoc = theBug.rdmPtInsideBug(filLength/2);
		Pt3D randomOrient = Pt3D.RandomUnitVec();
		new FilSegment (randomLoc,randomOrient,-1,monCt,false);
	}
	
	public static void makeTestFilament () {
		double filLength = 2*(Env.tstPlasmidOffset-Env.plasmidRadius.getValue());
		int monCt = (int)(filLength/Env.parMHalfMonoSize);
		Pt3D loc = new Pt3D(0,0,0);
		Pt3D ang = new Pt3D(1,0,0);
		FilSegment newSeg = new FilSegment (loc,ang,-1,monCt,false);
	}
	
	public static void makeLinkTstFils() {
		double filLength = 2*(Env.tstPlasmidOffset-Env.plasmidRadius.getValue());
		int monCt = (int)(filLength/Env.parMHalfMonoSize);
		Pt3D loc = new Pt3D(0,0,0);
		Pt3D ang = new Pt3D(1,0,0);
		new FilSegment (loc,ang,-1,monCt,false);
		
		filLength = 2*(Env.tstPlasmidOffset-Env.plasmidRadius.getValue());
		monCt = (int)(filLength/Env.parMHalfMonoSize);
		loc = new Pt3D(0,0,0);
		ang = new Pt3D(1,2,0);
		ang.unitVec();
		new FilSegment (loc,ang,-1,monCt,false);
	}
	
	public static void annealTst() {
		double filLength = 2*(Env.tstPlasmidOffset-Env.plasmidRadius.getValue());
		int monCt = (int)(filLength/Env.parMHalfMonoSize);
		Pt3D loc = new Pt3D(-5*Env.parMMonoSize,0,0);
		Pt3D ang = new Pt3D(-1,0,0);
		FilSegment fil1 = new FilSegment (loc,ang,-1,monCt,false);
		
		monCt = (int)(filLength/Env.parMHalfMonoSize);
		loc = new Pt3D(filLength-10*Env.parMMonoSize,0,0);
		ang = new Pt3D(1,0,0);
		ang.unitVec();
		FilSegment fil2 = new FilSegment (loc,ang,-1,monCt,false);
	}
	
	public static void crossedFilsTst() {
		double filLength = 2*(Env.tstPlasmidOffset-Env.plasmidRadius.getValue());
		int monCt = (int)(filLength/Env.parMHalfMonoSize);
		Pt3D loc = new Pt3D(-5*Env.parMMonoSize,0,0);
		Pt3D ang = new Pt3D(1,-.9,0);
		FilSegment fil1 = new FilSegment (loc,ang,-1,monCt,false);
		
		monCt = (int)(filLength/Env.parMHalfMonoSize);
		loc = new Pt3D(0,0,0);
		ang = new Pt3D(-1,-.9,0);
		ang.unitVec();
		FilSegment fil2 = new FilSegment (loc,ang,-1,monCt,false);
	}
	
	public static void plasmidsCrossedFilsTst() {
		double filLength = 3*(Env.tstPlasmidOffset-Env.plasmidRadius.getValue())+0.05;
		int monCt = (int)(filLength/Env.parMHalfMonoSize);
		Pt3D loc = new Pt3D(-0.25*filLength,0,0);
		Pt3D ang = new Pt3D(1,.4,0);
		FilSegment fil1 = new FilSegment (loc,ang,-1,monCt,false);
		
		monCt = (int)(filLength/Env.parMHalfMonoSize);
		loc = new Pt3D(0.25*filLength,0,0);
		ang = new Pt3D(-1,.4,0);
		ang.unitVec();
		FilSegment fil2 = new FilSegment (loc,ang,-1,monCt,false);
		
	
		Pt3D p1Loc = Pt3D.Add(fil1.end1,0.40*Env.plasmidRadius.getValue(),fil1.uVecR);
		new Plasmid (p1Loc);
		
		Pt3D p2Loc = Pt3D.Add(fil2.end1,0.40*Env.plasmidRadius.getValue(),fil2.uVecR);
		new Plasmid (p2Loc);
		
	}
	
	public static void plasmidsSingleFilTst() {
		double filLength = 0.95*Env.bugLength.getValue() - 4*Env.plasmidRadius.getValue();
		//double filLength = 2*(Env.tstPlasmidOffset-Env.plasmidRadius.getValue());
		int monCt = (int)(filLength/Env.parMHalfMonoSize);
		Pt3D loc = new Pt3D(0,0,0);
		Pt3D ang = new Pt3D(1,0,0);
		Pt3D rAng = new Pt3D(-1,0,0);
		FilSegment fil1 = new FilSegment (loc,ang,-1,monCt,false);
		
		// additional filaments
		//new FilSegment (loc,rAng,-1,monCt,false);
		//new FilSegment (loc,ang,-1,monCt,false);
	
		Pt3D p1Loc = Pt3D.Add(fil1.end1,Env.plasmidRadius.getValue(),fil1.uVecR);
		new Plasmid (p1Loc);
		
		Pt3D p2Loc = Pt3D.Add(fil1.end2,Env.plasmidRadius.getValue(),fil1.uVec);
		new Plasmid (p2Loc);
		
	}
	
	public static void persistenceLengthTst() {
		Env.testingPersistenceLength = true;
		Env.resetPersistenceTstValues();
		Env.bugOff = true;
		Env.plasmidBrownianMotionOff = true;
		Env.brownianFilMotionOff = false;
		int monCt = Env.persistenceSegNumber*Env.stdSegLength.getIntValue();
		Pt3D loc = new Pt3D(0,0,0);
		Pt3D ang = new Pt3D(1,0,0);
		FilSegment lpFil = new FilSegment (loc,ang,-1,monCt,false);
	
		loc.y = -1;
		new FilSegment (loc,ang,-1,monCt,false);
		
		// spit out expected diffusivities for rigid rod
		double lpFilLengthM = 1.0e-6*lpFil.length; // in meters
		double radiusM = radius*1.0e-6;
		double denomLogTerm = Math.log(lpFilLengthM/(2*radiusM));	//dimensionless
		double xDiff = (Env.Boltz*Env.tempK)/(2*Math.PI*Env.aeta.getValue()*lpFilLengthM)/(denomLogTerm + aParallel);
		double yDiff = (Env.Boltz*Env.tempK)/(4*Math.PI*Env.aeta.getValue()*lpFilLengthM)/(denomLogTerm + aOrthog);
		FileOps.talkln ("Expected diffusivities (if a rigid rod): " + xDiff + "," + yDiff + "," + yDiff);
		
	}
	
	public static void cantileverElasticityTuning() {
		Env.cantileverElasticityTst = true;
		Env.relaxWriteInterval = (int)3e3;
		Env.resetElasticityTstValues();
		Env.bugOff = true;
		Env.plasmidBrownianMotionOff = true;
		Env.brownianFilMotionOff = true;
		int monCt = (int)(Env.tstFilLength/Env.parMHalfMonoSize)-1;
		Pt3D loc = new Pt3D(0,0,0);
		Pt3D ang = new Pt3D(1,0,0);
		FilSegment fil1 = new FilSegment (loc,ang,-1,monCt,false);
		
		//more fils
		//new FilSegment (loc,ang,-1,monCt,false);
		//new FilSegment (loc,ang,-1,monCt,false);
	
		Pt3D p1Loc = Pt3D.Add(fil1.end1,0.95*Env.plasmidRadius.getValue(),fil1.uVecR);
		new Plasmid (p1Loc);
		
	}
	
	public static void simpleSupportElasticityTuning() {
		Env.simpleSupportElasticityTst = true;
		Env.bugOff = true;
		Env.plasmidBrownianMotionOff = true;
		Env.brownianFilMotionOff = true;
		int monCt = (int)(Env.tstFilLength/Env.parMHalfMonoSize)-1;
		Pt3D loc = new Pt3D(0,0,0);
		Pt3D ang = new Pt3D(1,0,0);
		FilSegment fil1 = new FilSegment (loc,ang,-1,monCt,false);
		
		//more fils
		//new FilSegment (loc,ang,-1,monCt,false);
		//new FilSegment (loc,ang,-1,monCt,false);
	
		Pt3D p1Loc = Pt3D.Add(fil1.end1,0.95*Env.plasmidRadius.getValue(),fil1.uVecR);
		new Plasmid (p1Loc);
		
		Pt3D p2Loc = Pt3D.Add(fil1.end2,0.95*Env.plasmidRadius.getValue(),fil1.uVec);
		new Plasmid (p2Loc);
		
	}
	
	public static void addCantileverTestForce (FilSegment curFil) {
		Env.plasmidBrownianMotionOff = true;
		Env.brownianFilMotionOff = true;
		Env.currentDeflection = curFil.end2.y;
		if (!Env.relaxCantilever) { 
			curFil.forceSum.add(Env.staticForce); 
		} else {
			if (Env.relaxWriteCounter >= Env.relaxWriteInterval) {
				FileOps.writeToRelaxFile();
				Env.relaxWriteCounter = 0;
			} else {
				Env.relaxWriteCounter++;
			}
		}
	}
	
	public static void addSimpleSupportTestForce1stMode (FilSegment curFil) {
		Env.plasmidBrownianMotionOff = true;
		Env.brownianFilMotionOff = true;
		Env.plasmidTorqSpring.setValue(0);
		if (Env.relaxSimpleSupport | Env.simulationTime > 100*Env.deltaT.getValue()) {
			FilSegment end1Seg = FilSegment.theFilSegments[0];
			FilSegment end2Seg = FilSegment.theFilSegments[Env.curMultipleOfSeg-1];
			double onLineY = (end1Seg.coord.y + end2Seg.coord.y)/2;
			Env.currentDeflection = curFil.coord.y-onLineY;
		}
		if (!Env.relaxSimpleSupport) { 
			Env.reportedTimeConstant = false;
			curFil.forceSum.add(Env.staticForce);
		} else {
			if (!Env.reportedTimeConstant & Env.currentDeflection > Env.relaxationDeflection) {
				Env.reportedTimeConstant = true;
				talkln ("tau = " + Env.simulationTime);
			}
			if (Env.relaxWriteCounter >= Env.relaxWriteInterval) {
				//FileOps.writeToRelaxFile();
				Env.relaxWriteCounter = 0;
			} else {
				Env.relaxWriteCounter++;
			}
		}

	}
	
	public static void addSimpleSupportTestForce2ndMode (FilSegment curFil) {
		Env.plasmidBrownianMotionOff = true;
		Env.brownianFilMotionOff = true;
		Env.plasmidTorqSpring.setValue(0);
		if (Env.relaxSimpleSupport | Env.simulationTime > 100*Env.deltaT.getValue()) {
			FilSegment end1Seg = FilSegment.theFilSegments[0];
			FilSegment end2Seg = FilSegment.theFilSegments[Env.curMultipleOfSeg-1];
			FilSegment forceSeg1Seg = FilSegment.theFilSegments[Env.forceSeg1];
			double endToEndSlope = (end2Seg.end2.y - end1Seg.end1.y)/(end2Seg.end2.x-end1Seg.end1.x);
			double onLineY = end1Seg.end1.y + (forceSeg1Seg.coord.x - end1Seg.end1.x)*endToEndSlope;
			Env.currentDeflection = forceSeg1Seg.coord.y - onLineY;
		}
		if (!Env.relaxSimpleSupport) { 
			if (curFil.filArrayPos == Env.forceSeg1) { curFil.forceSum.add(Env.staticForce); } 
			if (curFil.filArrayPos == Env.forceSeg2) { curFil.forceSum.add(Env.staticForceR); }
		} else {
			if (Env.relaxWriteCounter >= Env.relaxWriteInterval) {
				FileOps.writeToRelaxFile();
				Env.relaxWriteCounter = 0;
			} else {
				Env.relaxWriteCounter++;
			}
		}
		
	}
	
	public static void simpleSupportRelaxActions () { 
		Env.simulationTime = 0;
		Env.maxPlasTetherStrainDist.setActive(Env.relaxSimpleSupport);
	}
			
	
	public static void persistenceTrack () {
		// end1 segment
		FilSegment end1Seg = FilSegment.theFilSegments[0];
		
		// cycle through filsegments in link order to consistently enter their angle info
		FilSegment curFil = end1Seg;
		double dotUVecs;
		for (int i=0;i<Env.persistenceSegNumber;i++) {
			dotUVecs = Pt3D.Dot(end1Seg.uVec, curFil.uVec);
			Env.persistenceTrack[i].registerValue(dotUVecs);
			if (curFil.end2Fil != null) { curFil = curFil.end2Fil; }
		}
		
		if (Env.logFiles) {
			FileOps.writeToPersistenceLengthFile();
		} else {
			// write out the current average values
			talk ("simTime = " + Env.simulationTime + ";");
			for (int i=0;i<Env.persistenceSegNumber;i++) {
				talk (Env.persistenceTrack[i].runningAverageVal() + ";");
			}
			talkln("");
		}
		
	}
	
	public static void transDiffTrack () {
		// only track position of center segment coord
		FilSegment cenFilSeg = FilSegment.theFilSegments[Env.curMultipleOfSeg/2+1];
		Pt3D curPos = new Pt3D();
		curPos.scale(1e-6,cenFilSeg.coord);
		Pt3D posDelta = Pt3D.Sub(curPos,Env.lastPos);
		posDelta.XTox(cenFilSeg);		// translate motion into body-fixed axes of cenFilSeg
		
		Env.transDiffTrack.registerValue(posDelta);
		Env.lastPos.copy(curPos);
		
		if (Env.logFiles) {
			FileOps.writeToDiffusivityFile();
		} else {
			// calculate D = variance/2*deltaT
			Pt3D curDPt = Env.transDiffTrack.varianceOfPt();
			curDPt.scale(1/(2*Env.ckPersistenceInterval*Env.deltaT.getValue()));
			talk ("Apparent x-diffusivity for " + Env.persistenceSegNumber + " segments;");
			talk (curDPt.x + ";" + curDPt.y + ";" + curDPt.z + ";");
			talkln("");
			talkln("");
		}
	}
	
	/*public static void transDiffTrack () {
		// find centroid of filament segments
		Pt3D curPos = new Pt3D();
		Pt3D posDeltaInx = new Pt3D();
		Pt3D curXVec = new Pt3D();
		Pt3D curYVec = Pt3D.RandomUnitVec();
		Pt3D curZVec = new Pt3D();
		for (int i=0;i<Env.persistenceSegNumber;i++) {
			curPos.inc(FilSegment.theFilSegments[i].coord);
			curXVec.inc(FilSegment.theFilSegments[i].uVec);
			//curYVec.inc(FilSegment.theFilSegments[i].yVec);
			//curZVec.inc(FilSegment.theFilSegments[i].zVec);
		}
		curPos.scale(1e-6/Env.persistenceSegNumber);
		curXVec.unitVec();
		curZVec.cross(curXVec,curYVec);
		curYVec.cross(curZVec,curXVec);
		Pt3D posDelta = Pt3D.Sub(curPos, Env.lastPos);
		// transform posDelta to averaged coordinate system of flexible filament
		posDeltaInx.x = curXVec.x*posDelta.x + curXVec.y*posDelta.y + curXVec.z*posDelta.z;
		posDeltaInx.y = curYVec.x*posDelta.x + curYVec.y*posDelta.y + curYVec.z*posDelta.z;
		posDeltaInx.z = curZVec.x*posDelta.x + curZVec.y*posDelta.y + curZVec.z*posDelta.z;
		
		Env.transDiffTrack.registerValue(posDeltaInx);
		Env.lastPos.copy(curPos);
		
		if (Env.logFiles) {
			FileOps.writeToDiffusivityFile();
		} else {
			// calculate D = variance/2*deltaT
			Pt3D curDPt = Env.transDiffTrack.varianceOfPt();
			curDPt.scale(1/(2*Env.ckPersistenceInterval*Env.deltaT.getValue()));
			talk ("Apparent x-diffusivity for " + Env.persistenceSegNumber + " segments;");
			talk (curDPt.x + ";" + curDPt.y + ";" + curDPt.z + ";");
			talkln("");
			talkln("");
		}
	}*/
	
	public static void setFilamentID (FilSegment newFilSeg, int filID) {
		if (filID == -1) {
			newFilSeg.filID = filIDCt;
			filCt++;
			filIDCt++;
		} else {
			newFilSeg.filID = filID;
		}
	}
	
	public static void registerFilLifetime (FilSegment fil) {
		if (fil.bornTime >= Env.filLifetimeStart.getValue()) {
			double lifetime = Env.simulationTime - fil.bornTime;
			lifeTimes.addValue(lifetime);
		}
	}
	
	public static void addFilSegment (FilSegment newFilament) {
		theFilSegments[filSegmentCt] = newFilament;
		newFilament.filArrayPos = filSegmentCt;
		newFilament.bornTime = Env.simulationTime;
		filSegmentCt ++;
	}
	public static void removeFilSegment ( FilSegment byeFilament) {
		registerFilLifetime(byeFilament);
		int swapId = byeFilament.filArrayPos;
		theFilSegments[swapId] = theFilSegments[filSegmentCt-1];
		theFilSegments[swapId].filArrayPos = swapId;
		filSegmentCt --;
		byeFilament.removeMe = true;
	
	}
	public static void spawnRdmFilaments () {
		if (Env.mtRNG.raw() < (Env.kRdmNuc.getValue()*Math.pow(theBug.getParMConc()+theBug.getNonHydroParMConc(),Env.parMSeed.getIntValue())*Env.deltaT.getValue())/Bug.microMolarChangePerMonomer) {
			if (!Env.biochemOnly.isActive()) {
				makeRandomFilament();
			} else {
				makeOrderedFilament();
			}
		}
	}
	
	public static void spawnPlasmidFilaments () {
		Plasmid curP;
		for (int i=0;i<Plasmid.plasmidCt;i++) {
			curP = Plasmid.thePlasmids[i];
			if (curP.canBindFilament()) {
				if (Env.mtRNG.raw() < Env.kPlasmidNuc.getValue()*Env.deltaT.getValue()) {
					Pt3D rdmDir = Pt3D.RandomUnitVec();
					Pt3D newAttachPt = Pt3D.Scale(curP.getRadius(),rdmDir);
					newAttachPt.inc(curP.coord);
					FilSegment newFil = new FilSegment (newAttachPt,rdmDir,-1);
					newFil.translate(newFil.length/2,newFil.uVec);
					newFil.plasmidAtEnd1=true;
					newFil.end1Plasmid=curP;
					newFil.end1PAttachPt.XToxFromxOrigin(curP,newFil.end1);
					curP.filamentOn();
				}
			}
		}
		
	}
	
	public void setEnd1CatastrophyAppearance() {
		if (!graphicsMade) { makeGraphics(); }
		end1S.getShape().setAppearance(FilSegment.catasphApp);
	}
	
	public void setEnd2CatastrophyAppearance() {
		if (!graphicsMade) { makeGraphics(); }
		end2S.getShape().setAppearance(FilSegment.catasphApp);
	}
	
	public void setEnd1NoPlasmidAppearance() {
		if (!graphicsMade) { makeGraphics(); }
		end1S.getShape().setAppearance(FilSegment.noPlasmidApp);
	}
	
	public void setEnd2NoPlasmidAppearance() {
		if (!graphicsMade) { makeGraphics(); }
		end2S.getShape().setAppearance(FilSegment.noPlasmidApp);
	}
	
	public void setEnd1GotPlasmidAppearance() {
		if (!graphicsMade) { makeGraphics(); }
		end1S.getShape().setAppearance(FilSegment.gotPlasmidApp);
	}
	
	public void setEnd2GotPlasmidAppearance() {
		if (!graphicsMade) { makeGraphics(); }
		end2S.getShape().setAppearance(FilSegment.gotPlasmidApp);
	}
	
	public void ptFromHelixPos (Pt3D pt, double pos, double initAng) {
		double curAng = initAng + pos*Env.helixPitch;
		pt.x = pos;
		pt.y = Env.helixMonOffset*Math.cos(curAng);
		pt.z = Env.helixMonOffset*Math.sin(curAng);
	}
	
	public void resetGraphics () {
		Monomer curMon = minusMon;
		while (curMon != Monomer.plusGhost) {
			curMon.resetGraphics(this);
			curMon = curMon.frontMon;
		}
		/*if (Env.helixSpheres) {
			end1BG.detach();
			end2BG.detach();
		} else {
			G.addChild(end1BG);
			G.addChild(end2BG);
		}*/
			
	}
	
	public static void resetGraphicsAll () {
		for (int i=0;i<filSegmentCt;i++) {
			theFilSegments[i].resetGraphics();
		}
	}
	
	public int countLinkedMonomers () {
		int monCt = 0;
		Monomer curMon = minusMon;
		while (curMon != Monomer.plusGhost) {
			monCt++;
			curMon = curMon.frontMon;
		}
		return monCt;
	}
	
	public static double lengthSum () {
		double lengthSum = 0;
		for (int i=0;i<filSegmentCt;i++) {
			lengthSum += theFilSegments[i].length;
		}
		return lengthSum;
	}
	
	public static int linkedMonomerSum () {
		int monSum = 0;
		for (int i=0;i<filSegmentCt;i++) {
			monSum += theFilSegments[i].countLinkedMonomers();
		}
		return monSum;
	}
	
	public static int monomerSum () {
		int monSum = 0;
		for (int i=0;i<filSegmentCt;i++) {
			monSum += theFilSegments[i].monomerCt;
		}
		return monSum;
	}
	
	public void updateMonomerPositions () {
		boolean even = true;
		double oppAng = helixAng + Math.PI;
		boolean plasmidBound = Plasmid.filamentBoundToAnyPlasmid(filID);
		double pos = Env.parMHalfMonoSize;
		Monomer curMon = minusMon;
		while (curMon != Monomer.plusGhost) {
			if (curMon==null) { talkln ("skipping updateMonomerNoGraphics for " + String.valueOf(this) + " 'cause a monomer is null"); return; }
			if (curMon.removeMe) { talkln ("skipping updateMonomerNoGraphics for " + String.valueOf(this) + " 'cause a monomer is to be removed"); return; }
			
			curMon.plasmidBound = plasmidBound;
			if (even) {
				ptFromHelixPos(curMonStart,pos,helixAng);
				curMonStart.xToX(this);
				curMonStart.add(end1);
				curMon.updatePosition(curMonStart);
			} else {
				ptFromHelixPos(curMonStart,pos,oppAng);
				curMonStart.xToX(this);
				curMonStart.add(end1);
				curMon.updatePosition(curMonStart);
			}
			even = !even;
			pos += Env.parMHalfMonoSize;
			curMon = curMon.frontMon;
		}
	}
	
	public void updateMonomerGraphics () {
		boolean twoPoints = !Env.helixSpheres;
		boolean even = true;
		boolean plasmidBound = Plasmid.filamentBoundToAnyPlasmid(filID);
		double oppAng = helixAng + Math.PI;
		double pos = 0;
		if (!twoPoints) { pos += Env.parMHalfMonoSize; }		// if indicating sphere centers
		Monomer curMon = minusMon;
		while (curMon != Monomer.plusGhost) {
			if (curMon==null) { talkln ("skipping updateMonomerGraphics for " + String.valueOf(this) + " 'cause a monomer is null"); return; }
			if (curMon.removeMe) { talkln ("skipping updateMonomerGraphics for " + String.valueOf(this) + " 'cause a monomer is to be removed"); return; }
			
			curMon.plasmidBound = plasmidBound;
			if (even) {
				ptFromHelixPos(curMonStart,pos,helixAng);
				curMonStart.xToX(this);
				curMonStart.add(end1);
				if (twoPoints) {
					ptFromHelixPos(curMonStop,pos+Env.parMMonoSize,helixAng);
					curMonStop.xToX(this);
					curMonStop.add(end1);
				}
				curMon.updateGraphics(curMonStart,curMonStop);
			} else {
				ptFromHelixPos(curMonStart,pos,oppAng);
				curMonStart.xToX(this);
				curMonStart.add(end1);
				if (twoPoints) {

					ptFromHelixPos(curMonStop,pos+Env.parMMonoSize,oppAng);
					curMonStop.xToX(this);
					curMonStop.add(end1);
				}
				curMon.updateGraphics(curMonStart,curMonStop);
			}
			even = !even;
			pos += Env.parMHalfMonoSize;
			curMon = curMon.frontMon;
		}
	}

	public void makeGraphics () {
		if (!graphicsInitialized) {
			catasphApp.setTransparencyAttributes(tA);
			catasphApp.setMaterial(catasphMat);
			//noPlasmidApp.setTransparencyAttributes(tA);
			noPlasmidApp.setMaterial(noPlasmidMat);
			gotPlasmidApp.setTransparencyAttributes(tA);
			gotPlasmidApp.setMaterial(gotPlasmidMat);
			
			xLineApp.setColoringAttributes(xLineCA);
			yLineApp.setColoringAttributes(yLineCA);
			zLineApp.setColoringAttributes(zLineCA);
			graphicsInitialized = true;
		}
		/*// make material
		m = new Material (ambientC,emissiveC,diffuseC,specularC,shiny);
		m.setCapability(Material.ALLOW_COMPONENT_WRITE);
		*/
		// set capabilities
		setGraphicsCapabilities();
		
		// coordSphere
		coordS = new Sphere ((float)(2*Env.parMHalfMonoSize),Sphere.GENERATE_NORMALS, Env.parMEndSphereTessalation, noPlasmidApp);
		coordS.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		coordT3D = new Transform3D();
		coordTG = new TransformGroup(coordT3D);
		coordTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		coordTG.addChild(coordS);
		coordBG = new BranchGroup();
		coordBG.setCapability(BranchGroup.ALLOW_DETACH);
		coordBG.addChild(coordTG);
		
		// end1Sphere
		end1S = new Sphere ((float)(2*Env.parMHalfMonoSize),Sphere.GENERATE_NORMALS, Env.parMEndSphereTessalation, noPlasmidApp);
		end1S.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		end1T3D = new Transform3D();
		end1TG = new TransformGroup(end1T3D);
		end1TG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		end1TG.addChild(end1S);
		end1BG = new BranchGroup();
		end1BG.setCapability(BranchGroup.ALLOW_DETACH);
		end1BG.addChild(end1TG);
		
		// end2Sphere
		end2S = new Sphere ((float)(2*Env.parMHalfMonoSize),Sphere.GENERATE_NORMALS, Env.parMEndSphereTessalation, noPlasmidApp);
		end2S.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		end2T3D = new Transform3D();
		end2TG = new TransformGroup(end2T3D);
		end2TG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		end2TG.addChild(end2S);
		end2BG = new BranchGroup();
		end2BG.setCapability(BranchGroup.ALLOW_DETACH);
		end2BG.addChild(end2TG);
		
		// coordinate system
		// x-axis
		xLineEndPt = new Pt3D();
		xLine = new LineArray(2,LineArray.COORDINATES);
		xLine.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
		xLineEndPt.add(coord,0.5*length+coordLineLength,uVec);
		xLine.setCoordinate(0,coord);
		xLine.setCoordinate(1,xLineEndPt);
		xLineShape = new Shape3D();
		xLineShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		xLineShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		xLineShape.setGeometry(xLine);
		xLineShape.setAppearance(xLineApp);
		// y-axis
		yLineEndPt = new Pt3D();
		yLine = new LineArray(2,LineArray.COORDINATES);
		yLine.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
		yLineEndPt.add(coord,coordLineLength,yVec);
		yLine.setCoordinate(0,coord);
		yLine.setCoordinate(1,yLineEndPt);
		yLineShape = new Shape3D();
		yLineShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		yLineShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		yLineShape.setGeometry(yLine);
		yLineShape.setAppearance(yLineApp);
		// z-axis
		zLineEndPt = new Pt3D();
		zLine = new LineArray(2,LineArray.COORDINATES);
		zLine.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
		zLineEndPt.add(coord,coordLineLength,zVec);
		zLine.setCoordinate(0,coord);
		zLine.setCoordinate(1,zLineEndPt);
		zLineShape = new Shape3D();
		zLineShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		zLineShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		zLineShape.setGeometry(zLine);
		zLineShape.setAppearance(zLineApp);
		coordSysG = new BranchGroup();
		coordSysG.setCapability(BranchGroup.ALLOW_DETACH);
		coordSysG.addChild(xLineShape);
		coordSysG.addChild(yLineShape);
		coordSysG.addChild(zLineShape);
		
		if (!Env.helixSpheres) {
			G.addChild(end1BG);
			G.addChild(end2BG);
		}
		
		if (Env.filCoordSysOn) {
			G.addChild(coordBG);
			G.addChild(coordSysG);
			coordSysOn = true;
		}
		
		graphicsMade = true;
	}
	public void updateGraphics () {
		updateMonomerGraphics();
		
		if (!plasmidAtEnd1) { end1BG.detach(); }
		if (!plasmidAtEnd2) { end2BG.detach(); }
		if (plasmidAtEnd1 & !end1BGOn) { G.addChild(end1BG);  end1BGOn = true; }
		if (plasmidAtEnd2 & !end2BGOn) { G.addChild(end2BG);  end2BGOn = true; }
		
		boolean thisIsForceSeg = false;
		if (Env.simpleSupportElasticityTst) {
			if (filArrayPos == Env.forceSeg1) { thisIsForceSeg = true; }
			if (filArrayPos == Env.forceSeg2 & Env.secondMode) { thisIsForceSeg = true; }
		}
		if (Env.filCoordSysOn | thisIsForceSeg) { 
			addCoordSysGraphics(); 
		} else { 
			removeCoordSysGraphics();
		}
		
		end1MonCenter.add(end1,Env.parMHalfMonoSize,uVec);
		end1MonCenter.copyToVector3d(end1Vec3d);
		end1T3D.setTranslation(end1Vec3d);
		end1TG.setTransform(end1T3D);
		
		end2MonCenter.add(end2,-Env.parMHalfMonoSize,uVec);
		end2MonCenter.copyToVector3d(end2Vec3d);
		end2T3D.setTranslation(end2Vec3d);
		end2TG.setTransform(end2T3D);
		
		coord.copyToVector3d(coordVec3d);
		coordT3D.setTranslation(coordVec3d);
		coordTG.setTransform(coordT3D);
		
		
		xLineEndPt.add(coord,0.5*length+coordLineLength,uVec);
		xLine.setCoordinate(0,coord);
		xLine.setCoordinate(1,xLineEndPt);
		yLineEndPt.add(coord,coordLineLength,yVec);
		yLine.setCoordinate(0,coord);
		yLine.setCoordinate(1,yLineEndPt);
		zLineEndPt.add(coord,coordLineLength,zVec);
		zLine.setCoordinate(0,coord);
		zLine.setCoordinate(1,zLineEndPt);
		
		
		//if ((plasmidAtEnd1) | (plasmidAtEnd2)) { gotPlasmidAppearance(); }
		
		
	}
	
	public void addCoordSysGraphics () {
		if (!coordSysOn) {
			G.addChild(coordBG);
			G.addChild(coordSysG);
			coordSysOn = true;
		}
	}
	
	public void removeCoordSysGraphics () {
		if (coordSysOn) {
			coordBG.detach();
			coordSysG.detach();
			coordSysOn = false;
		}
	}
	
	public void hydrolizeInFilaments () {
		Monomer curMon = minusMon;
		while (curMon!=Monomer.plusGhost) {
			curMon.checkHydrolysis();	
			if (curMon.frontMon == null) { 
				talkln ("null curMon.frontMon in FilSegment.hydrolizeInFilaments()... skipping that FilSegment with monomerCt of " + monomerCt);
				return; // just leave method if there's an errant filament 
			} 
			curMon=curMon.frontMon;	
		}
	}
	
	public static void updateAllMonomerPositions () {
		for (int i=0;i<FilSegment.filSegmentCt;i++) {
			FilSegment.theFilSegments[i].updateMonomerPositions();
		}
	}
	
	public static int getTotalMonCt () {
		int ct = 0;
		for (int i=0;i<FilSegment.filSegmentCt;i++) {
			ct += FilSegment.theFilSegments[i].monomerCt;
		}
		return ct;
	}
	
	public static void setAnnealRate () {
		Stat filStat = filLengthStatistics();
		double L = filStat.mean/Env.parMHalfMonoSize; // ave length in subunits
		double ka = Env.annealingConstant.getValue()/L;
		double N2 = Math.pow(filStat.number,2)*Math.pow(Bug.milliMolarChangePerMonomer, 2);
		Env.oneRateAnnealing = ka*N2/Bug.milliMolarChangePerMonomer;
		//if (Env.mtRNG.raw() < 0.01) { talkln ("oneRateAnnealing is " + Env.oneRateAnnealing);}
	}
	
	//**** statistics
	public static Stat filLengthStatistics () {
		double curLength;
		double filLengthSum = 0;
		double filLengthSqrdSum = 0;
		int filNum = 0;
		for (int i=0;i<filSegmentCt;i++) {
			curLength = theFilSegments[i].length;
			if (curLength > Env.minLengthToCount.getValue()) {
				filLengthSum += curLength;
				filLengthSqrdSum += curLength*curLength;
				filNum++;
			}
		}
		double lengthMean = filLengthSum/filNum;
		double lengthStdDev = Math.sqrt((filLengthSqrdSum - (filLengthSum*filLengthSum/filNum))/(filNum-1));
		Stat filStat = new Stat (filNum,lengthMean,lengthStdDev);
		return filStat;
	}
	
	
	
}


