package parMSpindle;
/* FileOps... like it sounds */

/*  <ParMSpindle - a simulation of ParM based plasmid segregation>
    Copyright (C) <2008>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import infoCCD.Info;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.lang.Math;
import java.awt.*;
import java.io.*;
import java.text.DecimalFormat;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.JFileChooser;
import javax.swing.JFrame;

public class FileOps {
	// type definitions
	static final int NOTHING = 999;
	static final int TIME = -1;
	static final int PARM_CONCENTRATION = -2;
	static final int NUM_FILS = -3;
	static final int NUM_MONS = -4;
	static final int NUM_FILSEGS = -5;
	static final int NUM_FILLINKS = -6;
	static final int NUM_PLASMIDS = -7;
	static final int NUM_THINGS = 0;
	static final int FILSEGMENT = 1;
	static final int PLASMID = 2;
	static final int BUG = 3;
	static final int FILLINK = 4;
	
	static String sepString = ";";
	static String divStr = ":";
	static String endStr = ";";
	static String idDivStr = ":";
	static String commentStr = "//";
	// file, file writer, and print writer for program errors to file
	static File errorsOutFile;
	static FileOutputStream errorsOutFOS;
	static PrintStream errorsOutPS;	
	// file, file writer, and print writer for program messages to file
	static File messagesOutFile;
	static FileWriter messagesOutFW;
	static PrintWriter messagesOutPW;	
	// file, file writer, and print writer for capState file
	static File capStateFile;
	static FileWriter capStateFW;
	static PrintWriter capStatePW;
	// file, file writer, and print writer for information file
	static File outFile;
	static FileWriter outFW;
	static PrintWriter outPW;
	// file, file writer, and print writer for persistence length file
	static File lpFile;
	static FileWriter lpFW;
	static PrintWriter lpPW;
//	 file, file writer, and print writer for apparent diffusivity file
	static File aDFile;
	static FileWriter aDFW;
	static PrintWriter aDPW;
	// files and writers for filament lengths file
	static File filLengthFile;
	static FileWriter filLengthFW;
	static PrintWriter filLengthPW;
	// files and writers for filament length histogram file
	static File filLengthHistoFile;
	static FileWriter filLengthHistoFW;
	static PrintWriter filLengthHistoPW;
	// files and writers for filament lifetime histogram file
	static File filLifetimeHistoFile;
	static FileWriter filLifetimeHistoFW;
	static PrintWriter filLifetimeHistoPW;
	// files and writers for filament cap state histogram file
	static File filCapStateHistoFile;
	static FileWriter filCapStateHistoFW;
	static PrintWriter filCapStateHistoPW;
	// files and writers for filament cap state Low histogram file
	static File filCapStateLoHistoFile;
	static FileWriter filCapStateLoHistoFW;
	static PrintWriter filCapStateLoHistoPW;
	// files and writers for filament cap state High histogram file
	static File filCapStateHiHistoFile;
	static FileWriter filCapStateHiHistoFW;
	static PrintWriter filCapStateHiHistoPW;
	// files and writers for filament length histogram file
	static File relaxFile;
	static FileWriter relaxFileFW;
	static PrintWriter relaxFilePW;
	// flags for file setup
	static boolean capStateFileMade = false;
	static boolean outFileMade = false;
	static boolean lpFileMade = false;
	static boolean aDFileMade = false;
	static boolean filLengthFileMade = false;
	static boolean filLengthHistoFileMade = false;
	static boolean filLifetimeHistoFileMade = false;
	static boolean filCapStateHistoFilesMade = false;
	static boolean relaxFileMade = false;
	
	// file id formating
	static DecimalFormat fileIdFormat = new DecimalFormat ("#000000000.#;#000000000.#");


	public FileOps () {
	}
	

	// A class to filter out .java files
	class SourceFileFilter implements FilenameFilter {

    /**
    * Select only .java files.
    *
    * @param dir the directory in which the file was found.
    *
    * @param name the name of the file
    *
    * @return true if and only if the name should be
    * included in the file list; false otherwise.
    */
    public boolean accept ( File dir, String name )
      {
      if ( new File( dir, name ).isDirectory() )
         {
         return false;
         }
      name = name.toLowerCase();
      return name.endsWith( ".java" );
      }
    }
	
	//	 A class to filter out .param files
	class ParamFileFilter implements FilenameFilter {

    /**
    * Select only .param files.
    *
    * @param dir the directory in which the file was found.
    *
    * @param name the name of the file
    *
    * @return true if and only if the name should be
    * included in the file list; false otherwise.
    */
    public boolean accept ( File dir, String name )
      {
      if ( new File( dir, name ).isDirectory() )
         {
         return false;
         }
      name = name.toLowerCase();
      return name.endsWith( ".param" );
      }
    }
	
	// A class to filter out .qk files
	static class QKFileFilter implements FilenameFilter {

    /**
    * Select only .qk files.
    *
    * @param dir the directory in which the file was found.
    *
    * @param name the name of the file
    *
    * @return true if and only if the name should be
    * included in the file list; false otherwise.
    */
    public boolean accept ( File dir, String name )
      {
      if ( new File( dir, name ).isDirectory() )
         {
         return false;
         }
      name = name.toLowerCase();
      return name.endsWith( ".qk" );
      }
    }
    
    public void copyFile(File in, File out) throws Exception {
	    FileInputStream fis  = new FileInputStream(in);
	    FileOutputStream fos = new FileOutputStream(out);
	    byte[] buf = new byte[1024];
	    int i = 0;
	    while((i=fis.read(buf))!=-1) {
	      fos.write(buf, 0, i);
	    }
	    fis.close();
	    fos.close();
	}
    
    public static String [] getQKFileList (File dir) {
    		return dir.list(new QKFileFilter());
    }
    
	public static String getDirectoryToSave (String dialogTitle, Component parent) {
		File file = null;
		String fileName = null;
		JFileChooser saveTo = new JFileChooser();
		saveTo.setDialogTitle(dialogTitle);
		saveTo.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int fileQuery = saveTo.showSaveDialog(parent);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            file = saveTo.getSelectedFile();
            if (!file.exists()) { file.mkdir(); }
            fileName = file.getAbsolutePath();
            talkln (dialogTitle + fileName);
        } else {
            talkln ("Save cancelled by user");
            return null;
        }
        return fileName;
	}
	
	public static String getFileToSave (String dialogTitle, JFrame parent) {
		File file = null;
		String fileName = null;
		JFileChooser saveTo = new JFileChooser();
		saveTo.setDialogTitle(dialogTitle);
		int fileQuery = saveTo.showSaveDialog(parent);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            file = saveTo.getSelectedFile();
            fileName = file.getAbsolutePath();
            talkln (dialogTitle + fileName);
        } else {
            talkln ("Save cancelled by user");
            return null;
        }
        return fileName;
	}
	
	public static String getDirectoryToLoad (String dialogTitle, JFrame parent) {
		File file = null;
		String fileName = null;
		JFileChooser loadFrom = new JFileChooser();
		loadFrom.setDialogTitle(dialogTitle);
		loadFrom.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int fileQuery = loadFrom.showOpenDialog(parent);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            file = loadFrom.getSelectedFile();
            fileName = file.getAbsolutePath();
            //talkln (dialogTitle + fileName);
        } else {
            talkln ("Load cancelled by user");
            return null;
        }
        return fileName;
	}
	
	public static String getFileToLoad (String dialogTitle, JFrame parent) {
		File file = null;
		String fileName = null;
		JFileChooser loadFrom = new JFileChooser();
		loadFrom.setDialogTitle(dialogTitle);
		loadFrom.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		int fileQuery = loadFrom.showOpenDialog(parent);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            file = loadFrom.getSelectedFile();
            fileName = file.getAbsolutePath();
            //talkln (dialogTitle + fileName);
        } else {
            talkln ("Load cancelled by user");
            return null;
        }
        return fileName;
	}
	
	public static String makeNameFromIntStep (String base, int integrationStep) {
		return (base + String.valueOf(fileIdFormat.format(integrationStep)));
	}
	
	public void mkErrorFile () {
		try {
			errorsOutFile = new File (Env.logFolderPath + File.separator + Env.outFileName + "Errors");
			errorsOutFOS = new FileOutputStream(errorsOutFile);
			errorsOutPS = new PrintStream(errorsOutFOS);
			System.setErr(errorsOutPS);
			//Date timeNow = new Date();
			//errorsOutPW.println("This is the errors file for " + Env.outFileName + " run on " + timeNow.toString());
		} catch (IOException ioe) { talkln ("An error creating errorsOut stuff"); }
	}
		
	public void mkMessageFile () {
		try {
			messagesOutFile = new File (Env.logFolderPath + File.separator + Env.outFileName + "Messages");
			messagesOutFW = new FileWriter(messagesOutFile);
			messagesOutPW = new PrintWriter(messagesOutFW, true);
			Date timeNow = new Date();
			messagesOutPW.println("This is the message file for " + Env.outFileName + " run on " + timeNow.toString());
		} catch (IOException ioe) { talkln ("An error creating messagesOut stuff"); }
	}
	
	public void mkEnvironmentFile () {
		File envFile = new File ("Env.java");
		if ( ! envFile.exists() || ! envFile.canRead() ) {
			talkln ("Can't find 'Env.java'... you'd better store those values manually");
		} else {
			try {
				FileReader fileR = new FileReader(envFile);
				BufferedReader in = new BufferedReader(fileR);
				File envOut = new File(Env.logFolderPath + File.separator + Env.outFileName + "Environment");
				FileWriter envOutFW = new FileWriter(envOut);
				PrintWriter envOutPW = new PrintWriter(envOutFW, true);
				Date timeNow = new Date();
				envOutPW.println("This is the environment file for " + Env.outFileName + " run on " + timeNow.toString());
				String nextLine = in.readLine();
				while (nextLine != null) {
					envOutPW.println(nextLine);
					nextLine = in.readLine();
				}
			} catch (IOException ioe) { talkln ("An error trying to save environment file"); }
		}
	}
	
	public void mkSourceFolder () {
		try {
			File curDir = new File (".");
			System.out.println ("Directory for source files is: " + curDir.getCanonicalPath());
			String [] srcFiles = curDir.list(new SourceFileFilter());
			for (int i=0;i<srcFiles.length;i++) {
				File curSrcFile = new File (srcFiles[i]);
				File curOutSrcFile = new File (Env.srcFilePath + File.separator + curSrcFile.getName());
				copyFile(curSrcFile,curOutSrcFile);
			}
		} catch (Exception e) { talkln ("An error trying to copy source files"); }
	}
	
	public static void mkOutFile () {
		try {
			outFile = new File (Env.logFolderPath + File.separator + Env.outFileName + "Out");
			outFW = new FileWriter(outFile);
			outPW = new PrintWriter(outFW,true);
			Date timeNow = new Date();
			outPW.println("This is the out file for " + Env.outFileName + " run on " + timeNow.toString());
			outPW.print("time;");
			String iStr;
			// this next part is for the plasmid positions header
			for (int i=0;i<Plasmid.plasmidCt;i++) {
				iStr = String.valueOf(i);
				outPW.print("plasmid" + iStr + ".x;plasmid" + iStr + ".y;plasmid" + iStr + ".z;");
			}
			outPW.print("ParM concentration;");
			outPW.print("ParM Non-hydrolyzable concentration;");
			outPW.print("fil count;");
			outPW.print("fil length ave;");
			outPW.print("fil length stdev;");
			outPW.println("");
			outFileMade = true;
		} catch (IOException ioe) { talkln ("An error creating out file stuff"); }
	}
	
	public static void mkPersistenceLengthFile () {
		try {
			lpFile = new File (Env.logFolderPath + File.separator + Env.outFileName + "Lp");
			lpFW = new FileWriter(lpFile);
			lpPW = new PrintWriter(lpFW,true);
			Date timeNow = new Date();
			lpPW.println("This is the Lp file for " + Env.outFileName + " run on " + timeNow.toString());
			lpPW.print("!time arcLength->;");
			
			double initArc = 0;//Env.stdSegLength.getIntValue()*Env.parMHalfMonoSize/2.0;
			double arcDelta = (Env.stdSegLength.getIntValue()+1)*Env.parMHalfMonoSize;
			for (int i=0;i<Env.persistenceSegNumber;i++) {
				double curArc = initArc + i*arcDelta;
				lpPW.print(curArc + ";");
			}
			lpPW.println("");
			lpFileMade = true;
		} catch (IOException ioe) { talkln ("An error creating Lp file stuff"); }
	}
	
	public static void mkDiffusivityFile () {
		try {
			aDFile = new File (Env.logFolderPath + File.separator + Env.outFileName + "Diffusivity");
			aDFW = new FileWriter(aDFile);
			aDPW = new PrintWriter(aDFW,true);
			Date timeNow = new Date();
			aDPW.println("This is the apparent diffusivity file for " + Env.outFileName + " run on " + timeNow.toString());
			aDPW.print("!time;");
			aDPW.print("xD;yD;zD;");
			aDPW.println("");
			aDFileMade = true;
		} catch (IOException ioe) { talkln ("An error creating aD file stuff"); }
	}
	
	public static void mkFilLengthFile () {
		try {
			filLengthFile = new File (Env.logFolderPath + File.separator + Env.outFileName + "Filaments");
			filLengthFW = new FileWriter(filLengthFile);
			filLengthPW = new PrintWriter(filLengthFW,true);
			Date timeNow = new Date();
			filLengthPW.println("This is the filament lengths file for " + Env.outFileName + " run on " + timeNow.toString() + " with the format filID:length:monomerCt:");
			filLengthPW.print("time;");
			filLengthPW.println("");
			filLengthFileMade = true;
		} catch (IOException ioe) { talkln ("An error creating filament length file stuff"); }
	}
	
	public static void mkFilLengthHistoFile () {
		try { 
			HistogramPlus hist = FilHistoControl.filLengthHisto;
			filLengthHistoFile = new File (Env.logFolderPath + File.separator + Env.outFileName + "FilHisto");
			filLengthHistoFW = new FileWriter(filLengthHistoFile);
			filLengthHistoPW = new PrintWriter(filLengthHistoFW, true);
			Date timeNow = new Date();
			filLengthHistoPW.println("This is the filament length histogram file for " + Env.outFileName + " run on " + timeNow.toString());
			filLengthHistoPW.print("bins;");
			for (int i=0; i<hist.binCt; i++) {
				//filLengthHistoPW.print(hist.binIntervals[i] + "-" + String.valueOf(hist.binIntervals[i]+hist.stepVal) + sepString);
				filLengthHistoPW.print(String.valueOf(hist.binIntervals[i]+hist.stepVal/2) + sepString);
			}
			filLengthHistoPW.println("");
			filLengthHistoFileMade = true;
		} catch (IOException ioe) { talkln ("Error creating filament length histogram file"); }
	}
	
	public static void mkFilLifetimeHistoFile () {
		try { 
			HistogramPlus hist = FilSegment.lifeTimes;
			filLifetimeHistoFile = new File (Env.logFolderPath + File.separator + Env.outFileName + "FilLifeHisto");
			filLifetimeHistoFW = new FileWriter(filLifetimeHistoFile);
			filLifetimeHistoPW = new PrintWriter(filLifetimeHistoFW, true);
			Date timeNow = new Date();
			filLifetimeHistoPW.println("This is the filament lifetime histogram file for " + Env.outFileName + " run on " + timeNow.toString());
			filLifetimeHistoPW.print("bins;");
			for (int i=0; i<hist.binCt; i++) {
				//filLifetimeHistoPW.print(hist.binIntervals[i] + "-" + String.valueOf(hist.binIntervals[i]+hist.stepVal) + sepString);
				filLifetimeHistoPW.print(String.valueOf(hist.binIntervals[i]+hist.stepVal/2) + sepString);
			}
			filLifetimeHistoPW.println("");
			filLifetimeHistoFileMade = true;
		} catch (IOException ioe) { talkln ("Error creating filament lifetime histogram file"); }
	}
	
	public static void mkFilCapStateHistoFile () {
		try { 
			HistogramPlus hist = FilSegment.capStates;
			filCapStateHistoFile = new File (Env.logFolderPath + File.separator + Env.outFileName + "FilCapStateHisto");
			filCapStateHistoFW = new FileWriter(filCapStateHistoFile);
			filCapStateHistoPW = new PrintWriter(filCapStateHistoFW, true);
			Date timeNow = new Date();
			filCapStateHistoPW.println("This is the filament cap state histogram file for " + Env.outFileName + " run on " + timeNow.toString());
			filCapStateHistoPW.print("bins;");
			for (int i=0; i<hist.binCt; i++) {
				//filCapStateHistoPW.print(hist.binIntervals[i] + "-" + String.valueOf(hist.binIntervals[i]+hist.stepVal) + sepString);
				filCapStateHistoPW.print(String.valueOf(hist.binIntervals[i]) + sepString);
			}
			filCapStateHistoPW.println("");
			filCapStateHistoFilesMade = true;
		} catch (IOException ioe) { talkln ("Error creating filament cap state histogram file"); }
		
		try { 
			HistogramPlus hist = FilSegment.capStateLo;
			filCapStateLoHistoFile = new File (Env.logFolderPath + File.separator + Env.outFileName + "FilCapStateLoHisto");
			filCapStateLoHistoFW = new FileWriter(filCapStateLoHistoFile);
			filCapStateLoHistoPW = new PrintWriter(filCapStateLoHistoFW, true);
			Date timeNow = new Date();
			filCapStateLoHistoPW.println("This is the filament cap state LO histogram file for " + Env.outFileName + " run on " + timeNow.toString());
			filCapStateLoHistoPW.print("bins;");
			for (int i=0; i<hist.binCt; i++) {
				//filCapStateLoHistoPW.print(hist.binIntervals[i] + "-" + String.valueOf(hist.binIntervals[i]+hist.stepVal) + sepString);
				filCapStateLoHistoPW.print(String.valueOf(hist.binIntervals[i]) + sepString);
			}
			filCapStateLoHistoPW.println("");
		} catch (IOException ioe) { talkln ("Error creating filament cap state LO histogram file"); }
		
		try { 
			HistogramPlus hist = FilSegment.capStateHi;
			filCapStateHiHistoFile = new File (Env.logFolderPath + File.separator + Env.outFileName + "FilCapStateHiHisto");
			filCapStateHiHistoFW = new FileWriter(filCapStateHiHistoFile);
			filCapStateHiHistoPW = new PrintWriter(filCapStateHiHistoFW, true);
			Date timeNow = new Date();
			filCapStateHiHistoPW.println("This is the filament cap state HI histogram file for " + Env.outFileName + " run on " + timeNow.toString());
			filCapStateHiHistoPW.print("bins;");
			for (int i=0; i<hist.binCt; i++) {
				//filCapStateHiHistoPW.print(hist.binIntervals[i] + "-" + String.valueOf(hist.binIntervals[i]+hist.stepVal) + sepString);
				filCapStateHiHistoPW.print(String.valueOf(hist.binIntervals[i]) + sepString);
			}
			filCapStateHiHistoPW.println("");
		} catch (IOException ioe) { talkln ("Error creating filament cap state HI histogram file"); }
	}
	
	public static void mkRelaxFile () {
		try {
			relaxFile = new File ("/Users/jonathan/Desktop/relaxFile");
			int j=1;
			while (relaxFile.isFile()) {
				System.out.println (relaxFile.getName() + " exists.... adding integers to end of name");
				relaxFile = new File("/Users/jonathan/Desktop/relaxFile." + String.valueOf(j));
				j++;
			}
			relaxFileFW = new FileWriter(relaxFile);
			relaxFilePW = new PrintWriter(relaxFileFW,true);
			Date timeNow = new Date();
			relaxFilePW.println("Relaxation file for length " + Env.tstFilLength + ", secondMode is " + Env.secondMode + " run on " + timeNow.toString());
			relaxFilePW.println("simTime;;");
			relaxFileMade = true;
		} catch (IOException ioe) { talkln ("An error creating relaxFile"); }
	}
	
	public static void mkCapStateFile () {
		try {
			capStateFile = new File (Env.logFolderPath + File.separator + Env.outFileName + "CapStates");
			capStateFW = new FileWriter(capStateFile);
			capStatePW = new PrintWriter(capStateFW,true);
			Date timeNow = new Date();
			capStatePW.println("This is the capState file for " + Env.outFileName + " run on " + timeNow.toString() + " with the format filID:capStateEnd1:capStateEnd2:");
			capStatePW.print("time;filID;capStateEnd1;capStateEnd2; and so on;");
			capStatePW.println("");
			capStateFileMade = true;
		} catch (IOException ioe) { talkln ("An error creating cap state file stuff"); }
	}
	
	public static void writeToCapStateFile () {
		if (!capStateFileMade) { mkCapStateFile(); }
		printValue(Env.simulationTime,capStatePW);
		FilSegment curFil;
		for (int i=0;i<FilSegment.filCt;i++) {
			curFil = FilSegment.theFilSegments[i];
			int cap1ATP = curFil.capLengthEnd1();
			int cap2ATP = curFil.capLengthEnd2();
			capStatePW.print(String.valueOf(curFil.filID));
			capStatePW.print(idDivStr);
			printValue(cap1ATP,capStatePW);
			printValue(cap2ATP,capStatePW);
		}
		capStatePW.println("");
	}
	
	public static void writeToOutFile () {
		if (!outFileMade) { mkOutFile(); }
		printValue(Env.simulationTime,outPW);
		for (int i=0;i<Plasmid.plasmidCt;i++) {
			printPt3D(Plasmid.thePlasmids[i].coord, outPW);
		}
		printValue(Env.parMConc.getValue(),outPW);
		printValue(Env.parMConcNonHydro.getValue(),outPW);
		Stat filStats = FilSegment.filLengthStatistics();
		printValue(filStats.getNumber(),outPW);
		printValue(filStats.getMean(),outPW);
		printValue(filStats.getStdDev(),outPW);
		outPW.println("");
	}
	
	public static void writeToPersistenceLengthFile () {
		if (!lpFileMade) { mkPersistenceLengthFile(); }
		printValue(Env.simulationTime,lpPW);
		for (int i=0;i<Env.persistenceSegNumber;i++) {
			printValue(Env.persistenceTrack[i].runningAverageVal(),lpPW);
		}
		lpPW.println("");
	}
	
	public static void writeToDiffusivityFile () {
		if (!aDFileMade) { mkDiffusivityFile(); }
		printValue(Env.simulationTime,aDPW);
		// calculate D = variance/2*deltaT
		Pt3D curDPt = Env.transDiffTrack.varianceOfPt();
		curDPt.scale(1/(2*Env.ckPersistenceInterval*Env.deltaT.getValue()));
		printPt3D(curDPt,aDPW);
		aDPW.println("");
	}
	
	public static void writeToFilLengthFile () {
		if (!filLengthFileMade) { mkFilLengthFile(); }
		printValue(Env.simulationTime,filLengthPW);
		for (int i=0;i<FilSegment.filSegmentCt;i++) {
			filLengthPW.print(String.valueOf(FilSegment.theFilSegments[i].filID));
			filLengthPW.print(idDivStr);
			printValue(FilSegment.theFilSegments[i].length, filLengthPW);
			printValue(FilSegment.theFilSegments[i].monomerCt, filLengthPW);
		}
		filLengthPW.println("");
	}
	
	public static void writeToFilLengthHistoFile () {
		HistogramPlus hist = FilHistoControl.filLengthHisto;
		if (!filLengthHistoFileMade) { mkFilLengthHistoFile(); }
		printTime(filLengthHistoPW);
		for (int i=0;i<hist.binCt;i++) {
			printIntValue(hist.binSums[i],filLengthHistoPW);
		}
		filLengthHistoPW.println("");
	}
	
	public static void writeToFilLifetimeHistoFile () {
		HistogramPlus hist = FilSegment.lifeTimes;
		if (!filLifetimeHistoFileMade) { mkFilLifetimeHistoFile(); }
		printTime(filLifetimeHistoPW);
		for (int i=0;i<hist.binCt;i++) {
			printIntValue(hist.binSums[i],filLifetimeHistoPW);
		}
		filLifetimeHistoPW.println("");
	}
	
	public static void writeToFilCapStateHistoFile () {
		HistogramPlus hist = FilSegment.capStates;
		HistogramPlus histLo = FilSegment.capStateLo;
		HistogramPlus histHi = FilSegment.capStateHi;
		hist.clearBins();
		histLo.clearBins();
		histHi.clearBins();
		FilSegment curFil;
		for (int i=0;i<FilSegment.filCt;i++) {
			curFil = FilSegment.theFilSegments[i];
			int cap1ATP = curFil.capLengthEnd1();
			int cap2ATP = curFil.capLengthEnd2();
			hist.addValue(cap1ATP+.1);
			hist.addValue(cap2ATP+.1);
			if (cap1ATP > cap2ATP) { 
				histHi.addValue(cap1ATP+0.1); 
				histLo.addValue(cap2ATP+0.1);
			} else {
				histHi.addValue(cap2ATP+0.1); 
				histLo.addValue(cap1ATP+0.1);
			}
		}
		
		if (!filCapStateHistoFilesMade) { mkFilCapStateHistoFile(); }
		
		// the sum cap state file
		printTime(filCapStateHistoPW);
		for (int i=0;i<hist.binCt;i++) {
			printIntValue(hist.binSums[i],filCapStateHistoPW);
		}
		filCapStateHistoPW.println("");
		
		// the low cap state file
		printTime(filCapStateLoHistoPW);
		for (int i=0;i<histLo.binCt;i++) {
			printIntValue(histLo.binSums[i],filCapStateLoHistoPW);
		}
		filCapStateLoHistoPW.println("");
		
		// the high cap state file
		printTime(filCapStateHiHistoPW);
		for (int i=0;i<histHi.binCt;i++) {
			printIntValue(histHi.binSums[i],filCapStateHiHistoPW);
		}
		filCapStateHiHistoPW.println("");
	}
	
	public static void writeToRelaxFile () {
		if (!relaxFileMade) { mkRelaxFile(); }
		printValue(Env.simulationTime,relaxFilePW);
		printValue(Env.currentDeflection,relaxFilePW);
		relaxFilePW.println();
	}
	
	private static void printDivider (PrintWriter pw) {
		pw.print(sepString);
	}
	
	private static void printValue (int val, PrintWriter pw) {
		pw.print(String.valueOf(val));
		printDivider(pw);
	}
	
	private static void printIntValue (double val, PrintWriter pw) {
		int intVal = (int) val;
		pw.print(String.valueOf(intVal));
		printDivider(pw);
	}
	
	private static void printValue (double val, PrintWriter pw) {
		pw.print(String.valueOf(val));
		printDivider(pw);
	}
	
	private static void printTime (PrintWriter pw) {
		pw.print("time is " + Env.simulationTime + " s");
		printDivider(pw);
	}
	
	private static void printPt3D (Pt3D pt, PrintWriter pw) {
		pw.print(String.valueOf(pt.x));
		printDivider(pw);
		pw.print(String.valueOf(pt.y));
		printDivider(pw);
		pw.print(String.valueOf(pt.z));
		printDivider(pw);
	}
	
	public static void remoteParamConfigSave () {
		File configFile = new File (Env.logFolderPath + File.separator + Env.outFileName + "Params");
		writeParamConfig (configFile);
	}
	
	public static void writeParamConfig (File paramOutFile) {
		talk ("Saving parameter values to " + paramOutFile.getAbsolutePath() + "...");
		try {
			FileWriter paramOutFW = new FileWriter(paramOutFile);
			PrintWriter paramOutPW = new PrintWriter(paramOutFW, true);
			Date timeNow = new Date();
			paramOutPW.println(infoCCD.Info.appParamID +" This is a parameter configuration file created on " + timeNow.toString());
			Parameter curParam;
			String firstStr;
			for (int i=0;i<Parameter.paramCt;i++) {
				curParam = Parameter.theParams[i];
				firstStr = curParam.label + divStr + String.valueOf(curParam.isActive()) + divStr + String.valueOf(curParam.getValue()) + endStr;
				paramOutPW.print(firstStr);
				int spacesToAdd = 40-firstStr.length();
				for (int j=0;j<spacesToAdd;j++) { paramOutPW.print(" "); }
				paramOutPW.println(commentStr + curParam.parameterName);
			}
			talkln ("done.");
		} catch (IOException ioe) { talkln ("An error trying to write the parameter config file"); }
	}

	public static boolean loadParamLine (String curLine, String fromArg) {
		String curLabel,curValueStr,curActiveStr;
		boolean curActive = true;
		double curValue = 0;
		int div1Ind,div2Ind,endInd;
		// parse label and value strings
		div1Ind = curLine.indexOf(divStr);
		div2Ind = curLine.indexOf(divStr,div1Ind+1);
		endInd = curLine.indexOf(endStr);
		if ((div1Ind == -1) | (div2Ind == -1) | (endInd == -1)) { return false; }
		curLabel = curLine.substring(0,div1Ind).trim();
		curActiveStr = curLine.substring(div1Ind+1,div2Ind).trim();
		curValueStr = curLine.substring(div2Ind+1,endInd);
		// make value string a double
		try {
			Double valD = Double.valueOf(curValueStr);
			curValue = valD.doubleValue();
		} catch (java.lang.NumberFormatException nfe) {
			talkln("Invalid parameter value, " + curValueStr + " loaded from " + fromArg + "... exiting");
			System.exit(0);
		}
		// set boolean
		if (curActiveStr.equals("false")) { curActive = false; } else { curActive = true; }
		// find the matching parameter label from static list of parameters
		boolean matchFound = false;
		for (int i=0;i<Parameter.paramCt;i++) {
			if (Parameter.theParams[i].label.equals(curLabel)) {
				Parameter.theParams[i].setValue(curValue);
				Parameter.theParams[i].setActive(curActive);
				matchFound = true;
				if (Parameter.theParams[i].type == Parameter.BOOLEAN) {
					talkln ("     Parameter assigned: " + curLabel + " set to " + String.valueOf(Parameter.theParams[i].isActive()));
				} else if (Parameter.theParams[i].type == Parameter.INT) {
					talkln ("     Parameter assigned: " + curLabel + " set to " + String.valueOf(Parameter.theParams[i].getIntValue()));
				} else {
					talkln ("     Parameter assigned: " + curLabel + " set to " + String.valueOf(Parameter.theParams[i].getValue()));
				}
			}
			if (matchFound) { break; }
		}
		if (!matchFound) { 
			talkln ("     No match found for parameter label " + curLabel + " ... you best check that out!"); 
		}
		return matchFound;
		
	}
	
	public static void loadParamConfig (File paramFile, boolean restartNeeded) {
		talkln ("Loading parameter values from " + paramFile.getAbsolutePath() + "...");
		int misunderstandings = 0;
		if ( ! paramFile.exists() || ! paramFile.canRead() ) {
			talkln ("Can't find or access " + paramFile.getAbsolutePath() + " ... check the file path and name");
		} else {
			try {
				FileReader fileR = new FileReader(paramFile);
				BufferedReader in = new BufferedReader(fileR);
				String curLine = in.readLine();
				String fromArg = paramFile.getAbsolutePath();
				while (curLine != null) {
					if (!curLine.startsWith(commentStr) & !curLine.startsWith(Info.appParamID) & (curLine.length() >= 1)) {
						boolean matchFound = loadParamLine(curLine,fromArg);
						if (!matchFound) { misunderstandings++; }
					}
					// next line
					curLine = in.readLine();
					
				}
			} catch (IOException ioe) { talkln ("An error trying to load param file"); }
		}
		talkln ("Done... there were " + String.valueOf(misunderstandings) + " misunderstandings.");
		if (restartNeeded) {
			System.out.print ("Restarting simulation... ");
			ParMSpindle.restartRun(true);
			talkln("done.");
		}
		if (Env.remote) { 
			ParMSpindle.setRunning(); 
		} else {
			ParamGui.syncAllToParameters();
		}
		
	}
	
	public static void loadParamsFromViewer (String viewStr) {
		talkln ("Loading parameter values from viewer...");
		String fromArg = "the viewer";
		int misunderstandings = 0;
		int bolIndex = 0;
		int eolIndex = 0;
		String curLine;
		int finalIndex = viewStr.lastIndexOf("\n");
		while (bolIndex < finalIndex) {
			eolIndex = viewStr.indexOf("\n",bolIndex);
	    	curLine = viewStr.substring(bolIndex,eolIndex);
	    	//System.out.println(curLine);
	    	if (!curLine.startsWith(commentStr) & !curLine.startsWith(Info.appParamID) & (curLine.length() >= 1)) {
				boolean matchFound = loadParamLine(curLine,fromArg);
				if (!matchFound) { misunderstandings++; }
			}
			// next line
			bolIndex = eolIndex+1;
		}
		talkln ("Done... there were " + String.valueOf(misunderstandings) + " misunderstandings.");
		System.out.print ("Restarting simulation... ");
		ParMSpindle.restartRun(true);
		talkln("done.");
		if (Env.remote) { 
			ParMSpindle.setRunning(); 
		} else {
			ParamGui.syncAllToParameters();
		}
		
	}
	
	public static void takeQuickPicture (String fileName) { 
		Thing curT;
		Monomer mon;
		FilSegment fil;
		Plasmid plas;
		Bug bug;
		int noTs = Thing.thingCt;
		try {
			try {
			FileOutputStream fos = new FileOutputStream (fileName);
			DataOutputStream ds = new DataOutputStream (fos);
			// write general data before Things...
			// time
			ds.writeInt(FileOps.TIME);
			ds.writeFloat((float)Env.simulationTime);
			// ParM concentration
			ds.writeInt(FileOps.PARM_CONCENTRATION);
			ds.writeFloat(Env.parMConc.getFloatValue());
			// Plasmid number
			ds.writeInt(FileOps.NUM_PLASMIDS);
			ds.writeInt(Plasmid.plasmidCt);
			// Filament number
			ds.writeInt(FileOps.NUM_FILS);
			ds.writeInt(FilSegment.filCt);
			// Filament segment number
			ds.writeInt(FileOps.NUM_FILSEGS);
			ds.writeInt(FilSegment.filSegmentCt);
			// Monomer number
			ds.writeInt(FileOps.NUM_MONS);
			ds.writeInt(FilSegment.monomerSum());
			// FilLinks
			ds.writeInt(FileOps.NUM_FILLINKS);
			ds.writeInt(FilLink.filLinkCt);
			// Number of Things
			ds.writeInt(FileOps.NUM_THINGS);
			ds.writeInt(noTs);
			for (int i = 0; i<noTs; i++) {
				curT = Thing.theThings[i];
				if (curT instanceof FilSegment) {
					fil = (FilSegment) curT;
					ds.writeByte((byte)FILSEGMENT);	// identify as a filament
					Pt3D.writePt3D(ds,fil.coord);
					Pt3D.writePt3D(ds,fil.uVec);
					ds.writeInt(fil.monomerCt);
					if (fil.filAtEnd2) { ds.writeInt(fil.end2Fil.myThingNumber); } else { ds.writeInt(-1); }  // myThingNumber of filsegment at end2
					if (fil.plasmidAtEnd1) {  // myThingNumber of linked plasmids
						ds.writeInt(fil.end1Plasmid.myThingNumber); 
						Pt3D.writePt3D(ds, fil.end1PAttachPt);
					} else { ds.writeInt(-1); }	
					if (fil.plasmidAtEnd2) { 
						ds.writeInt(fil.end2Plasmid.myThingNumber); 
						Pt3D.writePt3D(ds, fil.end2PAttachPt);
					} else { ds.writeInt(-1); }
					mon = fil.minusMon;
					int monWriteCt = 0;
					while (mon != Monomer.plusGhost) {
						if (mon.monVec3D == null) {
							talk ("Writing null mon position in FileOps.takeQuickPicture()");
							talk ("  Monomer = " + mon);
							talk ("  Monomer.frontMon = " + mon.frontMon);
							talkln ("MonomerCt = " + fil.monomerCt + ", but monWriteCt = " + monWriteCt);
							for (int k = monWriteCt; k < fil.monomerCt; k++) {
								Pt3D.writeNullVec3d(ds);
								ds.writeInt(-666);
								ds.writeInt(-666);
							}
							break;
						} else {
							Pt3D.writeVec3d(ds,mon.monVec3D);
							monWriteCt++;
						}
						if (mon.plasmidBound) { ds.writeInt(1); } else { ds.writeInt(0); }
						ds.writeInt(mon.getState());
						mon = mon.frontMon;
					}
				} else if (curT instanceof Plasmid) {
					plas = (Plasmid) curT;
					ds.writeByte((byte)PLASMID);	// identify as a node
					Pt3D.writePt3D(ds,plas.coord);
					ds.writeFloat((float)plas.getRadius());
				} else if (curT instanceof Bug) {
					bug = (Bug) curT;
					ds.writeByte((byte)BUG);
					ds.writeFloat((float)bug.getLength());
					ds.writeFloat((float)bug.getRadius());
				}	
			} 
			// write out all the FilLink info
			for (int i=0;i<FilLink.filLinkCt;i++) {
				ds.writeByte((byte)FILLINK);
				if (FilLink.filLinks[i] != null & FilLink.filLinks[i].pt1 != null  & FilLink.filLinks[i].pt2 != null) {	// check if link's gone to null
					Pt3D.writePt3D(ds,FilLink.filLinks[i].pt1);
					Pt3D.writePt3D(ds,FilLink.filLinks[i].pt2);
				} else {
					Pt3D.writePt3D(ds, Env.farfarAway);
					Pt3D.writePt3D(ds, Env.farfarAway);
				}
			}
			} catch (NullPointerException nullE) { talkln ("In takeQuickPicture: " + nullE.getMessage()); }
		} catch (IOException ioe) { talkln ("some sort of error writing binary file"); }
	}
	
	public static void loadQuickPicture (String curFile) {
		Env.fromQKFile = true;
		FilHistoControl.filLengthHisto.clearBins();		// zero out filament length histogram
		int id = FileOps.NOTHING;
		int noTs = 0;	// number of Things
		int noFLs = 0;	// number of FilLinks
		int noPlas = 0;	// number of Plasmids
		int curType;
		int curMonCt;
		boolean plasmidBound = false;
		int plasOn = 0;	// 1 if plasmid attached, 0 if not
		Pt3D loc = new Pt3D();
		Pt3D ang = new Pt3D();
		Pt3D wastedPt3D = new Pt3D();
		// thing counters
		int monSum = 0;
		int plasSum = 0;
		try {
			FileInputStream fis = new FileInputStream (curFile);
			DataInputStream ds = new DataInputStream (fis);
			while (id != FileOps.NUM_THINGS) {
				id = ds.readInt();
				switch (id) {
				case FileOps.TIME:
					Env.simulationTime = ds.readFloat();
					break;
				case FileOps.PARM_CONCENTRATION:
					Env.parMConc.setValue(ds.readFloat());
					break;
				case FileOps.NUM_PLASMIDS:
					noPlas = ds.readInt();
					break;
				case FileOps.NUM_FILS:
					FilSegment.filamentRenderCt = ds.readInt();
					break;
				case FileOps.NUM_FILSEGS:
					FilSegment.filSegRenderCt = ds.readInt();
					break;
				case FileOps.NUM_MONS:
					Monomer.monRenderCt = ds.readInt();
					if (Env.filRenderOff) { Monomer.monCt = 0; }
					break;
				case FileOps.NUM_FILLINKS:
					noFLs = ds.readInt();
					break;
				case FileOps.NUM_THINGS:
					noTs = ds.readInt();	// number of things to read in following loop
					break;
				}
			}
			for (int i=0;i<noTs;i++) {
				curType = (int)ds.readByte();
				switch (curType) {
				case FILSEGMENT:
					double locShift = 0;
					loc.readPt3D(ds);
					ang.readPt3D(ds);
					if (Env.orderedCentered) { 
						locShift = loc.x;
						loc.x = 0; 
					}
					curMonCt = ds.readInt();
					ds.readInt();	// read in myThingNumber of end2 filsegment (or -1 if no link), unused here
					FilHistoControl.addValToFilLengthHisto(curMonCt);
					plasOn = ds.readInt();	// plasmid at end1
					if (plasOn != -1 ) { wastedPt3D.readPt3D(ds); } // read in attachPt.. unused
					plasOn = ds.readInt();	// plasmid at end2 
					if (plasOn != -1 ) { wastedPt3D.readPt3D(ds); } // read in attachPt.. unused
					for (int m=0;m<curMonCt;m++) {
						if (Monomer.monCt > monSum) {
							loc.readPt3D(ds);
							if (Env.orderedCentered) { loc.x += -locShift; }
							if (ds.readInt() == 1) { plasmidBound = true; } else { plasmidBound = false; }
							Monomer.theMonomers[monSum].setFromQKInfo(loc,ds.readInt(),plasmidBound);
						} else {
							Monomer.theMonomers[Monomer.monCt] = new Monomer();
							loc.readPt3D(ds);
							if (Env.orderedCentered) { loc.x += -locShift; }
							if (ds.readInt() == 1) { plasmidBound = true; } else { plasmidBound = false; }
							Monomer.theMonomers[Monomer.monCt].setFromQKInfo(loc,ds.readInt(),plasmidBound);
							Monomer.monCt++;
						}
						monSum++;
					}
					break;
				case PLASMID:
					loc.readPt3D(ds);
					Env.plasmidRadius.setValue((double)ds.readFloat());
					if (Plasmid.plasmidCt > plasSum) {
						Plasmid.thePlasmids[plasSum].coord.copy(loc);
					} else {
						new Plasmid(loc);
					}
					plasSum++;
					if (Env.filRenderOff & (plasSum == noPlas)) { return; }
					break;
				case BUG:
					if (Thing.theBug == null) {
						Env.bugLength.setValue((double)ds.readFloat());
						Env.bugRadius.setValue((double)ds.readFloat());
						Bug.makeInitialBug();
					} else {
						ds.readFloat(); //length
						ds.readFloat(); //radius
					}
				}
			}
			if (monSum != Monomer.monRenderCt) { talkln("monSum != Monomer.monRenderCt.... check that out"); }
			
			// FilLinks... not a subclass of Thing
			if (!Env.filRenderOff) {
				Pt3D filLinkPt1 = new Pt3D();
				Pt3D filLinkPt2 = new Pt3D();
				for (int i=0;i<noFLs;i++) {
					curType = (int)ds.readByte();
					if (curType != FILLINK) { talkln ("Thought we were reading FILLINK but we're not"); }
					filLinkPt1 = Pt3D.ReadPt3D(ds);
					filLinkPt2 = Pt3D.ReadPt3D(ds);
					if (i<FilLink.filLinkCt) {
						FilLink.filLinks[i].setPtsFromQKFile(filLinkPt1,filLinkPt2);
					} else {
						new FilLink (filLinkPt1,filLinkPt2);
					}
				}
				// effectively remove graphics from unused FilLinks
				for (int i=noFLs;i<FilLink.filLinkCt;i++) { FilLink.filLinks[i].setPtsFromQKFile(Env.farfarAway,Env.farfarAway); }
			}
		} catch (IOException ioe) { talkln ("some sort of error reading .qk file " + curFile); }
	}
	
	public static void loadQuickState (String curFile) {
		Env.loadingQKFile = true;
		ParMSpindle_Graphics.clearForRun();
		FilLink.filLinkCt = 0;
		int id = FileOps.NOTHING;
		int noTs = 0;
		int curType;
		int curMonCt;
		int monState;
		int filSum = 0;
		int end2ThingNumber = -1;
		Pt3D loc = new Pt3D();
		Pt3D vec = new Pt3D();
		FilSegment fil;
		Monomer curMon = new Monomer();
		try {
			FileInputStream fis = new FileInputStream (curFile);
			DataInputStream ds = new DataInputStream (fis);
			while (id != FileOps.NUM_THINGS) {
				id = ds.readInt();
				switch (id) {
				case FileOps.TIME:
					Env.simulationTime = ds.readFloat();
					break;
				case FileOps.PARM_CONCENTRATION:
					Env.parMConc.setValue(ds.readFloat());
					break;
				case FileOps.NUM_PLASMIDS:
					ds.readInt();
					break;
				case FileOps.NUM_FILS:
					ds.readInt();
					break;
				case FileOps.NUM_FILSEGS:
					ds.readInt(); // unused... value set when filsegs created
					break;
				case FileOps.NUM_MONS:
					ds.readInt();	//unused
					break;
				case FileOps.NUM_FILLINKS:
					ds.readInt();
					break;
				case FileOps.NUM_THINGS:
					noTs = ds.readInt();	// number of things to read in following loop
					break;
				}
			}
			for (int i=0;i<noTs;i++) {
				curType = (int)ds.readByte();
				switch (curType) {
				case FILSEGMENT:
					filSum++;
					loc.readPt3D(ds);
					vec.readPt3D(ds);
					curMonCt = ds.readInt();
					fil = new FilSegment(loc,vec,-1,curMonCt,true);
					fil.end2LinkThingNumber = ds.readInt();
					
					// plasmids
					fil.end1PlasmidThingNumber = ds.readInt();	// plasmid at end1
					if (fil.end1PlasmidThingNumber != -1) { fil.end1PAttachPt.readPt3D(ds); }
					fil.end2PlasmidThingNumber = ds.readInt();	// plasmid at end2 
					if (fil.end2PlasmidThingNumber != -1) { fil.end2PAttachPt.readPt3D(ds); }
					// set monomer states
					curMon = fil.minusMon;
					for (int m=0;m<curMonCt;m++) {
						loc.readPt3D(ds);	// unused but must read
						monState = ds.readInt();
						curMon.setState(monState);
						curMon = curMon.frontMon;
					}
					break;
				case PLASMID:
					loc.readPt3D(ds);
					Env.plasmidRadius.setValue((double)ds.readFloat());
					new Plasmid(loc);
					break;
				case BUG:
					Env.bugLength.setValue((double)ds.readFloat());
					Env.bugRadius.setValue((double)ds.readFloat());
					Bug.makeInitialBug();
				}
			}
			
			// set FilSegment links to other FilSegments and Plasmids
			FilSegment fil1, fil2;
			Plasmid plas;
			for (int i=0;i<FilSegment.filSegmentCt;i++) {
				fil1 = FilSegment.theFilSegments[i];
				if (fil1.end2LinkThingNumber != -1) {
					if (Thing.theThings[fil1.end2LinkThingNumber] instanceof FilSegment) {
						fil2 = (FilSegment)Thing.theThings[fil1.end2LinkThingNumber];
						FilSegment.link(fil1, fil2);
					} else {
						System.out.println ("Error in linking filsegments after QK state read");
					}
				}
				
				if (fil1.end1PlasmidThingNumber != -1) {
					if (Thing.theThings[fil1.end1PlasmidThingNumber] instanceof Plasmid) {
						plas = (Plasmid)Thing.theThings[fil1.end1PlasmidThingNumber];
						FilSegment.linkEnd1Plasmid(fil1,plas);
					} else {
						System.out.println ("Error in linking filsegment and plasmid after QK state read");
					}
				}
				
				if (fil1.end2PlasmidThingNumber != -1) {
					if (Thing.theThings[fil1.end2PlasmidThingNumber] instanceof Plasmid) {
						plas = (Plasmid)Thing.theThings[fil1.end2PlasmidThingNumber];
						FilSegment.linkEnd2Plasmid(fil1,plas);
					} else {
						System.out.println ("Error in linking filsegment and plasmid after QK state read");
					}
				}
			}
				
		} catch (IOException ioe) { talkln ("some sort of error loading .qk state " + curFile); }
		Env.loadingQKFile = false;
		//System.out.println ("Done loading quick state");
		ParMSpindle.parMGraphics.updateBugScene();
		//System.out.println ("Done updating bug scene");
	}
	
	public static void talkln (String info) {
		if (Env.remote & Env.logFiles) {
			//if (!outFileMade) { mkOutFile(); }
			//outPW.println(info);
			System.out.println(info);
		} else {
			System.out.println(info);
		}
	}
	
	public static void talk (String info) {
		if (Env.remote & Env.logFiles) {
			//if (!outFileMade) { mkOutFile(); }
			//outPW.print(info);
			System.out.print(info);
		} else {
			System.out.print(info);
		}
	}
}
