package parMSpindle;
/* RenderControl...Gui Control frame for Rendering */

/*  <ParMSpindle - a simulation of ParM based plasmid segregation>
    Copyright (C) <2008>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import javax.swing.*;
import java.io.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class RenderControl extends JFrame implements ActionListener {
	static ParMSpindle_Graphics parentSim;
	static String [] fileList;
	static int fileCt; 
	static int curFile;
	static boolean fileChanged = true;
	static boolean paused = true;
	static boolean toJPEG = false;
	static boolean skip = false;
	static boolean gotoFile = false;
	static boolean rotate = false;
	static double rotateAng = 1; // degrees
	static String titleString = "Render Control: ";
	static String blankTitleString = "Use 'O' to render files....";
	static String toJPEGPath = null;
	static String toJPEGName = null;
	static String pausedString = "renderIsPaused";
	static int fileStep = 1;
	static RenderThread renderer = new RenderThread();
	static javax.swing.Timer renderTimer;
	static Object renderWaitO = new Object();
	
	static JPanel buttonsPanel,renderPanel,renderPanel1,renderPanel2,renderPanel3,renderPanel4,renderPanel5,infoPanel;
	static BareButton stopB,playB,firstB,lastB,prevB,nextB,doneB,openB;
	static JLabel jpegLoc,curRenderName,curRenderFrac;
	static JTextField skipField,gotoField,rotateField;
	static JRadioButton jpegB,skipB,gotoB,filROffB,rotateB;

	public RenderControl (ParMSpindle_Graphics parentSim) {
		RenderControl.parentSim = parentSim;
		
		makeButtonsPanel();
		makeInfoPanel();
		makeRenderPanels();
		
		
		initialize();
		this.setTitle(titleString + blankTitleString);
		setSize(540,200);
		setResizable(false);
			
		this.setLocation(parentSim.getLocation().x + Env.frameWidth.getIntValue()+5,0);
		
	}
	
	public void initialize () {
		getContentPane().removeAll();
		//getContentPane().setLayout(new GridLayout(3,1));
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add (renderPanel,BorderLayout.CENTER);
		getContentPane().add (buttonsPanel,BorderLayout.SOUTH);
		//getContentPane().add (paramPanel, BorderLayout.CENTER);
		//getContentPane().add (applyPanel, BorderLayout.SOUTH);
	
	}
	
	public void startRenderTimer() {
		int delay = 100; //milliseconds
		ActionListener taskPerformer = new ActionListener() {
			String fileToRender,lastName;
			public void actionPerformed(ActionEvent evt) {
				if (ParMSpindle_Graphics.theCanvas.isRendering()) { return; }
				if (fileChanged) { 
					if (gotoFile) { 
						curFile = getGotoFile(); 
						gotoFile = false;
					}
					lastName = fileList[curFile];
					fileToRender = Env.fromQKFilePath + File.separator + lastName;
					curRenderFrac.setText(" (" + String.valueOf(curFile+1) + "/" + String.valueOf(fileCt) + ") ");
					curRenderName.setText(lastName);
					renderer.render(fileToRender,lastName); 
					fileChanged = false;
				}
				
				if (!paused) {
					if (curFile < fileCt-1) { 
						if (skip) { curFile += getSkip(); } else { curFile++; }
						if (curFile > fileCt-1) { curFile = fileCt-1; }
						fileChanged = true;
					} else {
						paused = true;
						fileChanged = false;
					}
				}
		    }
		};
		renderTimer = new javax.swing.Timer(delay, taskPerformer);
		renderTimer.start();
	}
	
	static class RenderThread extends Thread {
		
		public void run () {
		}
		
		public void render(String fileToRender, String lastName) {
			String curFileToRender = fileToRender.substring(0);
			if (rotate) { ParMSpindle.parMGraphics.rotateView(rotateAng); }
			System.out.println("Beginning render " + curFileToRender + ".....");
			if ((toJPEG) & (toJPEGPath != null)) {
				ParMSpindle_Graphics.theCanvas.setAltCapture(lastName);
				ParMSpindle_Graphics.theCanvas.writeJPEG_ = true;
			}
			FileOps.loadQuickPicture(curFileToRender);
			ParMSpindle_Graphics.updateQKBugScene();
			if (Env.makeFilLengthHisto.isActive()) { FilHistoControl.plotHist(); }
			while ((ParMSpindle_Graphics.theCanvas.isRendering()) | (ParMSpindle_Graphics.theCanvas.isWritingJPEG())) {
				try { Thread.sleep(200); } catch (InterruptedException e) { System.out.println("error sleeping in render"); }
			}
			System.out.println("Done with render.");
			System.out.println("");
		}
		
	}
	
	public void openRender () {
		ParMSpindle_Graphics.prepForQKRender();
		if (Env.fromQKFilePath != null) {
			setTitle (titleString + Env.fromQKFilePath);
			startRender();
		}
	}
	
	public void startRender () {
		resetRender();
		if  (fileCt == 0) { 
			System.out.println("No '.qk' files found at that location, try again");
			return;
		}
		ParMSpindle_Graphics.clearForRender();
		startRenderTimer();
	}
			
	public void resetRender () {
		File qkDir = new File(Env.fromQKFilePath);
		fileList = FileOps.getQKFileList(qkDir);  // get files from directory ending in .qk
		fileCt = fileList.length;
		curFile = 0;
		fileChanged = true;
		paused = true;
	}
	
	public void timerOn() {
		if (renderTimer==null) { return; }
		if (!renderTimer.isRunning()) { renderTimer.start(); }
	}
	
	public void timerOff() {
		if (renderTimer==null) { return; }
		if (renderTimer.isRunning()) { renderTimer.stop(); }
	}
	
	public int getSkip() {
		String skipString = skipField.getText();
		int skip = 1;
		try {
			Double valD = Double.valueOf(skipString);
			double val = valD.doubleValue();
			skip = (int)val;
		} catch (java.lang.NumberFormatException nfe) {
			System.out.println("Invalid field entry in skip field.... setting to 1");
			skip = 1;
		}
		if (skip < 1) { skip = 1; }
		skipField.setText(String.valueOf(skip));
		return skip;
	}
	
	public int getGotoFile() {
		String gotoString = gotoField.getText();
		int gotoF = curFile+1; // +1 because of 0 index start
		try {
			Double valD = Double.valueOf(gotoString);
			double val = valD.doubleValue();
			gotoF = (int)val;
		} catch (java.lang.NumberFormatException nfe) {
			System.out.println("Invalid field entry in goto field.... ignoring");
			gotoF = curFile+1;
		}
		if (gotoF < 1) { gotoF = 1; }
		if (gotoF > fileCt) { gotoF = fileCt; }
		gotoField.setText(String.valueOf(gotoF));
		return gotoF-1;	// return gotoF-1 because of 0 index start
	}
	
	public double getRotAngle() {
		double rotVal = 1;
		String rotAngString = rotateField.getText();
		rotAngString = rotAngString.substring(0,rotAngString.length()-1);	// trim off degree symbol
		try {
			Double valD = Double.valueOf(rotAngString);
			rotVal = valD.doubleValue();
		} catch (java.lang.NumberFormatException nfe) {
			System.out.println("Invalid field entry in rotate field.... setting to 1 Degree");
			rotVal = 1;
		}
		rotateField.setText(String.valueOf(rotVal) + "");
		return rotVal;
	}
	
	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
		
		if (act.equals("Stop")) {
			paused = true;
		}
		
		if (act.equals("Play")) {
			paused = false;
			fileChanged = true;
			timerOn();
		}
		
		if (act.equals("First")) {
			curFile = 0;
			paused = true;
			fileChanged = true;
			timerOn();
		}
		
		if (act.equals("Last")) {
			curFile = fileCt-1;
			paused = true;
			fileChanged = true;
			timerOn();
		}
		
		if (act.equals("Prev")) {
			if (curFile > 0) { curFile--; }
			fileChanged = true;
			timerOn();
		}
		
		if (act.equals("Next")) {
			if (curFile < fileCt-1) { curFile++; }
			fileChanged = true;
			timerOn();
		}
		
		if (act.equals("Open")) {
			ParMSpindle.setPaused();
			openRender();
		}
		
		if (act.equals("Done")) {
			timerOff();
			this.setVisible(false);
		}
		
		if (act.equals("WriteJPEGs")) {
			toJPEG = jpegB.isSelected();
			if (toJPEG) { prepForJPEGWriting(); }
		}
		
		if (act.equals("FilRenderOff")) {
			Env.filRenderOff = filROffB.isSelected();
			System.out.println ("filRenderOff is " + Env.filRenderOff);
		}
		
		if (act.equals("Rotate")) {
			rotate = rotateB.isSelected();
			rotateAng = getRotAngle();
		}
		
		if (act.equals("Skip")) {
			skip = skipB.isSelected();
		}

		if (act.equals("Goto")) {
			gotoFile = true;
			gotoB.setSelected(false);
			fileChanged = true;
		}
	}
	
	public void makeRenderPanels () {
		renderPanel = new JPanel();
		renderPanel.setBackground(Env.controlBackColor);
		
		renderPanel1 = new JPanel();
		renderPanel1.setBackground(Env.controlBackColor);
		
		renderPanel2 = new JPanel();
		renderPanel2.setBackground(Env.controlBackColor);
		
		renderPanel3 = new JPanel();
		renderPanel3.setBackground(Env.controlBackColor);
		
		renderPanel4 = new JPanel();
		renderPanel4.setBackground(Env.controlBackColor);
		
		renderPanel5 = new JPanel();
		renderPanel5.setBackground(Env.controlBackColor);
		
		jpegB = new JRadioButton ("Write JPEGs to");
		jpegB.setActionCommand("WriteJPEGs");
		jpegB.addActionListener(this);
		jpegB.setBackground(Env.controlBackColor);
		jpegB.setForeground(Env.controlForeColor);
		jpegB.setSelected(toJPEG);
		jpegLoc = new JLabel ("                    ");
		jpegLoc.setBackground(Env.controlBackColor);
		jpegLoc.setForeground(Env.controlForeColor);
		
		rotateB = new JRadioButton ("Rotate");
		rotateB.setActionCommand("Rotate");
		rotateB.addActionListener(this);
		rotateB.setBackground(Env.controlBackColor);
		rotateB.setForeground(Env.controlForeColor);
		rotateB.setSelected(rotate);
		rotateField = new JTextField ("1",10);
		rotateField.addActionListener(this);
		rotateField.setActionCommand("Rotate");
		rotateField.setBackground(Env.controlBackColor);
		rotateField.setForeground(Env.controlForeColor);
		rotateField.setBorder(BorderFactory.createEmptyBorder());
		rotateField.setEditable(true);
		
		filROffB = new JRadioButton ("Filament Render OFF");
		filROffB.setActionCommand("FilRenderOff");
		filROffB.addActionListener(this);
		filROffB.setBackground(Env.controlBackColor);
		filROffB.setForeground(Env.controlForeColor);
		filROffB.setSelected(Env.filRenderOff);
		
		skipB = new JRadioButton ("Skip every");
		skipB.setActionCommand("Skip");
		skipB.addActionListener(this);
		skipB.setBackground(Env.controlBackColor);
		skipB.setForeground(Env.controlForeColor);
		skipB.setSelected(skip);
		skipField = new JTextField ("1",4);
		skipField.setBackground(Env.controlBackColor);
		skipField.setForeground(Env.controlForeColor);
		skipField.setBorder(BorderFactory.createEmptyBorder());
		skipField.setEditable(true);
		
		gotoB = new JRadioButton("Goto file");
		gotoB.setActionCommand("Goto");
		gotoB.addActionListener(this);
		gotoB.setBackground(Env.controlBackColor);
		gotoB.setForeground(Env.controlForeColor);
		gotoField = new JTextField ("1",4);
		gotoField.setBackground(Env.controlBackColor);
		gotoField.setForeground(Env.controlForeColor);
		gotoField.setBorder(BorderFactory.createEmptyBorder());
		gotoField.setEditable(true);
	
		renderPanel1.setLayout(new FlowLayout(FlowLayout.LEFT));
		renderPanel1.add(jpegB);
		renderPanel1.add(jpegLoc);
		
		renderPanel2.setLayout(new FlowLayout(FlowLayout.LEFT));
		renderPanel2.add(filROffB);
		
		renderPanel3.setLayout(new FlowLayout(FlowLayout.LEFT));
		renderPanel3.add(rotateB);
		renderPanel3.add(rotateField);
		
		renderPanel4.setLayout(new FlowLayout(FlowLayout.LEFT));
		renderPanel4.add(skipB);
		renderPanel4.add(skipField);
		
		renderPanel5.setLayout(new FlowLayout(FlowLayout.LEFT));
		renderPanel5.add(gotoB);
		renderPanel5.add(gotoField);
		
		renderPanel.setLayout(new GridLayout(6,1));
		renderPanel.add(infoPanel);
		renderPanel.add(renderPanel1);
		renderPanel.add(renderPanel2);
		renderPanel.add(renderPanel3);
		renderPanel.add(renderPanel4);
		renderPanel.add(renderPanel5);
	}
	
	public void makeInfoPanel () {
		infoPanel = new JPanel();
		infoPanel.setBackground(Env.controlBackColor);
		
		JLabel curRenderLabel = new JLabel("Current file: ");
		curRenderLabel.setBackground(Env.controlBackColor);
		curRenderLabel.setForeground(Env.controlForeColor);
		
		curRenderFrac = new JLabel("");
		curRenderFrac.setBackground(Env.controlBackColor);
		curRenderFrac.setForeground(Env.controlForeColor);
		
		curRenderName =  new JLabel("");
		curRenderName.setBackground(Env.controlBackColor);
		curRenderName.setForeground(Env.controlForeColor);
		
		infoPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		infoPanel.add(curRenderLabel);
		infoPanel.add(curRenderFrac);
		infoPanel.add(curRenderName);
	}
		
	
	public void makeButtonsPanel () {
		buttonsPanel = new JPanel();
		buttonsPanel.setBackground(Env.controlBackColor);
	
		firstB = new BareButton ("<--");
		firstB.setActionCommand("First");
		firstB.setToolTipText("Goto first frame");
		firstB.addActionListener(this);
		firstB.setEnabled(true);
		
		prevB = new BareButton ("<<");
		prevB.setActionCommand("Prev");
		prevB.setToolTipText("Previous frame");
		prevB.addActionListener(this);
		prevB.setEnabled(true);
		
		stopB = new BareButton ("");
		stopB.setActionCommand("Stop");
		stopB.setToolTipText("Stop");
		stopB.addActionListener(this);
		stopB.setEnabled(true);
		
		playB = new BareButton (">");
		playB.setActionCommand("Play");
		playB.setToolTipText("Play");
		playB.addActionListener(this);
		playB.setEnabled(true);
		
		nextB = new BareButton (">>");
		nextB.setActionCommand("Next");
		nextB.setToolTipText("Next frame");
		nextB.addActionListener(this);
		nextB.setEnabled(true);
		
		lastB = new BareButton ("-->");
		lastB.setActionCommand("Last");
		lastB.setToolTipText("Goto last frame");
		lastB.addActionListener(this);
		lastB.setEnabled(true);
		
		JTextField blank = new JTextField("  ");
		blank.setBackground(Env.controlBackColor);
		blank.setBorder(BorderFactory.createEmptyBorder());
		
		openB = new BareButton ("O");
		openB.setActionCommand("Open");
		openB.setToolTipText("Open render");
		openB.addActionListener(this);
		openB.setEnabled(true);
		
		doneB = new BareButton ("X");
		doneB.setActionCommand("Done");
		doneB.setToolTipText("End render");
		doneB.addActionListener(this);
		doneB.setEnabled(true);
		
		buttonsPanel.setLayout(new GridLayout(1,9));
		buttonsPanel.add(firstB);
		buttonsPanel.add(prevB);
		buttonsPanel.add(stopB);
		buttonsPanel.add(playB);
		buttonsPanel.add(nextB);
		buttonsPanel.add(lastB);
		buttonsPanel.add(blank);
		buttonsPanel.add(openB);
		buttonsPanel.add(doneB);
	}
	
	public void prepForJPEGWriting () {
		toJPEGPath = null;
		toJPEGPath = FileOps.getDirectoryToSave("Create folder for JPEG files ...", this);
		if (toJPEGPath == null) { 
			toJPEGName = null;
			jpegLoc.setText("");
		} else {
			int lastbitIndex = toJPEGPath.lastIndexOf(File.separator);
			toJPEGName = toJPEGPath.substring(lastbitIndex+1);
			ParMSpindle_Graphics.theCanvas.capturePath = toJPEGPath;
			ParMSpindle_Graphics.theCanvas.captureName = toJPEGName;
			jpegLoc.setText(toJPEGPath);
		}
	}
	
}
	
